# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


from LIB import *
import matplotlib


# uncomment the following to use wx rather than wxagg
# matplotlib.use('WX')
# from matplotlib.backends.backend_wx import FigureCanvasWx as FigureCanvas
from matplotlib.figure import Figure
matplotlib.use('WXAgg')
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas




class MiniEditeur:
    def __init__(self, parent, objet = None, mode = 0):
        self.parent = parent
        self.commande = parent.commande
        self.objet = None
        self.init(objet, mode)

        
    def init(self, objet = None, mode = 0):
        """Rinitialise l'diteur.
        A utiliser en particulier avant de travailler sur un objet, pour s'assurer qu'il n'est pas en train d'tre dit."""
        self.retablir()
        self.caracteres = ""
        if objet is None:
            self.dernier_label = None
        else:
            self.dernier_label = objet.label()
        self.objet = objet
        self.mode = mode # mode = 0: caracteres alphanumeriques + "_", mode = 1: tout caractere ASCII.
        if objet:
            objet.label("...")
    
    def retablir(self):
        if self.objet is not None:
            self.objet.label(self.dernier_label)

    def key(self, key):
        if key == wx.WXK_BACK:
            self.caracteres = self.caracteres[:-1]
            self.objet.label(self.caracteres + "...")
            
        elif key == wx.WXK_ESCAPE:
            self.retablir()
            self.objet = None
          
        elif key == wx.WXK_RETURN:
            if self.caracteres:
                objet = self.objet
                caracteres = self.caracteres
                mode = self.mode
                dernier_label = self.dernier_label
                try: # si possible, on change le nom de l'objet ; sinon, on change son label.
                    nom = objet.nom()
                    if not re.match("[A-Za-z_][A-Za-z0-9_]*$", self.caracteres) or self.caracteres == nom:
                        self.mode = 1
                    if self.mode:
                        self.commande.executer("%s.label(%s)" %(nom, repr(self.caracteres)))
                    else:
                        self.commande.executer("%s.renomme(%s, legende = %s)" %(nom, repr(self.caracteres), param.NOM))
                        
                except: # on reste en mode edition
                    objet.label(caracteres + "...")
                    self.objet = objet
                    self.caracteres = caracteres
                    self.mode = mode
                    self.dernier_label = dernier_label
                    raise
            else: # si aucun nom n'est rentre, [ENTREE] est un equivalent de [ECHAP].
                self.retablir()
                self.objet = None

        elif key == 10: # Ctrl + Entree (a tester sous Linux !)
            self.caracteres += "\n"
            self.objet.label(self.caracteres + "...")            
        
        elif key >= 32 and key <= 255:           
            key = chr(key) # a-z, A-Z, 0-9 ou _
            self.caracteres += key
            self.objet.label(self.caracteres + "...")
    
        
    def __nonzero__(self):
        return self.objet is not None






class Moteur_graphique:
    def __init__(self, parent):
        self.parent = parent
        self.axes = self.parent.axes
        self.__empty_plot__ = self.axes.plot([0,1], [1,0])[0]
        self.__empty_fill__ = self.axes.fill([0], [0])[0]
        self.__empty_text__ = self.axes.text(0, 0, '')
        self.mise_en_cache_axes = [[],[], [], []] # plot, fill, text, artist
        
    def fast_plot(self, x, y, couleur = None, **kwargs):
        ligne = copy.copy(self.__empty_plot__)
        ligne._x = x
        ligne._y = y
        if couleur:
            ligne._color = couleur
        if kwargs:
            ligne.set(**kwargs)
        self.axes.add_artist(ligne)
        return ligne
    
    def fast_plots(self, *liste, **kwargs):
        for elt in liste:
            return [self.fast_plot(*elt, **kwargs) for elt in liste]

    def fast_fill(self, x, y, couleur = None, **kwargs):
        polygone = copy.copy(self.__empty_fill__)
        polygone.xy = zip(x, y)
        if couleur:
            polygone._facecolor = couleur
        if kwargs:
            polygone.set(**kwargs)
        self.axes.add_artist(polygone)
        return polygone
    
    def fast_fills(self, *liste, **kwargs):
        for elt in liste:
            return [self.fast_fill(*elt, **kwargs) for elt in liste]

    def fast_text(self, x, y, txt, **kwargs):
        texte = copy.copy(self.__empty_text__)
        texte._x = x
        texte._y = y
        texte._text = txt
        if kwargs:
            texte.set(**kwargs)
        self.axes.add_artist(texte)
        return texte
    
    def fast_texts(self, *liste):
        for elt in liste:
            return [self.fast_text(*elt, **kwargs) for elt in liste]

    def fast_clear(self, axes = True):
        self.axes.lines = []
        self.axes.patches = []
        self.axes.texts = []
        self.axes.tables = []
        self.axes.artists = []
        self.axes.images = []
        self.axes.legend_ = None
        self.axes.collections = []  # collection.Collection instances
        self.axes.axison = False
        if axes and self.parent.feuille_actuelle():
            self.draw_axes()

    def fast_draw(self):
        # bug dans matplotlib ?? Je suis oblige de faire marcher le zorder manuellement.
        self.axes.artists.sort(key=lambda x:getattr(x,"zorder"))
        # contournation d'un bug de matplotlib si un texte est tres excentre de la zone d'affichage.
        xmin, xmax, ymin, ymax = self.parent.fen()
        dx = xmax - xmin
        xsup = xmax + 100*dx
        xinf = xmin - 100*dx
        dy = ymax - ymin
        ysup = ymax + 100*dy
        yinf = ymin - 100*dy
        for artiste in self.axes.artists: # si le texte est trop excentre, on ne l'affiche pas (texte._visible = False)
            if isinstance(artiste, self.__empty_text__.__class__):
                artiste._visible = xinf < artiste._x < xsup and yinf < artiste._y < ysup
        self.axes.viewLim.intervalx().set_bounds(xmin, xmax)
        self.axes.viewLim.intervaly().set_bounds(ymin, ymax)
        self.parent.draw()
    
    
    def en_cache(self):
        return [self.axes.lines[:], self.axes.patches[:], self.axes.texts[:], self.axes.artists[:]]
    
    def restaurer(self, liste):
        self.axes.lines = liste[0][:]
        self.axes.patches = liste[1][:]
        self.axes.texts = liste[2][:]
        self.axes.artists = liste[3][:]
    
    
    
    def draw_axes(self):
        if self.parent.rafraichir_axes:
            self.parent.dessine_axes()
            self.parent.rafraichir_axes = False
            self.mise_en_cache_axes = self.en_cache()
        else:
            self.restaurer(self.mise_en_cache_axes)
            
            
    def ajouter(self, liste):
        self.axes.artists += liste
    
    def supprimer(self, liste):
        self.axes.artists = [artiste for artiste in self.axes.artists if artiste not in liste]
















    





class Affichage(FigureCanvas):
    def __init__(self, parent, fixe = False):
        "Si fixe = True, l'utilisateur ne peut pas zoomer ou recadrer la fenetre d'affichage avec la souris."
        self.figure = Figure(frameon=True, facecolor="w")
        FigureCanvas.__init__(self, parent, -1, self.figure)
        self.axes = self.figure.add_axes([0, 0, 1, 1], frameon=False)
        self.axes.set_xticks([])
        self.axes.set_yticks([])
        self.parent = parent
        self.commande = self.parent.commande
        self.debut_zoom = None # utilise pour zoomer avec Ctrl + Clic gauche (position initiale)
        self.debut_shift = None # utilise pour translater le contenu de la fenetre (position initiale)
        self.select = None # objet couramment selectionne
        self.select_memoire = None # objet devant etre prochainement selectionne (en cas de "litige" entre 2 objets)
        self.fixe = fixe
        self.interaction = None # fonction a lancer au prochain clic de souris (au lieu des actions par defaut)
        self.editeur = MiniEditeur(self) # edite les noms et etiquettes des points, textes, etc.
        
        self.geler_affichage = False # si on met la valeur a True, self.actualiser() ne fait plus rien.
        # cela permet d'effectuer un certain nombre d'actions rapidement et de n'actualiser qu'a la fin.
        # En particulier, cela sert pour charger une figure depuis un fichier .geo
        
        self.regenerer_liste = True # utilis pour savoir si il faut rgnrer la liste des objets de la feuille.
        
        self.sel = [] # liste ordonne (pour la dtection) des objets de la feuille actuelle.
        # elle est remise a jour quand self.commande.compteur a chang.
 
        self.graph = Moteur_graphique(self)
        self.parametres = ["taille", "style_point", "gradu", "affiche_axes", "affiche_quadrillage", "affiche_fleches", "repere", "resolution", "origine_axes", "utiliser_repere", "quadrillages", "liste_axes", "orthonorme"]
        self.initialiser()
        
        self.Bind(wx.EVT_MOUSEWHEEL, self.OnWheel)
        self.Bind(wx.EVT_MOTION, self.OnMotion)
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
        self.Bind(wx.EVT_RIGHT_UP, self.OnRightUp)
        self.Bind(wx.EVT_SIZE, self.repaint)
        self.Bind(wx.EVT_CHAR, self.OnChar)
        #self.Bind(wx.EVT_LEAVE_WINDOW, self.OnLeave)
        #self.Bind(wx.EVT_PAINT, self.OnPaint)
    
    
    
    def initialiser(self):
        # actions a effectuer avant de reexecuter l'historique
        for parametre in self.parametres:
            setattr(self, parametre, getattr(param, parametre))
        self.largeur_point = param.largeur_point[0]
        self.rafraichir_axes = True
        
    def message(self, txt, lieu = 0):
        self.parent.parent.parent.message(txt, lieu) # cf. geometrie.py
        
        
    def message_temporaire(txt = None): # A FINIR
        self.message_temp = txt
        if txt is None:
            self.message(self.message_def)
        else:
            self.message(txt)
    
    def exporter(self, nom, dpi = None):
        dpi = dpi or param.dpi_export 
        self.largeur_point = param.largeur_point[1] # pour l'impression, mieux vaut des points fins
        self.repaint()
        self.figure.savefig(nom, dpi = dpi)
        self.largeur_point = param.largeur_point[0] # mais a l'ecran, des points plus epais se voient mieux
        self.repaint()
        
    def feuille_actuelle(self):
        return self.parent.actuelle

##################################
# Les fonctions suivantes assurent la conversion pixel -> coordonnees
##################################
# en multipliant m par coeff(0), on convertit un ecart en abcisses de m pixels en coordonnees.
# en multipliant n par coeff(1), on convertit un ecart en ordonnees de n pixels en coordonnees.
# Les fonctions qui suivent permettent la conversion d'un couple de coordonnees en pixels, et reciproquement.
# Si le mode est fixe a 1, le pixel (0,0) sera le coin inferieur gauche (plus intuitif).
# Si le mode est fixe a -1, le pixel (0,0) sera le coin superieur gauche (convention, respectee par WxPython).
        
    

    def pas(self):
        return (self.fen(1)-self.fen(0))/(self.resolution)

    def coeff(self, i):
        return (self.fen(1+2*i) - self.fen(2*i))/self.GetSizeTuple()[i]
    
    
    def Xpix2coo(self, x):
        return array(x)*self.coeff(0) + self.fen(0)
    
    def Ypix2coo(self, y, mode = 1):
        return mode*array(y)*self.coeff(1) + where(mode == 1, self.fen(2), self.fen(3))
    
    def XYpix2coo(self, couple, mode = 1):
        return (self.Xpix2coo(couple[0]), self.Ypix2coo(couple[1], mode))
    
    def Xcoo2pix(self, x):
        return (array(x) - self.fen(0))/(self.coeff(0))
    
    def Ycoo2pix(self, y, mode = 1):
        return mode*(array(y) - where(mode == 1, self.fen(2), self.fen(3)))/self.coeff(1)

    def XYcoo2pix(self, couple, mode = 1):
        return (self.Xcoo2pix(couple[0]), self.Ycoo2pix(couple[1], mode))
    
    
#    Gestion de l'affichage
#################################

    
    def repaint(self, event = None): # Utilise pour l'evt RESIZE
        if event: event.Skip()
        if self.orthonorme:
            self.orthonormer(rafraichir = False)
        self.rafraichir_axes = True
        self.parent.affiche()
    

            

                
                
        
    def ecran_blanc(self): # affiche un ecran blanc essentiellement quand il n'y a pas de feuille
        self.clear(axes = False)
        self.draw()
        #self.ClearBackground()
        
        
    def affiche(self):
        if self.feuille_actuelle() == None:
            self.ecran_blanc()
            return

        self.feuille_actuelle().affiche()
        
        self.actualiser()


    def saturation(self, i):
        return self.coeff(i)/self.gradu[i]
    



    
    def afficher_repere(self, event = None):
        self.affiche_axes = not self.affiche_axes
        self.rafraichir_axes = True
        self.parent.affiche()

    def afficher_quadrillage(self, event = None):
        self.affiche_quadrillage = not self.affiche_quadrillage
        self.rafraichir_axes = True
        self.parent.affiche()
        
        
        
#   Alias
######################

    def clear(self, axes = True):
#         d_.deb()
        self.graph.fast_clear(axes)
#         d_.fin("Efface")

    effacer = clear
    
    def plot(self, x = array([]), y = array([]), couleur = "k", **kw):
        return self.graph.fast_plot(x, y, couleur, **kw)
        
    def fill(self, x = array([]), y = array([]), couleur = None, **kw):
        return self.graph.fast_fill(x, y, couleur, **kw)

    def text(self, x = 0, y = 0, texte = "", **kw):
        return self.graph.fast_text(x, y, texte, **kw)
    
    def actualiser(self):
        if not self.geler_affichage:
            self.graph.fast_draw()
    
#    Axes et quadrillage
#################################


    def dessine_axe(self, num = 0, nbr_axes = 1):
        "Dessine l'axe des abscisses (num = 0) ou des ordonnees (num = 1)."

        #un = nbr_axes == 1              # un seul axe
        #x = num == 0; y = not x         # axe des abscisses / des ordonnees
        #rep = self.utiliser_repere      # repere ou non
        vect = self.utiliser_repere and self.repere[1 + num] in string.ascii_lowercase # repere defini ou non par des vecteurs
        correspondance = {"b" : "bottom", "c": "center", "l": "left", "r": "right", "t": "top"}
        
        def f(a, b):
            return iff(num == 0, a, b)

        signe = f(-1, 1)
        origine = self.origine_axes[1 - num]
        
        # On cree l'axe proprement dit:       
        self.plot(f([self.fen(0), self.fen(1)], [origine, origine]), f([origine, origine], [self.fen(2), self.fen(3)]), "k")
        
        if self.affiche_fleches:
            # On cree la fleche au bout de l'axe:
            x, y = self.XYcoo2pix((f(self.fen(1), origine), f(origine, self.fen(3))))
            dx, dy = .5*self.taille[">"]*array([f(sqrt(3), 1), f(1, sqrt(3))])
            self.plot(self.Xpix2coo([x - dx, x, x + signe*dx]), self.Ypix2coo([y - dy, y, y - signe*dy]), "k")

            
        # FORMAT (_origine_ et _graduation_) : [texte, x, y, alignement horizontal, alignement vertical]
        # _origine_ : infos sur l'origine de l'axe 
        # _graduation_ : infos sur le 2eme point de l'axe

        ox, oy = self.origine_axes
        ux, uy = self.gradu
        kx, ky = self.coeff(0), self.coeff(1)
        
        if vect:
            x, y = self.XYcoo2pix((f(ox + ux, ox), f(oy, oy + uy)))
            self.plot(self.Xpix2coo([x - dx, x, x + signe*dx]), self.Ypix2coo([y - dy, y, y - signe*dy]), "k")
        
        
        if nbr_axes == 1:                   # un seul axe
            if num:                         # axe des ordonnees
                if self.utiliser_repere:    # utiliser un repere (meme origine sur chaque axe...)
                    _origine_ = [self.repere[0], ox - 8*kx, oy, "r", "c"]
                    if vect:                # reperage par des vecteurs
                        _graduation_ = ["$\\tt{\\vec{" + self.repere[2] + "}}$", ox - 8*kx, oy + .5*uy, "r", "c"]
                    else:                   # reperage par des points
                        _graduation_ = [self.repere[2], ox - 8*kx, oy + uy, "r", "c"]
                else:                       # pas un repere
                        _origine_ = [str(oy), ox - 8*kx, oy, "r", "c"]
                        _graduation_ = [str(oy + uy), ox - 8*kx, oy + uy, "r", "c"]
            else:                           # axe des abscisses
                if self.utiliser_repere:    # utiliser un repere (meme origine sur chaque axe...)
                    _origine_ = [self.repere[0], ox, oy - 8*ky, "c", "t"]
                    if vect:                # reperage par des vecteurs
                        _graduation_ = ["$\\tt{\\vec{" + self.repere[1] + "}}$", ox + .5*ux, oy - 8*ky, "c", "t"]
                    else:                   # reperage par des points
                        _graduation_ = [self.repere[1], ox + ux, oy - 8*ky, "c", "t"]
                else:                       # pas un repere
                        _origine_ = [str(ox), ox, oy - 8*ky, "c", "t"]
                        _graduation_ = [str(ox + ux), ox + ux, oy - 8*ky, "c", "t"]
        else:                               # 2 axes
            if num:                         # axe des ordonnees
                if self.utiliser_repere:
                    _origine_ = None
                    if vect:                # reperage par des vecteurs
                        _graduation_ = ["$\\tt{\\vec{" + self.repere[2] + "}}$", ox - 8*kx, oy + .5*uy, "r", "c"]
                    else:                   # reperage par des points
                        _graduation_ = [self.repere[2], ox - 8*kx, oy + uy, "r", "c"]
                else:                       # pas un repere
                        _origine_ = [str(oy), ox - 8*kx, oy + 3*ky, "r", "b"]
                        _graduation_ = [str(oy + uy), ox - 8*kx, oy + uy, "r", "b"]
            else:                           # axe des abscisses
                if self.utiliser_repere:
                    _origine_ = [self.repere[0], ox - 8*kx, oy - 8*ky, "r", "t"]
                    if vect:                # reperage par des vecteurs
                        _graduation_ = ["$\\tt{\\vec{" + self.repere[1] + "}}$", ox + .5*ux, oy - 8*ky, "c", "t"]
                    else:                   # reperage par des points
                        _graduation_ = [self.repere[1], ox + ux, oy - 8*ky, "c", "t"]
                else:                       # pas un repere
                        _origine_ = [str(ox), ox + 3*kx, oy - 8*ky, "l", "t"]
                        _graduation_ = [str(ox + ux), ox + ux, oy - 8*ky, "l", "t"]
        
        if _origine_ and self.gradu[num]:
            self.graph.fast_text(_origine_[1], _origine_[2], _origine_[0], horizontalalignment = correspondance[_origine_[3]], verticalalignment = correspondance[_origine_[4]])
            
        if _graduation_ and self.gradu[num] and self.saturation(num) < param.saturation:
            self.graph.fast_text(_graduation_[1], _graduation_[2], _graduation_[0], horizontalalignment = correspondance[_graduation_[3]], verticalalignment = correspondance[_graduation_[4]])
        



        
        
        
        
    def quadriller(self, hauteur = 3, pas = (None, None), style = None, epaisseur = 1, couleur = "k"):
        """Cree des graduations ou un quadrillage (si hauteur = None).
        Si pas[0] = 0 (respectivement, pas[1] = 0), les graduations seront
        uniquement horizontales (respectivement verticales)."""
        
        args = []
        origine = self.origine_axes
        pas = list(pas)
        # Je n'utilise pas des listes directement, mais des tuples.
        # Ceci afin de n'avoir jamais la MEME liste.
        
        if hauteur == None: # quadrillage
            xmin, xmax, ymin, ymax = self.fen()
            _min = ymin, xmin ; _max = ymax, xmax
            if not style:
                style = ":"
        else:               # graduations
            _min = [origine[i] - hauteur*self.coeff(i) for i in (1, 0)] # ymin, xmin
            _max = [origine[i] + hauteur*self.coeff(i) for i in (1, 0)] # ymax, xmax
            if not style:
                style = "-"
            
        #type_ligne = couleur + style
        
        for n in (0, 1):
            if pas[n] == None:  pas[n] = self.gradu[n]
            pas[n] = 1.*abs(pas[n])

            # graduations sur l'axe:
            if pas[n] and self.coeff(n)/pas[n] < param.saturation:
               
                maxi = self.fen(2*n + 1)
                if self.affiche_axes and (n in self.liste_axes) and hauteur: # les graduations qui chevauchent les fleches, c'est pas joli
                    maxi -= .7*self.taille[">"]*sqrt(3)*self.coeff(n)
                    
                valeurs = concatenate((arange(origine[n], self.fen(2*n), -pas[n]), arange(origine[n] + pas[n], maxi, pas[n])))
                for val in valeurs:
                    # si on affiche le quadrillage ou les graduations sur les axes, ca fait pas net:
                    
                    if val <> origine[n] or not self.affiche_axes or not (1 - n in self.liste_axes):
                        if n:   self.plot(array([_min[n], _max[n]]), array([val, val]), color = couleur, linestyle = style, linewidth = epaisseur)
                        else:   self.plot(array([val, val]), array([_min[n], _max[n]]), color = couleur, linestyle = style, linewidth = epaisseur)
                        
        
    



    
    def dessine_axes(self):
        #if self.rafraichir_axes:
            # on efface tout
        #    self.axes.lines = []
        #    self.axes.texts = []
        #    self.axes.patches = []

            if self.affiche_axes:
                for axe in self.liste_axes:
                    self.dessine_axe(axe, len(self.liste_axes))
                
            if self.affiche_quadrillage:   # quadrillage
                for quadrillage in self.quadrillages:
                    self.quadriller(None, *quadrillage)
            elif self.affiche_axes:
                self.quadriller()   # simples graduations
                
            self.axes_lignes = self.axes.lines[:]
            self.axes_textes = self.axes.texts[:]
            # self.axes_lignes doit etre une copie, distincte, de self.axes.lines.
            # Pour comprendre, essayez ceci : m = [1]; n = m; print n is m; n = m[:]; print n is m.
            #self.rafraichir_axes = False

        #else:     
        #    self.axes.lines = self.axes_lignes[:] # obtention de copies distinctes
        #    self.axes.texts = self.axes_textes[:]
        #    self.axes.patches = []





#      Gestion du zoom, etc...
########################################

        
    def fen(self, i = None):
        "Affiche ou change les limites de la fenetre d'affichage (commande de bas niveau)"
        if type(i) == int:      return 1.*self.feuille_actuelle().fenetre()[i]
        if i:
            self.feuille_actuelle().__fenetre__ = tuple(i)
            self.rafraichir_axes = True
        return 1.*array(self.feuille_actuelle().fenetre())

    def set_fenetre(self, xmin, xmax, ymin, ymax):
        if xmin == xmax or ymin == ymax:
            self.message("Reglage incorrect de la fenetre.")
            return
        self.fen((min(xmin, xmax), max(xmin, xmax), min(ymin, ymax), max(ymin, ymax)))
        if self.orthonorme:
            self.orthonormer(rafraichir = False)
        self.message("Fenetre modifiee.")
        self.feuille_actuelle().refresh(Droite, Demidroite)   # rafraichir la fenetre, et redessiner les droites.
        #self.parent.affiche()
        
    def synchroniser_fenetre(self):
        """Detecte la fenetre d'affichage et l'enregistre.
        Ce peut-etre utile si l'on utilise une commande de haut niveau de matplolib,
        qui calcule automatiquement la meilleure fenetre d'affichage."""
        self.fen(self.axes.viewLim.intervalx().get_bounds() + self.axes.viewLim.intervaly().get_bounds())
        
    def zoomer(self, coeff):
        xmin, xmax, ymin, ymax = self.fen()
        coeff = 1.*coeff # permet une division "exacte".
        x, y, rx, ry = (xmin+xmax)/2., (ymin+ymax)/2., (xmax-xmin)/2., (ymax-ymin)/2.
        self.set_fenetre(x - rx/coeff, x + rx/coeff, y - ry/coeff, y + ry/coeff)
        
    def zoom_in(self, event = None):
        self.commande.executer("zoomer(" + str(param.zoom_in) + ")")
    
    def zoom_out(self, event = None):
        self.commande.executer("zoomer(" + str(param.zoom_out) + ")")
    
    def repere_orthonormal(self, event = None):
        self.commande.executer("orthonormer()")
        
    def zoom_automatique(self, event = None):
        self.commande.executer("zoom_auto()")
        
    
    def zoom_auto(self, rafraichir = True):
        a = self.fen()
        compteur = 0
        condition = True
        while condition:
            compteur += 1
            self._zoom_auto(rafraichir) # a cause du texte dont la taille est independante de l'echelle, les calculs sont fausss.
            # Mais en ritrant le procd, l'affichage converge rapidement (A OPTIMISER ?).
            b = a
            a = self.fen()
            erreur_x = abs(a[0] - b[0]) + abs(a[1] - b[1])
            erreur_y = abs(a[2] - b[2]) + abs(a[3] - b[3])
            #dx = a[1] - a[0] # xmax - xmin
            #dy = a[3] - a[2] # ymax - ymin
            condition = erreur_x/self.coeff(0) > 1 or erreur_y/self.coeff(1) > 1 # l'erreur doit tre infrieure  1 pixel
            if compteur > 25:
                print "Echec du zoom automatique."
                break


    def _zoom_auto(self, rafraichir = False):
        actuelle = self.feuille_actuelle()
        if actuelle:
            xxyy = zip(*(obj.espace_vital() for obj in actuelle.objets(False) if obj.espace_vital() is not None))
            if xxyy:
                xmin = min(xxyy[0])
                xmax = max(xxyy[1])
                ymin = min(xxyy[2])
                ymax = max(xxyy[3])
                dx = (xmax-xmin)
                dy = (ymax-ymin)
                if dx < param.tolerance: # des valeurs trop proches pour xmin et xmax risquent de faire planter l'affichage.
                    r = (self.fen(1) - self.fen(0))/2
                    xmin -= r
                    xmax += r
                else:
                    xmin -= .05*dx
                    xmax += .05*dx 
        
                if dy < param.tolerance: # idem pour ymin et ymax
                    r = abs(self.fen(3) - self.fen(2))/2
                    ymin -= r
                    ymax += r
                else:
                    ymin -= .05*dy
                    ymax += .05*dy 
                
                if rafraichir:
                    self.set_fenetre(xmin, xmax, ymin, ymax)
                else:
                    self.fen((xmin, xmax, ymin, ymax))


    
    def orthonormer(self, mode = 1, rafraichir = True):
        """
        mode 0 : on orthonormalise le repere en restreignant la vue.
        mode 1 : on orthonormalise le repere en elargissant la vue."""
        if mode:
            xcoeff = where(self.coeff(0) < self.coeff(1), self.coeff(1)/self.coeff(0), 1)
            ycoeff = where(self.coeff(0) < self.coeff(1), 1, self.coeff(0)/self.coeff(1))
        else:
            xcoeff = where(self.coeff(0) > self.coeff(1), self.coeff(1)/self.coeff(0), 1)
            ycoeff = where(self.coeff(0) > self.coeff(1), 1, self.coeff(0)/self.coeff(1))
        xmin, xmax, ymin, ymax = self.fen()
        x, y, rx, ry = (xmin+xmax)/2., (ymin+ymax)/2., (xmax-xmin)/2., (ymax-ymin)/2.
        if rafraichir:
            self.set_fenetre(x - xcoeff*rx, x + xcoeff*rx, y - ycoeff*ry, y + ycoeff*ry)
        else:
            self.fen((x - xcoeff*rx, x + xcoeff*rx, y - ycoeff*ry, y + ycoeff*ry))



#    Gestion des evenements (essentiellement la souris).
###############################

    
    
    
    def interagir(self, fonction, aide = None):
        if self.feuille_actuelle() <> None or not fonction:
            self.interaction = fonction
            if fonction:
                self.SetCursor(wx.StockCursor(wx.CURSOR_QUESTION_ARROW))
            else:
                self.SetCursor(wx.StockCursor(wx.CURSOR_DEFAULT))
            if aide is not None:
                self.message(aide)
    
    
    def signal(self, event = None):
        if self.interaction:
            objet = self.interaction(selection = self.select, autres = self.selections, position = self.XYpix2coo(event.GetPositionTuple(), -1))
            self.detecter(event.GetPositionTuple())
            
    
    def deplacable(self, obj): # indique si un objet est deplacable
        return isinstance(obj, (Point, Glisseur_generique, Texte))
    
    def pointable(self, obj): # indique si un objet possede des coordonnees
        return isinstance(obj, (Point_generique, Texte))

    
    def infos(self):
        msg = self.select.nom_complet()
        if self.pointable(self.select):
            msg += " de coordonnees "+ str(tuple(self.select.coordonnees()))
        self.message(msg)



    def detecter(self, position = None):
        """Detecte les objets a proximite de la position indiquee.
        Typiquement, on utilise self.detecter(event.GetPositionTuple())."""
        
        self.debut_zoom = None
        actuelle = self.feuille_actuelle() # feuille courante
            
        if actuelle and not self.geler_affichage: 
            if position is None:
                position = self.ScreenToClient(wx.GetMousePosition())
            if param.affiche_coordonnees: # affichage des coordonnees dans la barre d'etat
                self.message(str(self.XYpix2coo(position, -1)), 1)
            x, y = position
            # on place les objets 'modifiables' en premier (points libres, glisseurs, textes)
            if self.regenerer_liste:
                self.regenerer_liste = False
                #self.sel = [obj for obj in actuelle.objets(False) if obj.modifiable]
                # les autres objets sont tris selon l'attribut niveau.
                #sel2 = [obj for obj in actuelle.objets(False) if not obj.modifiable]
                #sel2.sort(key=lambda x:getattr(x,"niveau"))
                #self.sel += sel2
                self.sel = [obj for obj in actuelle.objets(False)]
                self.sel.sort(key=lambda x:x.style("niveau"), reverse=True)              
            # liste des objets pres du pointeur de la souris :
            self.selections = []
            for obj in self.sel:
                try:
                    if obj.distance_inf(x, y, param.precision_selection):
                        self.selections.append(obj)
                except:
                    print_error()
                    self.message("Erreur: les coordonnees de %s sont incalculables." %obj.nom())
            proximite = len(self.selections)
            if proximite:
                self.select = self.selections[0]
                if self.select_memoire and self.select_memoire in self.selections:
                    self.select = self.select_memoire
                if self.interaction:
                    self.SetCursor(wx.StockCursor(wx.CURSOR_QUESTION_ARROW))
                else:
                    if self.deplacable(self.select):
                        self.SetCursor(wx.StockCursor(wx.CURSOR_HAND))
                    else:
                        self.SetCursor(wx.StockCursor(wx.CURSOR_DEFAULT))
                self.select.en_gras()
                #self.ancien_message = 
                self.infos()
            else:
                if self.interaction:
                    self.SetCursor(wx.StockCursor(wx.CURSOR_QUESTION_ARROW))
                else:
                    self.SetCursor(wx.StockCursor(wx.CURSOR_DEFAULT))
                if self.select:
                    self.select = None
                    #self.message(self.ancien_message)
                    self.message("")
                    self.actualiser()

        
    def OnWheel(self, event):     
        "Gestion du zoom par la roulette de la souris."
        if self.feuille_actuelle() == None or self.fixe: return
        if event.GetWheelRotation() > 0:
            self.zoom_in()
        else:
            self.zoom_out()



    def OnMotion(self, event):
        self.SetFocus()
        if self.feuille_actuelle() == None:
            return
                    
        if event.LeftIsDown():
            if not self.HasCapture():
                self.CaptureMouse()

            if event.ControlDown(): # selection d'un zone pour zoomer
                if not self.fixe:
                    #if not self.HasCapture():
                    #    self.CaptureMouse()
                    x, y = event.GetPositionTuple()
                    xmax, ymax = self.GetSize()
                    x = max(min(x, xmax), 0)
                    y = max(min(y, ymax), 0)
                    self.fin_zoom = self.XYpix2coo((x, y), -1)
                    self.debut_zoom = self.debut_zoom or self.fin_zoom
                    (x0, y0), (x1, y1) = self.debut_zoom, self.fin_zoom
                    config = self.graph.en_cache()
                    self.fill(array([x0,x0,x1,x1]), array([y0,y1,y1,y0]), facecolor='c', edgecolor='c',alpha = .1)
                    self.plot(array([x0,x0,x1,x1,x0]), array([y0,y1,y1,y0,y0]), 'c', alpha = 1)
    
                    self.actualiser() # pour ne pas tout rafraichir
                    self.graph.restaurer(config)
            
            elif self.select and not self.interaction: # deplacement d'un objet avec la souris
                self.debut_zoom = None
                if self.deplacable(self.select):
                    self.select(*self.XYpix2coo(event.GetPositionTuple(), -1))
                self.infos()

        elif event.RightIsDown() and self.debut_shift and not self.fixe: # deplacement de la feuille
            self.SetCursor(wx.StockCursor(wx.CURSOR_SIZING))
            self.fin_shift = self.XYpix2coo(event.GetPositionTuple(), -1)
            translation = array(self.fin_shift) - array(self.debut_shift)
            self.set_fenetre(self.fen(0) - translation[0], self.fen(1) - translation[0], self.fen(2) - translation[1], self.fen(3) - translation[1])
            if self.select:
                self.select = None
                
        elif not event.ControlDown():   # detection des objets a proximite du pointeur
            self.detecter(event.GetPositionTuple())
        
            

    def OnLeave(self, event):
        if self.feuille_actuelle() == None:
            return
                    
        if self.debut_zoom and not self.fixe:
            self.actualiser() # pour ne pas tout rafraichir
            

    
    
    def OnLeftUp(self, event):
        if self.feuille_actuelle() == None: return

        if self.HasCapture():
            self.ReleaseMouse()
        
        if self.debut_zoom and not self.fixe:
            #self.ReleaseMouse()
            if event.ControlDown():
                (x0, y0), (x1, y1) = self.debut_zoom, self.fin_zoom
                self.commande.executer("set_fenetre(" + str((x0, x1, y0, y1))[1:-1] + ")")
            else:
                self.parent.affiche()
            self.debut_zoom = None
            
        elif self.interaction:
            self.signal(event)
            self.SetCursor(wx.StockCursor(wx.CURSOR_DEFAULT))
            
        elif self.deplacable(self.select):
            self.parent.historique.enregistrer(self.select.nom() + str(tuple(self.select.coo())))
            # pas super precis : il se peut qu'on relache le bouton sans avoir deplace le point.
            # ca fait un enregistrement inutile dans l'historique...

    
    
    def OnRightDown(self, event):
        if self.feuille_actuelle() == None: return

        self.editeur.init()
        self.detecter(event.GetPositionTuple())

        if self.select and not event.ControlDown():
            menu = wx.Menu()
            menu.SetTitle(self.select.nom_complet())
            selections = [obj for obj in self.selections if obj is not self.select] # autres objets a proximite
            n = len(selections)
            for i in range(n):
                def select(event, obj = selections[i]):
                    self.select = self.select_memoire = obj
                menu.Append(i, "Selectionner " + selections[i].nom_complet())
                menu.Bind(wx.EVT_MENU, select, id =i)
            if n:
                menu.AppendSeparator()

            menu.Append(n, "Supprimer")
            def supprimer(event, select = self.select):
                self.commande.executer("%s.supprimer()" %select.nom())
            menu.Bind(wx.EVT_MENU, supprimer, id = n)

            menu.Append(n + 1, "Masquer")
            def masquer(event, select = self.select):   self.commande.executer("%s.cacher()" %select.nom())
            menu.Bind(wx.EVT_MENU, masquer, id = n + 1)
            
            menu.Append(n + 2, "Renommer")
            def renommer(event, select = self.select):
                dlg = wx.TextEntryDialog(self, "Note: pour personnaliser le texte de l'objet, modifiez plutot son etiquette.", "Renommer l'objet", select.nom())
                test = True
                while test:
                    test = dlg.ShowModal()
                    if test == wx.ID_OK:
                        try:    # on renomme, et on met l'affichage de la lgende en mode "Nom"
                            self.commande.executer("%s.renomme(%s, legende = %s)" %(select.nom(), repr(dlg.GetValue()), param.NOM))
                            break
                        except:
                            if param.debug: print_error()
                            
                    if test == wx.ID_CANCEL:
                        break
                dlg.Destroy()
            menu.Bind(wx.EVT_MENU, renommer, id = n + 2)

            menu.Append(n + 3, "Etiquette")
            def etiquette(event, select = self.select):
                old_label = select.style("label")
                if old_label is None:   # le style label n'existe pas pour l'objet
                    return
                dlg = wx.TextEntryDialog(self, "Note: le code LATEX doit etre entre $$. Ex: $\\alpha$", "Changer l'etiquette de l'objet", old_label)
                test = True
                while test:
                    test = dlg.ShowModal()
                    if test == wx.ID_OK:
                        try:
                            self.commande.executer("%s.label(%s)" %(select.nom(), repr(dlg.GetValue())))
                            break
                        except:
                            pass
                    if test == wx.ID_CANCEL:
                        #select.label(old_label)
                        break
                dlg.Destroy()
            menu.Bind(wx.EVT_MENU, etiquette, id = n + 3)
            
            
            menu.AppendSeparator()
            
            menu.Append(n + 4, "Proprietes")
            def proprietes(event, select = self.select):
                from GUI.proprietes_objets import Proprietes
                win = Proprietes(self, [select])
                win.Show(True)
            menu.Bind(wx.EVT_MENU, proprietes, id = n + 4)

            self.PopupMenu(menu)
            menu.Destroy()
            self.select = None
            self.SetCursor(wx.StockCursor(wx.CURSOR_DEFAULT))
        elif not self.fixe:
            self.debut_shift = self.XYpix2coo(event.GetPositionTuple(), -1)
            self.SetCursor(wx.StockCursor(wx.CURSOR_SIZING))
            if not self.HasCapture():
                self.CaptureMouse()


    
    
    def OnRightUp(self, event):
        if self.feuille_actuelle() == None or self.fixe: return

        if self.HasCapture():
            self.ReleaseMouse()
        self.SetCursor(wx.StockCursor(wx.CURSOR_DEFAULT))

#         (x0, y0), (x1, y1) = self.debut_zoom, self.fin_zoom
#         self.commande.executer("set_fenetre(" + str((x0, x1, y0, y1))[1:-1] + ")")
        if self.debut_shift:
            self.debut_shift= None
            self.parent.historique.enregistrer("set_fenetre(" + str((self.fen()))[1:-1] + ")")
#         self.parent.affiche()


    def editer(self, mode = 0):
        if isinstance(self.select, (Point_generique, Texte)):
            self.editeur.init(self.select, mode)
        elif self.select:
            dlg = wx.TextEntryDialog(self, "Note: pour personnaliser le texte de l'objet, modifiez plutot son etiquette.", "Renommer l'objet", self.select.nom())
            test = True
            while test:
                test = dlg.ShowModal()
                if test == wx.ID_OK:
                    try:
                        self.commande.executer("%s.renomme(%s)" %(self.select.nom(), repr(dlg.GetValue())))
                        break
                    except:
                        print_error()
                if test == wx.ID_CANCEL:
                    break
                    
            dlg.Destroy()

    
    def OnChar(self, event):
        key = event.GetKeyCode()
        debug(key)
        if key == wx.WXK_DELETE and self.select:
            if event.ShiftDown():
                self.commande.executer("%s.cacher()" %self.select.nom())
            else:
                self.commande.executer("%s.supprimer()" %self.select.nom())
        if self.editeur:
            self.editeur.key(key)
        elif key == 13: 
            self.editer(event.ShiftDown())
        
        debug(key)
        debug(chr(key))
        event.Skip()
    
    
    
#    def OnPaint(self, event):
#        self.rafraichir_axes = True
#        self.parent.affiche()


            
#      Fonctions de dessin
#         A SUPPRIMER ?
###############################

    def Ligne(self, A = (0, 0), B = (0, 0), couleur = "k"):
        """exemple : Droite([1, 3], [-2, 5], 'b'"""
        return self.plot(array((A[0], B[0])), array((A[1], B[1])), couleur)
        
    def Point(self, couple = (0, 0), couleur = "k"):
        #(x, y), (xx, yy), r = couple, self.XYcoo2pix(couple), self.taille["+"]
        #self.plot(self.Xpix2coo([xx - r, xx + r]), [y, y], couleur)
        #self.plot([x, x], self.Ypix2coo([yy - r, yy + r]), couleur)
        return self.plot(array([couple[0]]), array([couple[1]]), markeredgecolor = couleur, markerfacecolor = couleur, marker=self.style_point, markersize = 2*self.taille["+"], markeredgewidth = self.largeur_point)
        
    def Cercle(self, couple = (0, 0), r = 1, couleur = "k"):
        t = concatenate((arange(0, 2*pi , self.pas()), [0]))
        return self.plot(couple[0] + r*cos(t), couple[1] + r*sin(t), color = couleur)
        
    def Texte(self, texte = "", couple = (0, 0), couleur = "k", taille = 12):
        return self.graph.fast_text(couple[0], couple[1], texte, color = couleur, size = taille)
