# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Menu                                   #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA




# importation des librairies classiques
from LIB import *
from API import *



#-----------------------
#       Menus
#-----------------------


class RSSMenu(wx.Menu): # parent : wxgeometrie (Frame principal)
    def __init__(self, parent, titre = "", liste = None, fonction = None, aide = ""):
        wx.Menu.__init__(self)
        self.parent = parent
        self.table_ids = {}
        self.fonction = None
        self.titre = titre
        self.update(liste, fonction, aide)
        
    def update(self, liste = None, fonction = None, aide = None):
        if fonction <> None:
            self.fonction = fonction
        if aide <> None:
            self.aide = aide
        self.liste = liste or []
        
        for id in self.table_ids.keys():
            if self.fonction: # bof: pas super precis (si on rajoute une fonction entre temps, erreur)
                self.parent.parent.Unbind(wx.EVT_MENU, id = id)
            self.Delete(id)
        self.table_ids = {}
        
        for contenu in self.liste:
            id = wx.NewId()
            self.table_ids[id] = contenu
            self.Append(id, contenu, self.aide)
            if self.fonction:
                self.parent.parent.Bind(wx.EVT_MENU, self.gerer_evenement, id = id)
        

    def gerer_evenement(self, event): # rajoute a l'evenement la propriete nom_menu, avant de le rediriger.
        contenu = self.table_ids[event.GetId()]
        event.nom_menu = contenu
        self.fonction(event)




class Menu(wx.Menu):
    "Remplace la classe wx.Menu." 
    
    def __init__(self, menubar, liste):
        wx.Menu.__init__(self)
        self.parent = menubar.parent
        self.table_ids = {}
        for contenu in liste:
            id = wx.NewId() # pour etre sur que 2 menus (dans 2 modules differents) n'auront pas le meme id.
            
            if not contenu:
                self.AppendSeparator()
                
            elif isinstance(contenu, RSSMenu):
                self.AppendMenu(id, "&" + contenu.titre, contenu)

            else:
                
                if len(contenu) == 1: # menus predefinis
                    contenu = menubar.menus[contenu[0]]
                
                self.table_ids[id] = contenu
                if isinstance(contenu[1], (list, tuple)): # le menu contient un sous-menu
                    menu = Menu(menubar, contenu[1:])
                    self.AppendMenu(id, "&" + contenu[0], menu)
                    
                else:
                    if not contenu[1]:
                        contenu[1] = ""
                    # 0->titre, 1->description, 2->raccourci, 3->fonction associee[, 4->cocher ou non]
                    titre = "&" + contenu[0]
                    if contenu[2]:
                        titre += "\t" + contenu[2]
                    if len(contenu) == 5:   # le menu contient une case a cocher
                        self.Append(id, titre, contenu[1], wx.ITEM_CHECK)
                        self.Check(id, contenu[4])
                    else:                   # cas classique
                        self.Append(id, titre, contenu[1])
                    if contenu[3]:
                        self.parent.parent.Bind(wx.EVT_MENU, self.gerer_evenement, id = id)
                    

    def gerer_evenement(self, event): # rajoute a l'evenement la propriete nom_menu, avant de le rediriger.
        contenu = self.table_ids[event.GetId()]
        event.nom_menu = contenu[0]
        contenu[3](event)





class MenuBar(wx.MenuBar):
    """Remplace la classe wx.MenuBar pour creer une barre de menu.
    La methode de base est 'ajouter(self, *menu)', ou menu est une liste.
    Exemple:
    menu =  ["Outils",
                ["Options", "Parametres du programme", "Ctrl+O", fonction1],
                ["Memoriser le resultat", "Copie le resultat du calcul dans le presse-papier", None, fonction2],
                None,
                ["Autres", 
                    ["Rubrique 1", None, "Alt+Ctrl+R", fonction3],
                    ["Rubrique 2", "Rubrique non active pour l'instant", None, None]]]
    Pour un menu moins standard, on peut toujours utiliser wx.MenuBar.
    """
                    
    def ajouter(self, *contenu):
        if isinstance(contenu, RSSMenu):
            self.Append(contenu, "&" + contenu.titre)
        else:
            if len(contenu) == 1: # menus predefinis
                contenu = self.menus[contenu[0]]
            menu = Menu(self, contenu[1:])
            self.Append(menu, "&" + contenu[0])


    def __init__(self, panel = None, *contenu): # parent : wxgeometrie (Frame principal)
        wx.MenuBar.__init__(self)
        self.panel = panel
        self.panel.menu = self
        self.parent = panel.parent
        self.canvas = panel.canvas
        self.historique = panel.historique
        self.commande = panel.commande
        
       
        for item in contenu:
            self.ajouter(*item)


        # Menus predefinis:
        self.menus = {
        
    
"nouveau":  ["Nouveau", "Creer un nouveau fichier.", "Ctrl+N", self.parent.NewFile],
"ouvrir":   ["Ouvrir", "Ouvrir un fichier.", "Ctrl+O", self.parent.OpenFile],
"ouvrir ici":   ["Ouvrir ici", "Essayer d'ouvrir le fichier dans le module courant.", "Alt+Ctrl+O", self.parent.OpenFileHere],
"enregistrer":  ["Enregistrer sous...", "Enregistrer le document.", "Ctrl+S", self.parent.SaveFile],
"exporter": ["Exporter...", "Exporter l'image.", "Ctrl+E", self.parent.ExportFile],
"exporter&sauver": ["Exporter et sauver", "Exporter l'image, et sauvegarder le document.", "Alt+Ctrl+E", self.parent.ExportAndSaveFile],
"mise en page": ["Parametres d'impression", "Regler les parametres d'impression.", None, self.parent.PageSetup],
"imprimer": ["Imprimer", "Imprimer la figure geometrique courante.", "Ctrl+P", self.parent.Printout],
"fermer":   ["Fermer", "Fermer la feuille courante.", "Ctrl+F4", self.parent.CloseFile],
"quitter":  ["Quitter", "Fermer le programme.", "Alt+F4", self.parent.CloseWindow],



"onglet":   ["Onglet suivant", "Changer d'onglet.", "Ctrl+SPACE", self.parent.onglet_suivant], 
"securise": ["Mode securise", "Activer le mode securise (recommande).", "Alt+Ctrl+S", self.parent.Securite, param.securite],
"debug":    ["Deboguer", "Deboguer le programme (afficher les erreurs, ...).", None, self.parent.Debug, param.debug],
"options":  ["Options", "Parametres du programme.", None, None],

"aide":     ["Aide", "Obtenir de l'aide sur le programme.", None, self.parent.Aide],
"notes":    ["Notes de version", "Consulter l'historique du projet.", None, self.parent.Notes],
"about":    ["A propos...", "WxGeometrie (c) 2005 Nicolas Pourcelot - License : GPL version 2", None, self.parent.About],



        }


        self.menus["fichier"] = ["Fichier", ["nouveau"], ["ouvrir"], ["ouvrir ici"], ["enregistrer"], ["exporter"], ["exporter&sauver"], None, ["mise en page"], ["imprimer"], None, ["fermer"], ["quitter"]]
                
        self.menus["?"] = ["?", ["aide"], ["notes"], None, ["about"]]
        
        
        
        if self.historique:
            self.menus.update({
"annuler":  ["Annuler", "Annuler la derniere action.", "Ctrl+Z", self.panel.historique.annuler],
"refaire":  ["Refaire", "Refait la derniere action annulee.", "Ctrl+Y", self.panel.historique.retablir],
"historique":   ["Historique", "Historique des commandes.", "Ctrl+H", self.parent.Histo]
            })




        if self.canvas:
            self.menus.update({
"repere":   ["Afficher le repere", "Afficher le repere et les axes.", None, self.canvas.afficher_repere, self.canvas.affiche_axes],
"quadrillage":      ["Afficher le quadrillage", "Afficher le quadrillage.", None, self.canvas.afficher_quadrillage, self.canvas.affiche_quadrillage],
"fenetre":  ["Reglage de la fenetre", "Reglage de la fenetre d'affichage.", None, self.parent.creer["Fenetre"]],
"zoomer":   ["Zoomer", "Se rapprocher de la figure.", "Ctrl+PGUP", self.canvas.zoom_in],
"dezoomer": ["Dezoomer", "S'eloigner de la figure.", "Ctrl+PGDN", self.canvas.zoom_out],
"orthonormaliser":  ["Orthonormaliser", "Obtenir un repere orthonormal.", "Alt+Ctrl+O", self.canvas.repere_orthonormal],
"zoom_auto":  ["Zoom intelligent", "Reglage automatique de la fenetre d'affichage.", "Alt+Ctrl+A", self.canvas.zoom_automatique],

"modifier":["Modifier", "Editer les proprietes d'un ou plusieurs objets geometriques.", "Alt+Ctrl+M", self.parent.editer],
"supprimer":["Supprimer", "Supprime un ou plusieurs objets geometriques.", "Alt+Ctrl+S", self.parent.supprimer],

"creer":    ["Creer",
                ["Points",
                    ["Point libre", "Point quelconque.", "Ctrl+L", self.parent.creer["Point"]],
                    ["Milieu", "Milieu d'un segment.", "Ctrl+M", self.parent.creer["Milieu"]],
                    ["Barycentre", "Barycentre de n points.", "Ctrl+B", self.parent.creer["Barycentre"]],
                    ["Point final", "Point defini par une relation vectorielle.", "Ctrl+F", self.parent.creer["PointFinal"]],
                    ["Point sur droite", "Point appartenant a une droite.", "Alt+Ctrl+D", self.parent.creer["GlisseurDroite"]],
                    ["Point sur segment", "Point appartenant a un segment.", "Alt+Ctrl+G", self.parent.creer["GlisseurSegment"]],
                    ["Point sur cercle", "Point appartenant a un cercle.", "Alt+Ctrl+K", self.parent.creer["GlisseurCercle"]],
                ],
                ["Lignes",
                    ["Segment", "Segment defini par deux points.", "Ctrl+G", self.parent.creer["Segment"]],
                    ["Droite", "Droite definie par deux points.", "Ctrl+D", self.parent.creer["Droite"]],
                    ["Demi-droite", "Demi-droite definie par son origine et un autre point.", None, self.parent.creer["Demidroite"]],
                    ["Vecteur", "Vecteur defini par deux points.", "Ctrl+W", self.parent.creer["Vecteur"]],
                    ["Vecteur libre", "Vecteur defini par ses coordonnees.", None, self.parent.creer["VecteurLibre"]],
                    ["Parallele", "Parallele a une droite passant par un point.", None, self.parent.creer["Parallele"]],
                    ["Perpendiculaire", "Perpendiculaire a une droite passant par un point.", None, self.parent.creer["Perpendiculaire"]],
                    ["Mediatrice", "Mediatrice d'un segment.", None, self.parent.creer["Mediatrice"]],
                    ["Bissectrice", "Bissectrice d'un angle.", None, self.parent.creer["Bissectrice"]],
                    ["Tangente", "Tangente a un cercle.", None, self.parent.creer["Tangente"]],
                ],
                ["Cercles",
                    ["Cercle de centre fixe", "Cercle de centre fixe et de rayon variable.", "Ctrl+K", self.parent.creer["Cercle"]],
                    ["Cercle de centre et rayon fixes", "Cercle de centre et rayon fixes.", "Ctrl+R", self.parent.creer["CercleRayon"]],
                    ["Cercle de diametre fixe", "Cercle de diametre fixe.", None, self.parent.creer["CercleDiametre"]],
                    ["Arc de cercle", "Arc de cercle de sens direct.", None, self.parent.creer["ArcCercle"]],
                    ["Demi-cercle", "Demi-cercle de diametre donne, de sens direct.", None, self.parent.creer["DemiCercle"]],
                ],
                ["Intersections",
                    ["Intersection de deux droites", "Point defini par l'intersection de deux droites.", "Ctrl+I", self.parent.creer["InterDroites"]],
                    ["Intersection d'une droite et d'un cercle", "Point defini par l'intersection de deux droites.", "Alt+Ctrl+I", self.parent.creer["InterDroiteCercle"]],
                    ["Intersection de deux cercles", "Point defini par l'intersection de deux droites.", None,  self.parent.creer["InterCercles"]],
                ],
                ["Polygones",
                    ["Triangle", "Triangle defini par ses sommets.", None, self.parent.creer["Triangle"]],
                    ["Polygone quelconque", "Polygone quelconque, defini par ses sommets.", None, self.parent.creer["Polygone"]],
                ],
                ["Centres",
                    ["Centre d'un cercle", "Centre d'un cercle.", None, self.parent.creer["Centre"]],
                    ["Centre de gravite", "Centre de gravite d'un triangle (intersection des medianes).", None, self.parent.creer["CentreGravite"]],
                    ["Orthocentre", "Orthocentre d'un triangle (intersection des hauteurs).", None, self.parent.creer["Orthocentre"]],
                    ["Centre du cercle circonscrit", "Centre du cercle circonscrit d'un triangle (intersection des mediatrices).", None, self.parent.creer["CentreCercleCirconscrit"]],
                    ["Centre du cercle inscrit", "Centre du cercle inscrit d'un triangle (intersection des bissectrices).", None, self.parent.creer["CentreCercleInscrit"]],
                ],
                ["Divers",
                    ["Texte", "Champ de texte.", None, self.parent.creer["Texte"]],
                ],
            ]

            })
            
            




