# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Panel                     #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA




# importation des librairies classiques
from LIB import *
from API import *
from sauvegarde import *

#-----------------------
#       Panels
#-----------------------



#class Canvas_vide:
#    """Canvas bidon, qui sert pour l'initialisation des classes menu."""
#    def __getattr__(self, nom):
#        return None
#        
#    def __nonzero__(self):
#        return False
#        
#    def __repr__(self):
#        return "<Canvas Vide>"
#        
#    def __str__(self):
#        return "<Canvas Vide>"
  
  
  


class Panel_simple(wx.Panel):
    """Remplace la classe la classe wx.Panel pour les differents modules.
    Pour les modules ayant besoin des fonctions graphiques evoluees de WxGeometrie, mieux vaut utiliser
    la classe Panel_API_graphique, plus evoluee que celle-ci."""
    
    def __init__(self, parent, couleur = None):
        wx.Panel.__init__(self, parent, -1)
        
        self.SetBackgroundColour(couleur <> None or wx.NamedColor("WHITE"))
        self.canvas = self.historique = self.commande = None
        self.parent = parent
        self.nom = self.__class__.__name__.lower()
        
        self.actuelle = None # pas de feuille
        
        self.menu = None

        #if self.menu == None:
        #    import MOD
        #    self.menu = getattr(MOD, self.nom + "MenuBar")
        #    self.menu.module = self
        #    del MOD
    
    
    def message(self, texte = ""):
        self.parent.parent.barre.SetStatusText(texte, 0)

    def changer_titre(self, texte = None):
        if texte == None:
            texte = ""
            if self.actuelle <> None:
                texte = self.actuelle.nom()            
        self.parent.parent.titre(texte)


    def ouvrir(self, fichier):
        fgeo, message = ouvrir_fichierGEO(fichier)
        self.message(message)
        self._ouvrir(fgeo)
            



    def sauvegarder(self, fichier = ""):
        while fichier[-4:] == ".geo": fichier = fichier[:-4]
        if not fichier:
            if self.actuelle:
                fichier = self.actuelle.nom()
            else:
                fichier = "sauvegarde"
        fichier += ".geo"     # le nom par defaut sera ainsi [nom_de_la_feuille].geo"
        f = None
        try:
            f = open(fichier, "w")
            fgeo = FichierGEO(module = self.nom)
            self._sauvegarder(fgeo)
            f.write(fgeo.exporter())
            f.close()
            self.message("Sauvegarde effectuee.")   
        except:
            if f:   f.close()
            self.message("Echec de la sauvegarde.")
            if param.debug: raise
        


    def _ouvrir(self, fgeo): # fonction a reecrire pour chaque module
        pass

        
    def _sauvegarder(self, fgeo): # fonction a reecrire pour chaque module
        pass

        




class Panel_API_graphique(Panel_simple):
    """Pour les modules ayant besoin de TOUTE l'API de WxGeometrie (canvas, historique, gestion des commandes, etc...)
    et pas seulement des bibliotheques, mieux vaut utiliser cette classe.
    Cela concerne essentiellement les modules qui ont besoin de tracer des objets geometriques."""

    def __init__(self, parent):
        Panel_simple.__init__(self, parent)
        
        # contruire toujours dans cet ordre :
        self.historique = Historique(self)
        self.commande = Commande(self)
        self.canvas = Affichage(self)
        self.numero_feuille = 0
        self.feuilles = {}
        self.doc_ouverts = RSSMenu(self.parent, "Documents ouverts", [], self.charger, "Documents ouverts.")
        self.nouveau()
        self.canvas.initialiser()


    def nouveau(self):
        self.commande.executer("creer_feuille()")
        
    def charger(self, event):
        self.commande.executer("charger_feuille('%s')" %event.nom_menu)
    
    def fermer(self):
        self.commande.executer("fermer_feuille()")


    def creer_feuille(self, nom = ""):
        if not nom:
            self.numero_feuille += 1
            nom = "nouvelle_feuille_" + str(self.numero_feuille)
            
        if self.f(nom):    # il y a une feuille de ce nom ouverte
            self.fermer_feuille(nom)

        self.actuelle = self.feuilles[nom] = Feuille(self)
        self.update(self.feuilles[nom])

        
        
    def charger_feuille(self, nom): 
        self.update(self.feuilles[nom])

    


    def fermer_feuille(self, nom = None): # ferme une feuille
        # AJOUTER UNE PROPOSITION DE SAUVEGARDE
        if self.actuelle: # si il y a une feuille (!)
            if nom == None:
                nom = self.actuelle.nom()
            self.feuilles.pop(nom)
            if self.feuilles:
                self.charger_feuille(self.feuilles.keys()[-1])
            else:
                self.update(None)


    def fermer_feuilles(self):
        "Ferme toute les feuilles."
        i = 0
        while self.actuelle and i < 100000:
            i += 1
            #try:
            self.fermer_feuille()
            #except:
            #    print "Fermeture incorrecte de la feuille %s !" %self.actuelle.nom()
            #    self.feuilles.pop(self.actuelle.nom())
            #    if param.debug:
            #        raise

    
    def update(self, feuille):
        self.actuelle = feuille
        self.changer_titre()
        liste_feuilles = self.feuilles.keys()
        liste_feuilles.sort()
        self.doc_ouverts.update(liste_feuilles)
        self.canvas.rafraichir_axes = True
        self.affiche()



    def nom_feuille(self, feuille):
        if not feuille in self.feuilles.values():
            return ""
        return [nom for nom, f in self.feuilles.items() if f == feuille][0]
    
    def feuille(self, nom):
        return self.feuilles.get(nom, None)
        #liste = filter(lambda x:x[1] == nom, self.feuilles.items())
        #return liste and liste[0][0] or None  # si le nom "nom" n'existe pas, renvoie None
        
    def f(self, nom) : return self.feuille(nom) # alias de feuille()


    def _sauvegarder(self, fgeo):
        fgeo.contenu["Figure"] = [self.actuelle.sauvegarder()]
        
        fgeo.contenu["Affichage"] = [{}]
        for parametre in self.canvas.parametres:
            fgeo.contenu["Affichage"][0][parametre] = [repr(getattr(self.canvas, parametre))]

       
        
    def _ouvrir(self, fgeo):
        # Filtrage pour importer des versions anciennes des fichiers .geo
        # Il faudra aussi grer ici dans le futur l'import d'autres formats de fichiers.
        try:
            version = [int(n) for n in fgeo.version.split(".")]
        except:
            debug("Version de fichier inconnue.")
            version = [int(n) for n in param.version.split(".")] # par defaut, on suppose que c'est la version actuelle
        # FILTRAGE A INSERER ICI !
        # fin du filtrage.
            
        if fgeo.contenu.has_key("Affichage"):
            if fgeo.contenu["Affichage"]:
                parametres = fgeo.contenu["Affichage"][0]
                for parametre in parametres.keys():
                    setattr(self.canvas, parametre, securite.eval_safe(parametres[parametre][0]))

        if fgeo.contenu.has_key("Figure"):
            for figure in fgeo.contenu["Figure"]:
                if version < [0, 106]:  # PATCH PROVISOIRE, en attendant d'avoir des filtres d'import pour les vieilles versions
                    figure = figure.replace("'label': None, ", "'label': '', 'legende': 1, ")
                self.canvas.geler_affichage = True # on evite de rafraichir a chaque fois.
                self.commande.executer("creer_feuille()") # pour que ca apparaisse dans l'historique
                try:
                    for commande in figure.split("\n"):
                        self.commande.executer(commande, abreviations = False)
                finally:
                    self.canvas.geler_affichage = False
                    self.rafraichir_feuille()
            
        for macro in fgeo.contenu.get("Macro", []):
            if param.multi_threading:
                thread.start_new_thread(self.commande.executer, (":" + macro + "\nwx.Yield()",), {"abreviations":False})
            else:
                self.commande.executer(":" + macro, abreviations = False)
                

    def rafraichir_feuille(self): # rafraichit la feuille actuelle
        if self.actuelle:
            self.actuelle.refresh()
    
 
    def affiche(self, event = None):
        try:
            self.canvas.effacer()
            if self.actuelle: # si il y a une feuille pour dessiner
                self._affiche()
            self.canvas.affiche()
        except:
            print_error()
            self.message("Erreur d'affichage !")


    def _affiche(self): # fonction a reecrire pour chaque module
        pass




