# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                 Sauvegarde                  #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA




# importation des librairies classiques
from LIB import *
from API import *
from xml.dom.minidom import parseString




#----------------------------------
#     DOM pour lire un fichier
#----------------------------------



class FichierGEO:
    """Classe utilisee pour manipuler un fichier .geo.
    On peut aussi utiliser le DOM XML, mais les specifications du format de fichier .geo
    sont plus restreintes, et cette classe offre une surcouche au DOM XML pour un acces plus simple."""
    
    def __init__(self, texte = "", encoding = "utf-8", **args):
        self.infos = args
        self.encoding = encoding
        self.contenu = {}
        for attribut, defaut in [("type", "WxGeometrie"), ("version", param.version), ("module", ""), ("nom", "")]:
            setattr(self, attribut, self.infos.get(attribut, defaut))

        if texte: self.importer(texte)

            
                        
    def importer(self, texte):
        def contenu_node(node):
            # Le contenu d'une node (contenu entre 2 balises) est soit du texte, soit (ou exclusif!) d'autres balises.
            # Le contenu autorise est ainsi passablement plus restreint que dans la specification XML.
            # Pour plus de detail, lire la doc sur le format de fichier.
            
            # 1er cas: contenu vide -> assimile a du texte.
            if not node.childNodes:
                return ""

            # 2eme cas: le contenu est purement du texte.
            if not "Element" in [subnode.__class__.__name__ for subnode in node.childNodes]:
                return node.childNodes[0].nodeValue.strip("\n").replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")
                                       
            # 3eme cas: le contenu est un ensemble de balises -> regroupees dans un dictionnaire.
            # ce qui n'est pas entre deux balises est ignore.
            dico = {}
            for elt in node.childNodes:
                if elt.__class__.__name__ == "Element":
                    contenu = contenu_node(elt)
                    if dico.has_key(elt.nodeName):
                        dico[elt.nodeName] += [contenu]
                    else:
                        dico[elt.nodeName] = [contenu]
            return dico


        xml = parseString(texte)
        self.encoding = xml.encoding
        self.document = xml.childNodes[0]
        self.infos = {} # contient tous les attributs du document
        for attribut in self.document._attrs.keys():
            self.infos[attribut] = self.document._attrs[attribut].value
        for attribut in ("type", "version", "module", "nom"):
            setattr(self, attribut, self.infos.get(attribut, ""))

        self.contenu = contenu_node(self.document)





    def exporter(self):
        def convertir_contenu(dictionnaire):
            texte = ""
            for balise in dictionnaire.keys():
                for elt in dictionnaire[balise]:
                    texte += "<%s>\n" %balise
                    if isinstance(elt, (str, unicode)):
                        texte += elt.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;").strip("\n") + "\n"
                    elif isinstance(elt, dict):
                        texte += convertir_contenu(elt)
                    texte += "</%s>\n" %balise
                    
            return texte

        texte = "<?xml version='1.0' encoding='%s'?>\n" %self.encoding
        texte += "<Document type='%s' version='%s' module='%s'>\n" %(self.type, self.version, self.module)
        texte += convertir_contenu(self.contenu)
        texte += "</Document>"
        return texte
        
        

def ouvrir_fichierGEO(path):
    try:
        f=open(path,"rU")
    except IOError:
        return None, "Le fichier n'existe pas, ou est inaccessible."
    except UnicodeError:
        return None, "Caracteres non reconnus."
    except:
        return None, "Impossible d'ouvrir le fichier."
    try:
        texte = f.read()
    finally:
        f.close()
    fgeo = FichierGEO(texte)
    return fgeo, "Le fichier %s a bien ete ouvert." %path
