# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)


##--------------------------------------#######
#                 Aide                        #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


from LIB import *
import  wx.html as  html



class Help(wx.Frame):
    def __init__(self, parent, path):
        wx.Frame.__init__(self, parent, -1, size=wx.Size(700,610))

        self.path = path
        
        self.SetBackgroundColour(wx.Colour(255, 225, 153))
        self.html = html.HtmlWindow(self, -1, style=wx.NO_FULL_REPAINT_ON_RESIZE)
        self.html.SetRelatedFrame(self, " %s ")
        #self.html.SetRelatedStatusBar(0)

        self.printer = html.HtmlEasyPrinting()

        self.box = wx.BoxSizer(wx.VERTICAL)

        subbox = wx.BoxSizer(wx.HORIZONTAL)
        
        icones = [("maison","Page d'accueil.", self.OnHome), ("gauche","Page precedente.", self.OnBack), ("droite","Page suivante.", self.OnForward), ("print","Imprimer la page.", self.OnPrint)]

        for i in range(len(icones)):
            icone = icones[i]
            bmp = wx.Image('images' + os.sep + icone[0] + '.png', wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            bouton = wx.BitmapButton(self, -1, bmp, style=wx.NO_BORDER)
            subbox.Add(bouton, 0, wx.ALL,5)
            bouton.SetToolTipString(icone[1])
            bouton.Bind(wx.EVT_BUTTON, icone[2])



        self.box.Add(subbox, 0)
        self.box.Add(self.html, 1, wx.GROW)
        self.SetSizer(self.box)
        self.SetAutoLayout(True)

        self.OnHome(None)


    def OnHome(self, event):
        self.html.LoadPage(self.path)



    def OnBack(self, event):
        self.html.HistoryBack()


    def OnForward(self, event):
        self.html.HistoryForward()


    def OnPrint(self, event):
        self.printer.GetPrintData().SetPaperId(wx.PAPER_LETTER)
        self.printer.PrintFile(self.html.GetOpenedPage())







class About(wx.Dialog):
    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, -1, "A propos de Wxgeometrie")
        self.SetBackgroundColour(wx.WHITE)
        
        panel = wx.Panel(self, -1)
        panel.SetBackgroundColour(wx.WHITE)
        
        panelSizer = wx.BoxSizer(wx.VERTICAL)
        
        italic = wx.Font(panel.GetFont().GetPointSize(),
                          panel.GetFont().GetFamily(),
                          wx.ITALIC, wx.NORMAL)
        
        logo = wx.StaticBitmap(panel, -1, wx.Image(name = "images"+os.sep+"logo2.png").ConvertToBitmap())
            
        panelSizer.Add(logo, 0, wx.ALIGN_CENTRE)

        textes = [["WxGeometrie version %s" % param.version]]
        textes.append([])
        textes.append(["De la geometrie dynamique, un traceur de courbes, et bien plus..."])
        textes.append(["WxGeometrie est un logiciel libre, vous pouvez l'utiliser et le modifier comme vous le souhaitez."])
        textes.append(["Copyright 2005-2006 Nicolas Pourcelot (wxgeo@users.sourceforge.net)", italic])
        textes.append([])
        textes.append(["A Sophie."])
        textes.append(["'Le reve est bien reel. Effleurant votre main, je peux toucher le ciel!'  Alain Ayroles", italic])
        textes.append(["Tous mes remerciements a la communaute du libre."])
        textes.append([])
        
        for texte in textes:
            l = len(texte)
            if l:
                txt = wx.StaticText(panel, -1, texte[0])
                if l > 1:  txt.SetFont(texte[1])
                panelSizer.Add(txt, 0, wx.ALIGN_LEFT)
            else:
                panelSizer.Add((5, 5)) # Spacer.
                
        
        btnOK = wx.Button(panel, wx.ID_OK, "OK")
        
        panelSizer.Add(btnOK, 0, wx.ALL | wx.ALIGN_CENTRE, 5)
        
        panel.SetAutoLayout(True)
        panel.SetSizer(panelSizer)
        panelSizer.Fit(panel)
        
        topSizer = wx.BoxSizer(wx.HORIZONTAL)
        topSizer.Add(panel, 0, wx.ALL, 10)
        
        self.SetAutoLayout(True)
        self.SetSizer(topSizer)
        topSizer.Fit(self)
        
        self.Centre()
