# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                 Fenetres                              #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



# importation des librairies classiques
from LIB import *




####################################################################################################

# Differentes boites de dialogue pour creer des objets geometriques

# A REECRIRE EN UTILISANT wxGridBagSizer ??

class Dialogue(wx.Dialog):
    
    objet = None # classe associee (utilise pour les boites de dialogue de creation d'objets geometriques)
    
    def __init__(self, parent, titre = "", size = wx.DefaultSize):
        """S'il s'agit d'un dialogue pour la creation d'objets,
        le titre par defaut est genere automatiquement a partir de l'attribut de classe 'objet'."""
        
        if self.objet and not titre:
            titre = "Creer " + self.objet.titre() 
               
        provider = wx.SimpleHelpProvider()
        wx.HelpProvider_Set(provider)
        # Instead of calling wx.Dialog.__init__ we precreate the dialog
        # so we can set an extra style that must be set before
        # creation, and then we create the GUI dialog using the Create
        # method.
        pre = wx.PreDialog()
        pre.SetExtraStyle(wx.DIALOG_EX_CONTEXTHELP)
        pre.Create(parent, -1, titre, wx.DefaultPosition, size, wx.DEFAULT_DIALOG_STYLE) # si l'objet est nul, ce n'est pas une boite de creation d'objet

        # This next step is the most important, it turns this Python
        # object into the real wrapper of the dialog (instead of pre)
        # as far as the wxPython extension is concerned.
        self.PostCreate(pre)

        # Now continue with the normal construction of the dialog
        # contents
        self.parent = parent
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.champs = {}
                
        if self.objet : self.ajoute([("Nom %s : " %self.objet.titre("du", False)), ("nom", 10)], "Entrez le nom de l'objet. Exemples : A, AB, Cercle, M2...")

        
        
        
    def ajoute(self, contenu, aide = ""):
        # ajoute une ligne de contenu dans la boite de dialogue
        # format du contenu : ("texte statique",("nom de champ", taille), etc...)
        # exemple : ("entrez l'abscisse :",("absc",10),"cm")
        # pour un champ, un 3eme argument peut-etre saisi, pour donner le comportement en cas d'agrandissement,
        # et un 4eme, pour indiquer une valeur initiale du champ
        # le champ cree sera accessible via self.champ("abcs")
        
        self.box = wx.BoxSizer(wx.HORIZONTAL)
        for txt in contenu:
            if type(txt) == str:     # texte statique
                texte = wx.StaticText(self, -1, txt)
                texte.SetHelpText(aide)
                self.box.Add(texte, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
                
            else:  # champ de saisie
                self.champs[txt[0]] = wx.TextCtrl(self,-1, (len(txt) == 4) and str(txt[3]) or "", size = (10*txt[1],-1))
                self.champs[txt[0]].SetHelpText(aide)
                self.box.Add(self.champs[txt[0]], (len(txt) == 3) and txt[2] or 0, wx.ALIGN_CENTRE|wx.ALL, 5)
        self.sizer.Add(self.box, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5) # ajoute la ligne
        
        
    def finalise(self):     # cette fonction est a appeler une fois tout le contenu ajoute.

        line = wx.StaticLine(self, -1, size=(20,-1), style=wx.LI_HORIZONTAL)
        self.sizer.Add(line, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.RIGHT|wx.TOP, 5)

        self.box = wx.BoxSizer(wx.HORIZONTAL)

        if wx.Platform != "__WXMSW__":
            btn = wx.ContextHelpButton(self)
            self.box.Add(btn, 0, wx.ALIGN_CENTRE|wx.ALL, 5)

        btn = wx.Button(self, wx.ID_OK, self.objet and " Creer " or "Enregistrer")
        btn.SetHelpText(self.objet and "Creer l'objet." or "Enregistrer les modifications.")
        btn.SetDefault()
        self.box.Add(btn, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        btn = wx.Button(self, wx.ID_CANCEL, " Annuler ")
        btn.SetHelpText("Quitter sans rien changer.")
        self.box.Add(btn, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        self.sizer.Add(self.box, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        self.SetSizer(self.sizer)
        self.SetAutoLayout(True)
        self.sizer.Fit(self)
        self.CenterOnScreen()
    
    def affiche(self):
        return self.ShowModal()
    
    def commande(self):
        return self.champ("nom") + "=" + self.objet.classe() + "(" + self.parametres() + ")"

    def champ(self, etiquette):
        return self.champs[etiquette].GetValue()

    def parametres(self):   # le parametre par defaut est None
        return ",".join([etiquette + "=" + (self.champ(etiquette).strip() or "None") for etiquette in self.champs if etiquette <> "nom"])





class DialoguePoint(Dialogue):
    objet = Point
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Coordonnees du point :"], "Entrez les coordonnees de votre nouveau point.")
        self.ajoute(["Abscisse :", ("x", 10)], "Entrez ici l'abscisse du point. Exemple : 3.25")
        self.ajoute(["Ordonnee :", ("y", 10)], "Entrez ici l'ordonnee du point. Exemple : -5")
        self.finalise()



class DialogueSegment(Dialogue):
    objet = Segment
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Extremites du segment :"], "Entrez les extremites du nouveau segment.")
        self.ajoute(["Premier point :", ("point1", 10)], "Entrez ici le premier point. Exemple : A")
        self.ajoute(["Deuxieme point :", ("point2", 10)], "Entrez ici le deuxieme point. Exemple : B")
        self.finalise()



class DialogueDroite(Dialogue):
    objet = Droite
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Donnez deux points de la droite :"], "Definissez la droite en entrant deux points de la droite.")
        self.ajoute(["Premier point :", ("point1", 10)], "Entrez ici le premier point. Exemple : A")
        self.ajoute(["Deuxieme point :", ("point2", 10)], "Entrez ici le deuxieme point. Exemple : B")
        self.finalise()



class DialogueDemidroite(Dialogue):
    objet = Demidroite
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Donnez deux points de la demi-droite :"], "Definissez la demi-droite en entrant son origine, et un autre point.")
        self.ajoute(["Origine :", ("origine", 10)], "Entrez ici son origine. Exemple : A")
        self.ajoute(["Deuxieme point :", ("point", 10)], "Entrez ici un deuxieme point. Exemple : B")
        self.finalise()


class DialogueVecteur(Dialogue):
    objet = Vecteur
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Extremites du vecteur :"], "Entrez les extremites du nouveau vecteur.")
        self.ajoute(["Point de depart :", ("point1", 10)], "Entrez ici le premier point. Exemple : A")
        self.ajoute(["Point d'arrivee :", ("point2", 10)], "Entrez ici le deuxieme point. Exemple : B")
        self.finalise()


class DialogueVecteurLibre(Dialogue):
    objet = Vecteur_libre
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Coordonnees du vecteur :"], "Entrez les coordonnees de votre nouveau vecteur.")
        self.ajoute(["Abscisse :", ("x", 10)], "Entrez ici l'abscisse du vecteur. Exemple : 3.25")
        self.ajoute(["Ordonnee :", ("y", 10)], "Entrez ici l'ordonnee du vecteur. Exemple : -5")
        self.finalise()


class DialogueMilieu(Dialogue):
    objet = Milieu
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Construire le milieu des points :"], "Entrez les extremites du segment.")
        self.ajoute(["Premier point :", ("point1", 10)], "Entrez ici le premier point. Exemple : A")
        self.ajoute(["Deuxieme point :", ("point2", 10)], "Entrez ici le deuxieme point. Exemple : B")
        self.finalise()



class DialogueBarycentre(Dialogue):
    objet = Barycentre
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Construire le barycentre :"], "Entrez les points, puis les coefficients.")
        self.ajoute(["des points :", ("points", 10)], "Entrez ici les points separes par des virgules. Exemple : A,B,C")
        self.ajoute(["avec les coefficients :", ("coeffs", 10)], "Entrez ici les coefficients (de somme non nulle !) separes par des virgules. Exemple : 5,3,1.5")
        self.finalise()

    def parametres(self):
        return "[" + self.champ("points") + "],[" + self.champ("coeffs") + "]"




class DialoguePointFinal(Dialogue):
    objet = Point_final
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Construire le point tel qu'on ait :"], "Entrez la relation vectorielle.")
        self.ajoute([("point1",2), ">", ("point2", 2), "=", ("relation", 10, 1)], \
        """Entrez ici la relation vectorielle. Exemple, si N est le point a construire :\nA>N = 2 B>C + 5/3 D>E.
        Attention : le membre de droite ne doit contenir que des points deja existants.""")
        self.finalise()

    def commande(self):
        relation = regexp("[A-Z_a-z][A-Z_a-z0-9 ]*>[ ]*[A-Z_a-z][A-Z_a-z0-9]*", self.champ("relation"), "'(' + x + ')'")
        relation = regexp("[0-9][ ]*[(]", relation, "x[:-1] + '*('")
        autre_point = self.champ("point1")
        if autre_point == self.champ("nom"):
            relation = "-(%s)" %relation
            autre_point = self.champ("point2")
        return self.champ("nom") + "=" + autre_point + "+(" +  relation + ")"





class DialogueCercle(Dialogue):
    objet = Cercle
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer une cercle de centre :", ("centre", 5)], "Entrez ici le centre du cercle. Exemple : M")
        self.ajoute(["et de rayon :", ("rayon", 5)], "Entrez ici le rayon. Exemple : 3")
        self.finalise()


    
class DialogueCercleRayon(Dialogue):
    objet = Cercle_rayon
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer une cercle de centre :", ("centre", 5)], "Entrez ici le centre du cercle. Exemple : O")
        self.ajoute(["auquel appartient :", ("point", 5)], "Entrez ici un point du cercle. Exemple : M")
        self.finalise()

    
    
class DialogueCercleDiametre(Dialogue):
    objet = Cercle_diametre
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Donnez un diametre du cercle :"])
        self.ajoute(["Creer une cercle de diametre : [", ("point1", 5), ("point2", 5), "]"], "Entrez les extremites du diametre. Exemple : A et B")
        self.finalise()




class DialogueArcCercle(Dialogue):
    objet = Arc_cercle
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Centre du cercle :", ("centre", 5)], "Entrez ici le centre du cercle. Exemple : O")
        self.ajoute(["Premier point de l'arc : ", ("point1", 5)], "Entrez le 1er point de l'arc. L'arc est parcouru dans le sens direct. Exemple A")
        self.ajoute(["Deuxieme point : ", ("point2", 5)], "Entrez un 2e point. Il ne sera pas forcement sur l'arc. Exemple : B")
        self.finalise()
    
    

class DialogueDemiCercle(Dialogue):
    objet = Demicercle
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Donnez un diametre du demi-cercle :"])
        self.ajoute(["Creer une demi-cercle de diametre : [", ("point1", 5), ("point2", 5), "]"], "Entrez les extremites du diametre, dans le sens direct. Exemple : A et B")
        self.finalise()


    
class DialogueParallele(Dialogue):
    objet = Parallele
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer la parallele a :", ("droite", 5)], "Entrez une droite. Exemple : d")
        self.ajoute(["Passant par :", ("point", 5)], "Entrez un point. Exemple : M")
        self.finalise()

    
    
    
class DialoguePerpendiculaire(Dialogue):
    objet = Perpendiculaire
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer la perpendiculaire a :", ("droite", 5)], "Entrez une droite. Exemple : d")
        self.ajoute(["Passant par :", ("point", 5)], "Entrez un point. Exemple : M")
        self.finalise()

    
    
class DialogueMediatrice(Dialogue):
    objet = Mediatrice
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer la mediatrice du segment : [", ("objet1", 5), ("objet2", 5), "]"], "Entrez les extremites du segment. Exemple : A et B")
        self.finalise()


    
class DialogueBissectrice(Dialogue):  
    objet = Bissectrice
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer la bissectrice de l'angle :", ("pt1", 5), ("pt2", 5), ("pt3", 5)], "Entrez le nom de l'angle, nomme par 3 points. Exemple : B A C")
        self.finalise()


class DialogueTangente(Dialogue):  
    objet = Tangente
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer la tangente au cercle :", ("cercle", 5)], "Entrez le nom du cercle. Exemple : Cer")
        self.ajoute(["Passant par :", ("point", 5)], "Entrez un point. Exemple : M")
        self.finalise()


class DialogueInterDroites(Dialogue):
    objet = Intersection_droites
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer le point d'intersection des droites :", ("droite1", 5), "et", ("droite2", 5)], "Entrez les noms des deux droites. Exemple : d1 et d2")
        self.finalise()
    
    
class DialogueInterDroiteCercle(Dialogue):
    objet = Intersection_droite_cercle
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer un point d'intersection entre :", "la droite", ("droite", 5), "et le cercle", ("cercle", 5)], "Entrez les noms dela droite, et du cercle. Exemple : AB et Cer")
        self.finalise()


class DialogueInterCercles(Dialogue): 
    objet = Intersection_cercles
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer un point d'intersection des cercles :", ("cercle1", 5), "et", ("cercle2", 5)], "Entrez les noms des deux cercles. Exemple : C1 et C2")
        self.finalise()




class DialogueGlisseurDroite(Dialogue):
    objet = Glisseur_droite
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer un point sur la droite :", ("droite", 5)], "Entrez le nom de la droite. Exemple : d")
        self.finalise()




class DialogueGlisseurCercle(Dialogue):
    objet = Glisseur_cercle
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer un point sur le cercle :", ("cercle", 5)], "Entrez le nom du cercle. Exemple : Cer")
        self.finalise()




class DialogueGlisseurSegment(Dialogue):
    objet = Glisseur_segment
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer un point sur le segment :", ("segment", 5)], "Entrez le nom du segment. Exemple : AB")
        self.finalise()


class DialoguePolygone(Dialogue):
    objet = Polygone
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer un polygone de sommets :", ("sommets", 10)], "Entrez les sommets du polygone. Exemple : A,B,C,D")
        self.finalise()

    def commande(self): # a cause de l'initialisation speciale de Polygone : __init__(*points)
        return self.champ("nom") + "=Polygone(" + self.champ("sommets") + ")"


class DialogueTriangle(Dialogue):    
    objet = Triangle
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer un triangle de sommets :", ("point1", 5), ",", ("point2", 5), "et", ("point3", 5)], "Entrez le nom des points. Exemple : A , B et C")
        self.finalise()


class DialogueCentre(Dialogue):  
    objet = Centre
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer le centre du cercle :", ("cercle", 5)], "Entrez le nom du cercle. Exemple : Cer")
        self.finalise()


class DialogueOrthocentre(Dialogue):
    objet = Orthocentre
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer l'orthocentre du triangle :", ("triangle", 5)], "Entrez le nom du triangle. Exemple : ABC")
        self.finalise()


class DialogueCentreGravite(Dialogue):  
    objet = Centre_gravite
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer le centre de gravite du triangle :", ("triangle", 5)], "Entrez le nom du triangle. Exemple : ABC")
        self.finalise()


class DialogueCentreCercleCirconscrit(Dialogue):
    objet = Centre_cercle_circonscrit
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer le centre du cercle circonscrit a :", ("triangle", 5)], "Entrez le nom du triangle. Exemple : ABC")
        self.finalise()


class DialogueCentreCercleInscrit(Dialogue):
    objet = Centre_cercle_inscrit
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Creer le centre du cercle inscrit dans :", ("triangle", 5)], "Entrez le nom du triangle. Exemple : ABC")
        self.finalise()


class DialogueTexte(Dialogue):
    objet = Texte
    def __init__(self, parent):
        Dialogue.__init__(self, parent)
        self.ajoute(["Texte :", ("texte", 10)], "Entrez ici le texte. Exemple : Bonjour!")
        self.ajoute(["Coordonnees du texte :"], "Entrez les coordonnees de votre nouveau texte.")
        self.ajoute(["Abscisse :", ("x", 10)], "Entrez ici l'abscisse du texte. Exemple : 3.25")
        self.ajoute(["Ordonnee :", ("y", 10)], "Entrez ici l'ordonnee du texte. Exemple : -5")
        self.finalise()

    def commande(self): # Le champ texte doit etre converti...
        return "%s=Texte(%s,%s,%s)" %(self.champ("nom"), repr(self.champ("texte")), (self.champ("x") or "None"), (self.champ("y") or "None"))


###############################################################################

class DialogueFenetre(Dialogue):
    def __init__(self, parent):
        Dialogue.__init__(self, parent, "Regler la fenetre d'affichage")
        fen = self.parent.actuel().canvas.fen()
        self.ajoute(["Entrez les valeurs extremales de la fenetre d'affichage."])
        self.ajoute(["Xmin :", ("xmin", 10, 1, round(fen[0], 4))], "Abscisse minimale. Exemple : -5")
        self.ajoute(["Xmax :", ("xmax", 10, 1, round(fen[1], 4))], "Abscisse maximale. Exemple : 5")
        self.ajoute(["Ymin :", ("ymin", 10, 1, round(fen[2], 4))], "Ordonnee minimale. Exemple : -5")
        self.ajoute(["Ymax :", ("ymax", 10, 1, round(fen[3], 4))], "Ordonnee maximale. Exemple : 5")
        self.finalise()

    def commande(self):
        return "set_fenetre(" + self.parametres() + ")"
        


################################################################################

class SupprimerObjet(wx.lib.dialogs.MultipleChoiceDialog):
    def __init__(self, parent):
        liste = parent.actuel().actuelle.inventaire()
        wx.lib.dialogs.MultipleChoiceDialog.__init__(self, parent, "Selectionnez les objets a supprimer", "Supprimer", liste, size=(250,400))


class EditerObjet(wx.lib.dialogs.MultipleChoiceDialog):
    def __init__(self, parent):
        liste = parent.actuel().actuelle.inventaire()
        wx.lib.dialogs.MultipleChoiceDialog.__init__(self, parent, "Selectionnez les objets a editer", "Editer", liste, size=(250,400))

