# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------##
#              WxGeometrie               #
#          fenetre principale            #
##--------------------------------------##
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


# importation des librairies specifiques et des composantes du programme
from GUI import *
from API import *

######################################################


class WxGeometrie(wx.Frame):
    def __init__(self, app):
        wx.Frame.__init__(self, parent = None, title = "WxGeometrie", pos=wx.DefaultPosition, size=wx.Size(*param.dimensions_fenetre), style=wx.DEFAULT_FRAME_STYLE)

        self.application = app # pour acceder a l'application en interne
            
        # Repertoires par defaut
        self.repertoire = param.repertoire # memorise le repertoire de lancement
        self.rep_save = param.rep_save # repertoire ou on sauve les fichiers
        self.rep_open = param.rep_open # repertoire ou on ouvre les fichiers
        self.rep_export = param.rep_export # repertoire ou on exporte les fichiers

        self.SetIcon(wx.Icon("images" + os.sep + "icone.ico", wx.BITMAP_TYPE_ICO))
        
        
        # Barre d'outils
        tb = self.CreateToolBar( wx.TB_HORIZONTAL
                                 | wx.NO_BORDER
                                 | wx.TB_FLAT
                                 | wx.TB_TEXT )
        tb.AddSeparator()
        tb.AddControl(wx.StaticText(tb, -1, "Ligne de commande :"))
        tb.AddSeparator()
        self.commande = wx.TextCtrl(tb, -1, "", size=(300, -1))
        tb.AddControl(self.commande)
        self.commande.Bind(wx.EVT_CHAR, self.EvtChar)
        
        tb.AddSeparator()
        tb.AddSeparator()
        tb.AddSeparator()
        tb.AddSeparator()
        self.b = wx.Button(tb, -1, "Tout recharger")
        tb.AddControl(self.b)
        self.Bind(wx.EVT_BUTTON, self.recharger, self.b) # recharge les librairies et les figures (permet des modifications en temps reel)
        #if param.securite: self.b.Disable()
        self.b.Disable()  # IL FAUDRA REPENSER CE BOUTON !!


        # Final thing to do for a toolbar is call the Realize() method. This
        # causes it to render (more or less, that is).
        tb.Realize()
        
        
        # Barre de statut
        self.barre = wx.StatusBar(self, -1)
        self.barre.SetFieldsCount(2)
        self.barre.SetStatusWidths([-3, -2])
        self.barre.SetStatusText("  Bienvenue !", 1)
        self.barre.SetStatusText("Geometrie version " + param.version, 0)
        self.SetStatusBar(self.barre)
        
        # Creation des onglets et de leur contenu
        self.onglets = Onglets(self, -1)
        
        # Creation des menus
        #self.menus = [onglet.menu(self) for onglet in self.onglets.liste_onglets()]
        #[GeoMenuBar(self), TraMenuBar(self), CalcMenuBar(self)]
        

        #affiche le menu
        #self.changer_menu(0)
        
        # historique de la toolbar
        self.historique = []
        self.position = -1
        
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        
        #self.DragAcceptFiles(True)
        #self.Bind(wx.EVT_DROP_FILES, self.OnDrop)

#####################################################################





    def recharger(self, event):
        reload(param)
        
        self.onglets.actuel().historique.rafraichir()
        self.barre.SetStatusText("Parametres et figure recharges.", 0)

    def message(self, texte, lieu = 0):
        self.barre.SetStatusText(texte, lieu)
        
    def titre(self, texte = ""):
        if texte:
            self.SetTitle("WxGeometrie - " + texte)
        else:
            self.SetTitle("WxGeometrie")




    def EvtChar(self,event):    # gere la ligne de commandes
        commande = self.commande.GetValue()
        code = event.GetKeyCode()

        if code == 13:
            try:
                self.onglets.actuel().commande.executer(commande)
                self.barre.SetStatusText("Commande executee.", 0)
                self.historique.append(commande)
                self.position = len(self.historique) - 1
                self.commande.Clear()
            except:
                self.barre.SetStatusText("Commande incorrecte.", 0)
                if param.debug:
                    raise

        elif code == 317:   # on remonte dans l'historique des commandes
            if self.position >= 0:
                self.commande.SetValue(self.historique[self.position])
                self.position = self.position - 1

        elif code == 319:   # on redescend dans l'historique vers les commandes recentes
            if self.position < len(self.historique) - 1:
                if self.position == (len(self.historique) - 2):
                    self.commande.Clear()
                else:
                    self.commande.SetValue(self.historique[self.position + 2])
                self.position = self.position + 1
        else:
            event.Skip()


    def OnClose(self, event):
        if param.confirmer_quitter:
            panel = self.onglets.actuel()
            if hasattr(panel, "canvas") and hasattr(panel.canvas, "Freeze"):
                panel.canvas.Freeze()
            dlg = wx.MessageDialog(self, 'Voulez-vous quitter WxGeometrie ?', 'Quitter WxGeometrie ?', wx.YES_NO | wx.ICON_QUESTION)
            reponse = dlg.ShowModal()
            if hasattr(panel, "canvas") and hasattr(panel.canvas, "Thaw"):
                panel.canvas.Thaw()
            dlg.Destroy()
            if reponse == wx.ID_NO: return
        if param.enregistrer_messages and param.debug:
            filename = param.emplacement_log + os.sep + "messages.log"
            f = open(filename, "w")
            if self.application.stdioWin and self.application.stdioWin.text:
                f.write(self.application.stdioWin.text.GetValue()) # on copie les messages (erreurs...) dans messages.log
            f.close()
        if self.application.stdioWin:   # fermeture de la fenetre de messages
            self.application.stdioWin.close()

        for onglet in self.onglets.liste_onglets():
            try:
                if isinstance(onglet, Panel_API_graphique):
                    if param.historique_log:    onglet.historique.stocker()
                    onglet.fermer_feuilles()

            except:
                print_error()
                debug("Fermeture incorrecte de l'onglet : ", str(onglet))
        event.Skip()
