# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------##
#                  Onglets               #
##--------------------------------------##
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# from LIB import *
import modules
from GUI import *
from API import *



class Onglets(wx.Notebook):
    def __init__(self, parent, id):
        style = wx.NB_TOP
        self.parent = parent
#         if wx.Platform == "__WXMAC__":
#             style = 0
        wx.Notebook.__init__(self, parent, id, style=style)


        ###############################
        ###############################
        # Creation de fonctions associees aux entrees du menu "Creer"
        import GUI.dialogues_geometrie as DG
        self.creer = {}
        dialogues = [(nom[8:], DG.__dict__[nom]) for nom in DG.__dict__.keys() if nom.startswith("Dialogue")]
        for dialogue in dialogues:
            def f(event = None, self = self, dialogue = dialogue[1]):
                self.creer_objet(dialogue)
            self.creer[dialogue[0]] = f
        del DG
        ###############################
        ###############################


        self.onglets = [] # liste des onglets
        
        # Ajoute les differentes composantes :
        for nom in modules.liste():
            nom = nom.lower() # le nom du module est toujours en minuscules.
            module = modules.liste(nom)
            panel = module.__panel__(self)  # ex: panel = Geometre(self)
            setattr(self, nom, panel)       #     self.geometre = panel
            menu = module.__menu__(panel)   #     menu = GeometreMenuBar(panel)
            self.nouvel_onglet(panel)

        
        self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.changer) # adaptation du titre de l'application et du menu.
        self.actualise_onglet(self.onglets[0]) # affiche le titre et le menu du 1er onglet
        



    def nouvel_onglet(self, onglet):
        self.onglets.append(onglet)
        self.AddPage(onglet, onglet.__titre__)
        
    
    def changer(self, event):
        event.Skip()
        self.actualise_onglet(self.onglets[event.GetSelection()])   # onglet selectionne
        
        
    
    def actualise_onglet(self, onglet):
        self.parent.SetMenuBar(onglet.menu) # change le menu de la fenetre
        onglet.changer_titre() # change le titre de la fenetre


    def onglet_actuel(self): # le nom de l'onglet actuel
        return param.modules[self.GetSelection()]
    
    
    def onglet(self, nom):
        "nom : nom ou numero de l'onglet."
        if type(nom) == int:
            return self.onglets[nom]
        nom = nom.lower()
        if not hasattr(self, nom):
            return
        if getattr(self, nom) in self.onglets:
            return getattr(self, nom)

    def actuel(self): # l'onglet couramment ouvert
        return self.onglets[self.GetSelection()]
        
    
    def onglet_suivant(self, event):
        self.AdvanceSelection() 

    def liste_onglets(self):
        return self.onglets
    
    def changer_onglet(self, onglet):
        "onglet : l'onglet proprement dit, ou son nom, ou son numero."
        if type(onglet) in (str, unicode):
            onglet = self.onglets.index(getattr(self, nom.lower()))
        elif type(onglet) not in (int, long):
            onglet = self.onglets.index(onglet)
        self.SetSelection(onglet)
            
    
    
    
#####################################################################
#####################################################################




    def NewFile(self, event = None):
        self.actuel().nouveau()


    def SaveFile(self, event = None):
        dlg = wx.FileDialog(
            self, message="Enregistrer sous ...", defaultDir=self.parent.rep_save, 
            defaultFile="", wildcard="Fichiers Geometrie (*.geo)|*.geo|Tous les fichiers (*.*)|*.*", style=wx.SAVE | wx.OVERWRITE_PROMPT | wx.CHANGE_DIR
            )
        # dlg.SetFilterIndex(0)   # inutile (par defaut, la valeur est 0).
        
        if dlg.ShowModal() == wx.ID_OK:
            self.parent.rep_save = os.getcwd()
            os.chdir(self.parent.repertoire)
            path = dlg.GetPath()
            self.actuel().sauvegarder(path)
        
        dlg.Destroy()

        
    def OpenFile(self, event = None, detecter_module = True):
        # Attention :
        # "This dialog is set up to change the current working directory to the path chosen."
        # on enregistre donc le nouveau getcwd() - "repertoire_ouverture_fichiers = os.getcwd()", pour restaurer ensuite l'ancien
        dlg = wx.FileDialog(
            self, message="Choisissez un fichier", defaultDir=self.parent.rep_open, 
            defaultFile="", wildcard="Fichiers Geometrie (*.geo)|*.geo|Tous les fichiers (*.*)|*.*", style=wx.OPEN | wx.MULTIPLE | wx.CHANGE_DIR
            )
            
        # Show the dialog and retrieve the user response. If it is the OK response, 
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            self.parent.rep_open = os.getcwd()
            os.chdir(self.parent.repertoire)
            # This returns a Python list of files that were selected.
            paths = dlg.GetPaths()
            for path in paths:
                if detecter_module: # on detecte dans quel onglet le fichier doit etre ouvert
                    self.ouvrir(path)
                else: # on l'ouvre dans l'onglet actuellement ouvert
                    self.actuel().ouvrir(path) # ouvre tous les fichiers selectionnes

        dlg.Destroy()



    def OpenFileHere(self, event = None):
        """Ouvrir le fichier dans le module courant.
        Par defaut, sinon, le fichier est ouvert dans le module qui l'a cree."""
        self.OpenFile(detecter_module = False)
        
        
    def ouvrir(self, path):
        "Ouvre un fichier dans l'onglet adquat."
        fgeo, message = ouvrir_fichierGEO(path)
        module = self.onglet(fgeo.module)
        self.parent.message(message)
        if module is not None:
            self.changer_onglet(module) # affiche cet onglet
            module._ouvrir(fgeo) # charge le fichier dans le bon onglet
            #print_error()
        else:
            self.parent.message("Impossible de determiner le module adequat.")
        
    
    def ExportFile(self, event = None, lieu = None, sauvegarde = False):
        """Le parametre sauvegarde indique qu'il faut faire une sauvegarde simultanee.
        (attention, on ne verifie pas que le fichier .geo n'existe pas !).
        """
        lieu = lieu or self.onglet_actuel() # par defaut, le lieu est l'onglet courant
        """Saves the file to the type specified in the extension. If no file
        name is specified a dialog box is provided.  Returns True if sucessful,
        otherwise False.
        
        .png  Save a Portable Network Graphics file.
        .svg  Save a Scalable Vector Graphics file.
        .eps  Save an Encapsulated PostScript file.
        """
        dlg1 = wx.FileDialog(
                self, 
                "Exporter l'image en png, bmp, svg, ps ou eps", self.parent.rep_export, "",
                "PNG files (*.png)|*.png|SVG file (*.svg)|*.svg|PS files (*.ps)|*.ps|EPS files (*.eps)|*.eps",
                wx.SAVE | wx.OVERWRITE_PROMPT | wx.CHANGE_DIR
                )
        try:
            while 1:
                if dlg1.ShowModal() == wx.ID_OK:
                    self.parent.rep_export = os.getcwd()
                    if sauvegarde:
                        self.parent.rep_save = self.parent.rep_export
                    os.chdir(self.parent.repertoire)
                    fileName = dlg1.GetPath()
                    # Check for proper exension
                    if string.lower(fileName[-3:]) not in ['png','svg','eps','.ps']:
                        dlg2 = wx.MessageDialog(self, "L'extension du fichier\n"
                        'doit etre\n'
                        'png, svg, ps ou eps',
                          'Nom de fichier incorrect', wx.OK | wx.ICON_ERROR)
                        try:
                            dlg2.ShowModal()
                        finally:
                            dlg2.Destroy()
                    else:
                        break # now save file
                else: # exit without saving
                    return False
        finally:
            dlg1.Destroy()

        # Save Bitmap
        if sauvegarde:
            self.onglet(lieu).sauvegarder(mreplace(fileName, (".png", ".eps", ".svg", ".ps"), ".geo"))
        return self.onglet(lieu).canvas.exporter(fileName)

        
    def ExportAndSaveFile(self, event = None, lieu = None):
        self.ExportFile(event = event, lieu = lieu, sauvegarde = True)
        
        
    def CloseFile(self, event = None):
        self.actuel().fermer()
    
    
    def PageSetup(self, event = None):
        pass
    def Printout(self, event = None):
        pass


    
        
    def CloseWindow(self, event):
        self.parent.Close(True)




            
    def supprimer(self, event = None):
        if self.actuel().actuelle: # s'il y a une feuille de travail ouverte
            dlg = SupprimerObjet(self)
            if (dlg.ShowModal() == wx.ID_OK):
                for selection in dlg.GetValueString():
                    try:  # il est normal que des erreurs soient renvoyees si un objet est deja supprime.
                        objet = selection.split()[0]
                        self.actuel().commande.executer("del " + objet)
                    except AttributeError:
                        if param.debug:
                            print_error()
                    except: # fusionner ?
                        if param.debug: print_error()
            dlg.Destroy()


    def editer(self, event = None):
        from GUI.proprietes_objets import Proprietes #  deplacer !!
        feuille = self.actuel().actuelle
        if feuille:
            objets = []
            dlg = EditerObjet(self)
            val = dlg.ShowModal()
            if val == wx.ID_OK:
                objets = [feuille.objets[selection.split()[0]] for selection in dlg.GetValueString()]
            dlg.Destroy()
            
            if objets:
                win = Proprietes(self.actuel().canvas, objets)
                win.Show(True)
                def fermeture(event, win = win):
                    win.Unbind(wx.EVT_CLOSE)
                    win.Close()
                    self.editer()
                win.Bind(wx.EVT_CLOSE, fermeture)
    
    
    def creer_objet(self, classe):
        
        dl = classe(self)
 
        # this does not return until the dialog is closed.
        while 1:
            resultat = dl.affiche()
            if resultat == wx.ID_OK:
                try:
                    self.actuel().commande.executer(dl.commande())
                    break
                except:
                    if param.debug:
                        print_error()
                        break
                    self.actuel().canvas.message('Erreur - parametres incorrects.')
            else:
                break
        dl.Destroy()

    
    def Securite(self, event):
        self.actuel().commande.securite = not self.actuel().commande.securite
        if param.securite:  self.parent.b.Disable()
        else:  self.parent.b.Enable()
        

    def Debug(self, event):
        param.debug = not param.debug
        app = self.parent.application
        if param.debug:
            app.RedirectStdio(None)     # redirection vers une fenetre de messages
        else:
            app.RestoreStdio()
            if app.stdioWin:
                app.stdioWin.close()
    
    
    
    def Histo(self, event):
        h = FenHistorique(self)
        h.SetSize((400, 500))
        h.CenterOnParent(wx.BOTH)
        h.Show(True)


    def Aide(self,event):
        aide = Help(self, "doc" + os.sep + "help.htm")
        aide.Show(True)



    def Notes(self,event):
        f = open("doc" + os.sep + "notes.txt", "r")
        msg = f.read()
        f.close()
        msg = msg.replace("WxGeometrie", "WxGeometrie version " + param.version, 1)
        dlg = wx.lib.dialogs.ScrolledMessageDialog(self, msg, "Notes de version")
        dlg.ShowModal()
    
    
    def About(self, event):
        dialog = About(self)
        btn = dialog.ShowModal()
        dialog.Destroy()

