# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                 Fenetres                              #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



# importation des librairies classiques
from LIB import *

import  wx.lib.colourselect as  __csel__
from matplotlib.colors import colorConverter as __colorConverter__


class ProprietesAffichage(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        self.parent = parent
        self.objets = parent.objets
        self.sizer = wx.BoxSizer(wx.VERTICAL)

        self.changements = {} # ce dictionnaire contiendra tous les styles modifis
        encadre = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Mode d'affichage"), wx.HORIZONTAL)
        objets = [objet for objet in self.objets if objet.style("fixe") is not None]
        if objets:
            cb1 = wx.CheckBox(self, -1, "Objet fixe", style = wx.CHK_3STATE)
            cb1.Bind(wx.EVT_CHECKBOX, self.EvtFixe)
            encadre.Add(cb1, 0, wx.ALL, 5)
            fixe = [objet.style("fixe") is True for objet in objets]
            if True not in fixe:
                etat = wx.CHK_UNCHECKED
            elif False not in fixe:
                etat = wx.CHK_CHECKED
            else:
                etat = wx.CHK_UNDETERMINED
            cb1.SetValue(etat)
            
            
            
        objets = [objet for objet in self.objets if objet.style("visible") is not None]
        if objets:
            cb2 = wx.CheckBox(self, -1, "Objet visible", style = wx.CHK_3STATE)
            cb2.Bind(wx.EVT_CHECKBOX, self.EvtVisible)
            encadre.Add(cb2, 0, wx.ALL, 5)
            style = [objet.style("visible") is True for objet in objets]
            if True not in style:
                etat = wx.CHK_UNCHECKED
            elif False not in style:
                etat = wx.CHK_CHECKED
            else:
                etat = wx.CHK_UNDETERMINED
            cb2.SetValue(etat)
            
        
        encadre1 = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Etiquette"), wx.VERTICAL)
        if len(self.objets) == 1:
            etiquette = wx.TextCtrl(self, value = self.objets[0].style("label"), size=wx.Size(200, -1))
            self.Bind(wx.EVT_TEXT, self.EvtEtiquette, etiquette)
            encadre1.Add(etiquette, 0, wx.ALL, 5)
        
        objets = [objet for objet in self.objets if objet.style("legende") is not None]
        if objets:
            leg = objets[0].style("legende")
            legende = wx.BoxSizer(wx.HORIZONTAL)
            self.radio_nom = wx.RadioButton(self, -1, "Nom", style = wx.RB_GROUP)
            self.radio_nom.SetValue(0)
            self.radio_etiquette = wx.RadioButton(self, -1, "Etiquette")
            self.radio_etiquette.SetValue(0)
            self.radio_aucun = wx.RadioButton(self, -1, "Aucun")
            self.radio_aucun.SetValue(0)
            if False not in [objet.style("legende") == leg for objet in objets]:
                if leg == param.NOM:
                    self.radio_nom.SetValue(1)
                elif leg == param.LABEL:
                    self.radio_etiquette.SetValue(1)
                elif leg == param.RIEN:
                    self.radio_aucun.SetValue(1)

            self.Bind(wx.EVT_RADIOBUTTON, self.EvtLegende, self.radio_nom)
            self.Bind(wx.EVT_RADIOBUTTON, self.EvtLegende, self.radio_etiquette)
            self.Bind(wx.EVT_RADIOBUTTON, self.EvtLegende, self.radio_aucun)
            legende.Add(self.radio_nom, 0, wx.ALL, 5)
            legende.Add(self.radio_etiquette, 0, wx.ALL, 5)
            legende.Add(self.radio_aucun, 0, wx.ALL, 5)
            encadre1.Add(wx.StaticText(self, -1, "Afficher : "), 0, wx.ALL,5)
            encadre1.Add(legende, 0, wx.ALL, 5)

        
                
                
        encadre2 = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Styles"), wx.VERTICAL)

        
        objets = [objet for objet in self.objets if objet.style("style") is not None]
        # on ne peut regler les styles simultanement que pour des objets de meme categorie
        categorie = objets and objets[0].style("categorie") or None
        
        
        if objets and categorie and (False not in [objet.style("categorie") == categorie for objet in objets]): 
            choix = wx.BoxSizer(wx.HORIZONTAL)
            choix.Add(wx.StaticText(self, -1, "Style de l'objet : "), 0, wx.ALL,5)
            
            #categorie = objets[0].style("categorie") or "lignes"
            self.liste_styles = getattr(param, "styles_de_" + categorie, [])
            self.style = wx.Choice(self, -1, (100, 50), choices = self.liste_styles)
            self.Bind(wx.EVT_CHOICE, self.EvtChoice, self.style)

            style = objets[0].style("style")
            if style in self.liste_styles and (False not in [objet.style("style") == style for objet in objets]):
                self.style.SetSelection(self.liste_styles.index(style)) # on slectionne le style actuel
            choix.Add(self.style, 0, wx.ALL, 5)
            encadre2.Add(choix, 0, wx.ALL, 5)
         
                
        objets = [objet for objet in self.objets if objet.style("famille") is not None]
        categorie = objets and objets[0].style("categorie") or None

        if objets and categorie and (False not in [objet.style("categorie") == categorie for objet in objets]):
            choix = wx.BoxSizer(wx.HORIZONTAL)
            choix.Add(wx.StaticText(self, -1, "Police : "), 0, wx.ALL,5)
            
            #categorie = self.objet.style("categorie") or "lignes"
            self.liste_familles = getattr(param, "familles_de_" + categorie, [])
            self.famille = wx.Choice(self, -1, (100, 50), choices = self.liste_familles)
            self.Bind(wx.EVT_CHOICE, self.EvtFamille, self.famille)

            famille = objets[0].style("famille")
            if famille in self.liste_familles and (False not in [objet.style("famille") == famille for objet in objets]):
                self.famille.SetSelection(self.liste_familles.index(famille)) # on slectionne la famille actuelle

            choix.Add(self.famille, 0, wx.ALL, 5)
            encadre2.Add(choix, 0, wx.ALL, 5)


        objets = [objet for objet in self.objets if objet.style("couleur") is not None]
        if objets:
            couleur = objets[0].style("couleur")
            choix = wx.BoxSizer(wx.HORIZONTAL)
            choix.Add(wx.StaticText(self, -1, "Couleur de l'objet : "), 0, wx.ALL,5)
            if False not in [objet.style("couleur") == couleur for objet in objets]:
                couleur = __colorConverter__.to_rgb(couleur)
                couleur = tuple(int(255*i) for i in couleur) # conversion du format matplotlib au format wx
            else:
                couleur = self.GetBackgroundColour()
            b = __csel__.ColourSelect(self, -1, colour = couleur)
            b.Bind(__csel__.EVT_COLOURSELECT, self.OnSelectColour)
            choix.Add(b, 0, wx.ALL, 5)
            encadre2.Add(choix, 0, wx.ALL, 5)
            
            
        objets = [objet for objet in self.objets if objet.style("epaisseur") is not None]
        if objets:
            epaiss = objets[0].style("epaisseur")
            epaisseur = wx.BoxSizer(wx.HORIZONTAL)
            epaisseur.Add(wx.StaticText(self, -1, "Epaisseur (en 10e de pixels) : "), 0, wx.ALL,5)
            self.epaisseur = wx.SpinCtrl(self, -1, "", (30, 50))
            self.epaisseur.SetRange(1,10000)
            if False not in [objet.style("epaisseur") == epaiss for objet in objets]:
                self.epaisseur.SetValue(10*epaiss)
            else:
                self.epaisseur.SetValueString("")
            self.Bind(wx.EVT_TEXT, self.EvtEpaisseur, self.epaisseur)
            epaisseur.Add(self.epaisseur, 0, wx.ALL, 5)
            encadre2.Add(epaisseur, 0, wx.ALL, 5)


        objets = [objet for objet in self.objets if objet.style("taille") is not None]
        if objets:
            tail = objets[0].style("taille")
            taille = wx.BoxSizer(wx.HORIZONTAL)
            taille.Add(wx.StaticText(self, -1, "Taille (en 10e de pixels) : "), 0, wx.ALL,5)
            self.taille = wx.SpinCtrl(self, -1, "", (30, 50))
            self.taille.SetRange(1,10000)
            if False not in [objet.style("taille") == tail for objet in objets]:
                self.taille.SetValue(10*tail)
            else:
                self.taille.SetValueString("")
            self.Bind(wx.EVT_TEXT, self.EvtTaille, self.taille)
            taille.Add(self.taille, 0, wx.ALL, 5)
            encadre2.Add(taille, 0, wx.ALL, 5)
        
        
        objets = [objet for objet in self.objets if objet.style("angle") is not None]
        if objets:
            ang = objets[0].style("angle")
            angle = wx.BoxSizer(wx.HORIZONTAL)
            angle.Add(wx.StaticText(self, -1, "angle (en degre) : "), 0, wx.ALL,5)
            self.angle = wx.SpinCtrl(self, -1, "", (30, 50))
            self.angle.SetRange(0, 360)
            if False not in [objet.style("angle") == ang for objet in objets]:
                self.angle.SetValue(ang)
            else:
                self.angle.SetValueString("")
            self.Bind(wx.EVT_TEXT, self.EvtAngle, self.angle)
            angle.Add(self.angle, 0, wx.ALL, 5)
            encadre2.Add(angle, 0, wx.ALL, 5)
        
        boutons = wx.BoxSizer(wx.HORIZONTAL)
        ok = wx.Button(self, wx.ID_OK)
        ok.Bind(wx.EVT_BUTTON, self.EvtOk)
        appliquer = wx.Button(self, label = "Appliquer")
        appliquer.Bind(wx.EVT_BUTTON, self.EvtAppliquer)
        supprimer = wx.Button(self, label = "Supprimer")
        supprimer.Bind(wx.EVT_BUTTON, self.EvtSupprimer)
        annuler = wx.Button(self, label = "Annuler")
        annuler.Bind(wx.EVT_BUTTON, self.EvtAnnuler)
        
        boutons.Add(ok, 0, wx.ALL, 5)
        boutons.Add(appliquer, 0, wx.ALL, 5)
        boutons.Add(supprimer, 0, wx.ALL, 5)
        boutons.Add(annuler, 0, wx.ALL, 5)
        
        if encadre.GetChildren(): # ne pas afficher un cadre vide !
            self.sizer.Add(encadre, 0, wx.ALL, 5)
        else:
            encadre.GetStaticBox().Destroy()
        if encadre1.GetChildren():
            self.sizer.Add(encadre1, 0, wx.ALL, 5)
        else:
            encadre1.GetStaticBox().Destroy()
        if encadre2.GetChildren():
            self.sizer.Add(encadre2, 0, wx.ALL, 5)
        else:
            encadre2.GetStaticBox().Destroy()
        self.sizer.Add(boutons, 0, wx.ALL, 5)
        self.SetSizerAndFit(self.sizer)
        self.parent.parent.dim1 = self.sizer.CalcMin().Get()
        
    def EvtLegende(self, event):
        radio = event.GetEventObject()
        if radio is self.radio_nom:
            self.changements["legende"] = param.NOM
        elif radio is self.radio_etiquette:
            self.changements["legende"] = param.LABEL
        else:
            self.changements["legende"] = param.RIEN
    
    def EvtFixe(self, event):
        self.changements["fixe"] = event.IsChecked()
    
    def EvtVisible(self, event):
        self.changements["visible"] = event.IsChecked()
    
    def EvtEtiquette(self, event):
        self.changements["label"] = event.GetString()
        
    def OnSelectColour(self, event):
        couleur = tuple(i/255 for i in event.GetValue().Get()) # conversion du format wx au format matplotlib
        self.changements["couleur"] = couleur
        
    def EvtChoice(self, event):
        self.changements["style"] = self.liste_styles[event.GetSelection()]
        
    def EvtFamille(self, event):
        self.changements["famille"] = self.liste_familles[event.GetSelection()]

    def EvtOk(self, event):
        self.EvtAppliquer(event)
        self.EvtAnnuler(event)
        
    def EvtAppliquer(self, event):
        self.parent.parent.parent.geler_affichage = True
        try:
            for objet in self.objets:
                changements = self.changements.copy()
                for key in changements:
                    if objet.style(key) is None: # le style n'a pas de sens pour l'objet
                        changements.pop(key)
                self.parent.parent.parent.commande.executer("%s.modifier(**%s)" %(objet.nom(), changements))
        finally:
            self.parent.parent.parent.geler_affichage = False
            self.parent.parent.parent.actualiser()
        
    def EvtSupprimer(self, event):
        for objet in self.objets:
            self.parent.parent.parent.commande.executer("%s.supprimer()" %objet.nom())
        self.EvtAnnuler(event)
    
    def EvtAnnuler(self, event):
        self.parent.parent.parent.SetFocus()
        self.parent.parent.Close() # fermeture de la frame
        
    def EvtEpaisseur(self, event):
        self.changements["epaisseur"] = self.epaisseur.GetValue()/10

    def EvtTaille(self, event):
        self.changements["taille"] = self.taille.GetValue()/10

    def EvtAngle(self, event):
        self.changements["angle"] = self.angle.GetValue()
        


class ProprietesInfos(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        self.parent = parent
        self.objets = parent.objets
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        if len(self.objets) == 1:
            self.objet = self.objets[0]
        else:
            self.objet = None # cela n'a pas vraiment de sens d'afficher une longueur pour 3 segments differents par exemple...
        
        def formater(valeur):
            if type(valeur) in (str, unicode):
                return  valeur
            return str(nchiffres(valeur, param.chiffres_significatifs))
        
        def nouveau_controle(valeur = "", editable = False):
            t = wx.TextCtrl(self, value = formater(valeur), size=wx.Size(300, -1))
            t.SetEditable(editable)
            return t
            
        if hasattr(self.objet, "aire"):
            #aire = wx.BoxSizer(wx.HORIZONTAL)
            self.sizer.Add(wx.StaticText(self, -1, "Aire : "), 0, wx.ALL, 5)
            aire_text = nouveau_controle(self.objet.aire())
            #self.Bind(wx.EVT_TEXT, self.EvtAire, aire_text)
            self.sizer.Add(aire_text, 0, wx.ALL, 5)
            #self.sizer.Add(aire, 0, wx.ALL, 5)

        if hasattr(self.objet, "centre"):
            #centre = wx.BoxSizer(wx.HORIZONTAL)
            self.sizer.Add(wx.StaticText(self, -1, "Centre de coordonnees : "), 0, wx.ALL, 5)
            centre = self.objet.centre()
            if centre is not None:
                x, y = centre
                valeur = formater(x) + " ; " + formater(y)
            else:
                valeur = "L'objet n'est pas defini."
            centre_text = nouveau_controle(valeur)
            #self.Bind(wx.EVT_TEXT, self.EvtCentre, centre_text)
            self.sizer.Add(centre_text, 0, wx.ALL, 5)
            #self.sizer.Add(centre, 0, wx.ALL, 5)

        if hasattr(self.objet, "_coordonnees"):
            #centre = wx.BoxSizer(wx.HORIZONTAL)
            self.sizer.Add(wx.StaticText(self, -1, "Coordonnees : "), 0, wx.ALL, 5)
            coo = self.objet.coordonnees()
            if coo is not None:
                x, y = coo
                valeur = formater(x) + " ; " + formater(y)
            else:
                valeur = "L'objet n'est pas defini."
            coo_text = nouveau_controle(valeur)
            #self.Bind(wx.EVT_TEXT, self.EvtCentre, centre_text)
            self.sizer.Add(coo_text, 0, wx.ALL, 5)
            #self.sizer.Add(centre, 0, wx.ALL, 5)

        if hasattr(self.objet, "rayon"):
            #rayon = wx.BoxSizer(wx.HORIZONTAL)
            self.sizer.Add(wx.StaticText(self, -1, "Rayon : "), 0, wx.ALL, 5)
            rayon_text = nouveau_controle(self.objet.rayon())
            #self.Bind(wx.EVT_TEXT, self.EvtRayon, rayon_text)
            self.sizer.Add(rayon_text, 0, wx.ALL, 5)
            #self.sizer.Add(rayon, 0, wx.ALL, 5)
        
        if hasattr(self.objet, "longueur"):
            #longueur = wx.BoxSizer(wx.HORIZONTAL)
            self.sizer.Add(wx.StaticText(self, -1, "Longueur : "), 0, wx.ALL, 5)
            longueur_text = nouveau_controle(self.objet.longueur())
            #self.Bind(wx.EVT_TEXT, self.EvtLongueur, longueur_text)
            self.sizer.Add(longueur_text, 0, wx.ALL, 5)
            #self.sizer.Add(longueur, 0, wx.ALL, 5)
        
        if hasattr(self.objet, "perimetre"):
            self.sizer.Add(wx.StaticText(self, -1, "Perimetre : "), 0, wx.ALL, 5)
            perimetre_text = nouveau_controle(self.objet.perimetre())
            self.sizer.Add(perimetre_text, 0, wx.ALL, 5)

        if hasattr(self.objet, "norme"):
            #norme = wx.BoxSizer(wx.HORIZONTAL)
            self.sizer.Add(wx.StaticText(self, -1, "Norme : "), 0, wx.ALL, 5)
            norme_text = nouveau_controle(self.objet.norme())
            #self.Bind(wx.EVT_TEXT, self.EvtNorme, norme_text)
            self.sizer.Add(norme_text, 0, wx.ALL, 5)
            #self.sizer.Add(norme, 0, wx.ALL, 5)

        if hasattr(self.objet, "mise_en_equation"):
            #mise_en_equation = wx.BoxSizer(wx.HORIZONTAL)
            self.sizer.Add(wx.StaticText(self, -1, "Equation cartesienne : "), 0, wx.ALL, 5)
            mise_en_equation_text = nouveau_controle(self.objet.mise_en_equation())
            #self.Bind(wx.EVT_TEXT, self.EvtMise_en_equation, mise_en_equation_text)
            self.sizer.Add(mise_en_equation_text, 0, wx.ALL, 5)
            #self.sizer.Add(mise_en_equation, 0, wx.ALL, 5)
        
        
        
        self.SetSizerAndFit(self.sizer)
        self.parent.parent.dim2 = self.sizer.CalcMin().Get()




class ProprietesAvance(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        self.parent = parent
        self.objets = parent.objets
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        
        self.SetSizerAndFit(self.sizer)
        self.parent.parent.dim3 = self.sizer.CalcMin().Get()



class OngletsProprietes(wx.Notebook):
    def __init__(self, parent):
        style = wx.NB_TOP
        self.parent = parent
        self.objets = parent.objets 
        wx.Notebook.__init__(self, parent)
        self.affichage = ProprietesAffichage(self)
        self.AddPage(self.affichage, "Affichage")
        self.infos = ProprietesInfos(self)
        self.AddPage(self.infos, "Informations")
        self.avance = ProprietesAvance(self)
        self.AddPage(self.avance, "Avance")
        











class Proprietes(wx.MiniFrame):
        def __init__(self, parent, objets):
            self.objets = objets
            if len(objets) == 0:
                self.Close()
            if len(objets) == 1:
                titre = "de l'objet " + objets[0].nom()
            else:
                titre = "des objets"
            wx.MiniFrame.__init__(self, parent, -1, "Proprietes " + titre, style=wx.DEFAULT_FRAME_STYLE | wx.TINY_CAPTION_HORIZ)
            self.parent = parent
            self.onglets = OngletsProprietes(self)
            self.SetSize(wx.Size(*(max(dimensions) + 50 for dimensions in zip(self.dim1, self.dim2, self.dim3))))



"""





















def OnButton(self, evt):
    dlg = wx.ColourDialog(self)

    # Ensure the full colour dialog is displayed, 
    # not the abbreviated version.
    dlg.GetColourData().SetChooseFull(True)

    if dlg.ShowModal() == wx.ID_OK:

        # If the user selected OK, then the dialog's wx.ColourData will
        # contain valid information. Fetch the data ...
        data = dlg.GetColourData()

        # ... then do something with it. The actual colour data will be
        # returned as a three-tuple (r, g, b) in this particular case.
        self.log.WriteText('You selected: %s\n' % str(data.GetColour().Get()))

    # Once the dialog is destroyed, Mr. wx.ColourData is no longer your
    # friend. Don't use it again!
    dlg.Destroy()















        # build several examples of buttons with different colours and sizes
        buttonData = [
            ("Default Size",         (255, 255, 0),   wx.DefaultSize, ""),
            ("Another Size",         (255, 0, 255),   (60, 20),      ""),
            ("Another Colour",       (0, 255, 0),     wx.DefaultSize, ""),
            ("Larger Size",          (0, 0, 255),     (60, 60),      ""),
            ("With a Label",         (127, 0, 255),   wx.DefaultSize, "Colour..."),
            ("Another Colour/Label", (255, 100, 130), (120, -1),     "Choose Colour..."),
            ]

        self.buttonRefs = [] # for saving references to buttons

        # build each button and save a reference to it
        for name, color, size, label in buttonData:
            b = __csel__.ColourSelect(self, -1, label, color, size = size)

            b.Bind(__csel__.EVT_COLOURSELECT, self.OnSelectColour)


    def OnSelectColour(self, event):
        self.log.WriteText("Colour selected: %s" % str(event.GetValue()))


"""