# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Divers                    #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



from cercles import *



class NewVariable(Objet):
    """Une variable.
    
    Une variable numerique ; l'argument peut etre un nombre, ou une expression sous forme de chaine de caracteres.
    Exemple: Variable(17.5), Variable('AB.longeur()+1').
    Dans ce dernier cas, il est necessaire qu'une feuille de travail soit definie."""
    
    def __init__(self, valeur = 0):
        #if isinstance(valeur, Variable): # permet de faire des copies "en dur" ?
        #    valeur = valeur.valeur
        self.attributs = []
        self.vassaux = []   # lors de sa creation, l'objet n'a aucun vassal (aucun objet ne depend de lui)
        self.val(valeur)
        Objet.__init__(self)
        
    
    
        #self.enregistre((int, float, long, Cercle_generique, Segment, Glisseur_generique))
        

    def val(self, val = None, auto_refresh = True):  # variable propose une interface standard pour afficher une valeur (ou la modifier, si c'est une valeur libre), independamment de son origine.
        if val == None:
            if isinstance(self.valeur, list):
                liste = self.valeur[:]
                for i in range(len(liste)):
                    if i%2:
                        liste[i] = "self.__feuille__.objets['%s']" %liste[i].nom()
                return eval("".join(liste))
            else:
                return self.valeur
                
        else:
            if auto_refresh:  self.refresh()

            # on supprime la variable de la liste des vassaux pour les objets dont elle ne dependra plus desormais:
            for objet in self.ancetres(): 
                objet.vassaux = [vassal for vassal in objet.vassaux if vassal <> self]
            
            self.attributs = [] # et on initialise les attributs.
        
            if isinstance(val, str):
                liste = regexp("[^.][A-Ba-b_][A-Ba-b0-9_]*", " " + val.replace(" ",""), "x[0]+'$'+x[1:]+'$'").split("$")
                for i in range(len(liste)):
                    if i%2:
                        liste[i] = self.__feuille__.objets[liste[i]]
                        self.attributs.append(liste[i])
                self.valeur = liste
                dependances = self.ancetres()
                # plus self.hierarchie est faible, plus l'objet est haut place dans la hierarchie
                if dependances: self.hierarchie = max([objet.hierarchie for objet in dependances]) + 1
                else:           self.hierarchie = 0 # au sommet, les objets libres (ou suzerains)
                
                for objet in dependances:   # l'objet est vassal de chacun des objets dont il depend
                    objet.vassaux.append(self)
                    
            elif isinstance(valeur, (int, float, long)):
                self.valeur = valeur
                self.hierarchie = 0
            
            else:
                raise TypeError, "Les arguments d'une variable doivent etre de type str ou numerique, et non %s" %type(valeur)
                
            


    _call = val
            
    def __repr__(self):   # donne une representation affichable de l'objet qui sert a generer la variable
        if isinstance(self.valeur, (int, float, long)):
            return "Variable(%s)" %self.val()
        else:
            liste = self.valeur[:]
            for i in range(len(liste)):
                if i%2:
                    liste[i] = liste[i].nom()
            return "Variable('%s')" % "".join(liste).strip()
            

    
    def varier(self, debut = 0, fin = 1, pas = 0.02, periode = 0.03):
        if self.__canvas__():
            self.variation = [wx.Timer(self.__canvas__()), debut, fin, pas]
            self.variation[0].Start(1000*periode)
            
            self.__canvas__().Bind(wx.EVT_TIMER, self._varier)
        
    def _varier(self, event = None):
        self(self.variation[1])
        self.variation[1] += self.variation[3]
        if self.variation[3]*(self.variation[1] - self.variation[2]) > 0:
            self.variation[0].Stop()
        
        
        
    for operation in ("add", "sub", "mul", "div", "pow"): # definit __add__, __radd__, __sub__, etc...
        oper = "__" + operation + "__"
        roper = "__r" + operation + "__"
        def __op__(self, y, oper = oper):
            if isinstance(y, Variable):
                resultat = getattr(self(), oper)(y())
            else:
                resultat = getattr(self(), oper)(y)
                
            return Variable(resultat)
            
    
        def __rop__(self, y, roper = roper):
            return Variable(getattr(self(), roper)(y))
            
        exec("%s=__op__; %s=__rop__" %(oper, roper))
        
    del __op__, __rop__
        
    
    def __float__(self):
        return self()
        
    def __neg__(self):
        return 0 - self
        
    def __pos__(self):
        return 0 + self

    def __eq__(self, y):
        return hasattr(y, "__float__") and float(self) == float(y)
        
    def __ne__(self, y):
        return not self == y
        
    def __nonzero__(self):
        return self <> 0
        
    def __gt__(self, y):
        return hasattr(y, "__float__") and float(self) > float(y)
        
    def __ge__(self, y):
        return self > y or self == y





# LE CODE A ETE MODIFIE DE MANIERE A INTRODUIRE SYSTEMATIQUEMENT UNE "VARIABLE" COMME ARGUMENT DES DIFFERENTS OBJETS.
# (Sauf pour les objets non rataches a une __feuille__ - objets temporaires en general)

class Variable(Objet):
    """Une variable.
    
    Une variable numerique correpondant a un nombre, a un segment (longueur), a un cercle (rayon), ou un glisseur (parametre)."""
    
    def __init__(self, valeur = 0):
        if isinstance(valeur, Variable):
            valeur = valeur.valeur
        self.enregistre((int, float, long, Cercle_generique, Segment, Glisseur_generique))
        Objet.__init__(self)

    def val(self, val = None, auto_refresh = True):  # variable propose une interface standard pour afficher une valeur (ou la modifier, si c'est une valeur libre), independamment de son origine.
        if isinstance(self.valeur, (int, float, long)):
            if val <> None:
                self.valeur = val
                if auto_refresh: self.refresh()
            return 1.0*self.valeur      # travailler sur des decimaux evite de faire des divisions euclidiennes par erreur
        if isinstance(self.valeur, Cercle_generique):
            return 1.0*self.valeur.rayon()
        if isinstance(self.valeur, Segment):
            return 1.0*self.valeur.longueur()
        if isinstance(self.valeur, Glisseur_generique):
            return self.valeur.k.val()


    _call = val
            
    def __str__(self):   # donne une representation affichable de l'objet qui sert a generer la variable
        if isinstance(self.valeur, (int, float, long)):
            return repr(self.val())
        else:
            return "variable dependant de : " + repr(self.valeur)
            

    
    def varier(self, debut = 0, fin = 1, pas = 0.02, periode = 0.03):
        if self.__canvas__():
            self.variation = [wx.Timer(self.__canvas__()), debut, fin, pas]
            self.variation[0].Start(1000*periode)
            
            self.__canvas__().Bind(wx.EVT_TIMER, self._varier)
        
    def _varier(self, event = None):
        self(self.variation[1])
        self.variation[1] += self.variation[3]
        if self.variation[3]*(self.variation[1] - self.variation[2]) > 0:
            self.variation[0].Stop()
        
        
        
    for operation in ("add", "sub", "mul", "div", "pow"): # definit __add__, __radd__, __sub__, etc...
        oper = "__" + operation + "__"
        roper = "__r" + operation + "__"
        def __op__(self, y, oper = oper):
            if isinstance(y, Variable):
                resultat = getattr(self(), oper)(y())
            else:
                resultat = getattr(self(), oper)(y)
                
            return Variable(resultat)
            
    
        def __rop__(self, y, roper = roper):
            return Variable(getattr(self(), roper)(y))
            
        exec("%s=__op__; %s=__rop__" %(oper, roper))
        
    del __op__, __rop__
        
    
    def __float__(self):
        return self()
        
    def __neg__(self):
        return 0 - self
        
    def __pos__(self):
        return 0 + self

    def __eq__(self, y):
        return hasattr(y, "__float__") and float(self) == float(y)
        
    def __ne__(self, y):
        return not self == y
        
    def __nonzero__(self):
        return self <> 0
        
    def __gt__(self, y):
        return hasattr(y, "__float__") and float(self) > float(y)
        
    def __ge__(self, y):
        return self > y or self == y
    



class Rotation(Objet):
    """Une rotation.
    
    Une rotation definie par son centre, et un angle en radian (r), grad (g) ou degre (d)"""
    
    def __init__(self, centre = None, valeur = pi, unite = "r"):
        # l'unite par defaut est le radian.
        if centre == None:
            centre = Point(0, 0)
        valeur = self.nb2var(valeur)
        self.enregistre(Point_generique, Variable, str)
        Objet.__init__(self)
        
    #def centre(self):
    #    return self._centre.coordonnees()
    
    def angle(self, valeur = None, unite = "", auto_refresh = True):
        self.unite = unite or self.unite
        return self.valeur.val(valeur, auto_refresh)
        
    def radian(self):
        conversion = {"r":1., "g":pi/200., "d": pi/180.}
        return self.angle()*conversion[self.unite]



class Produit_scalaire(Objet):
    """Un produit scalaire.
    
    Le produit scalaire de deux vecteurs"""
    
    def __init__(self, vecteur1, vecteur2):
        self.enregistre(Vecteur_generique, Vecteur_generique)
        Objet.__init__(self)

    def val(self):
        return sum(self.vecteur1.coo()*self.vecteur2.coo())
    

# NOTE :
# A l'avenir, une autre methode pourrait etre d'implementer le produit scalaire comme multiplication directement dans l'objet vecteur (def __mul__(self, vecteur): etc...).
# On pourrait aussi imaginer que 2*un vecteur soit defini, ou encore vecteur + vecteur mais c'est un peu complique.
# Il y a differents types de vecteurs avec des dependances differentes, quel serait le type final ?
    
    

# A FAIRE :
# - GERER LES DIFFERENTES UNITES POUR LES ANGLES.
# - POUVOIR ENTRER UN ANGLE COMME ARGUMENT POUR LES ROTATIONS.

class Angle_generique(Objet):
    """Un angle generique.
    
    Classe mere des angles orientes ou non. Unite non supportee pour l'instant."""
    
    def __init__(self, vecteur1, vecteur2, unite = "r"):
        # l'unite par defaut est le radian.
        self.enregistre(Vecteur_generique, Vecteur_generique, str)
        Objet.__init__(self)
        
    def val(self):
        angle_geomq = arccos(Produit_scalaire(self.vecteur1, self.vecteur2).val()/(self.vecteur1.norme()*self.vecteur2.norme()))
        x1, y1 = self.vecteur1.coo(); x2, y2 = self.vecteur2.coo()
        determinant = x1*y2-x2*y1
        signe = determinant/abs(determinant)
        return signe*angle_geomq        



class Angle(Angle_generique):
    """Un angle.
    
    Un angle non oriente, defini par 3 points A, B, C -> angle /ABC\."""

    def __init__(self, point1, point2, point3, unite = "r"):
        # l'unite par defaut est le radian.
        self.enregistre(Point_generique, Point_generique, Point_generique)
        Angle_generique.__init__(self, point2>point1, point2>point3)

    def val(self):
        return abs(Angle_generique.val())
        
        

class Angle_oriente(Angle_generique):
    """Un angle oriente.
    
    Un angle oriente defini par deux vecteurs."""



class Texte(Objet):
    """Un texte.
    
    Un texte a afficher"""
    
    try:
        __empty_font__ = matplotlib.font_manager.FontProperties()
    except NameError:
        pass
    
    def _coordonnees(self, x = None, y = None):
        if isinstance(x, (tuple, list)):
            x, y = x[0], x[1]
        return (self.x.val(x, False), self.y.val(y))
        # False : ne rafraichit l'affichage qu'une fois (deja que c'est lent !!).

    def _affiche(self):
        #self.__canvas__().Texte(self.texte, self.coo())
        if not self.__representation__:
            self.__representation__ = [self.__canvas__().text(va = "center", ha = "center")]
            self.__representation__[0]._fontproperties = copy.copy(self.__empty_font__)
        x, y = self.coordonnees()
        text = self.__representation__[0]
        text._x = x ; text._y = y
        text._color = self.style("couleur")
        font = text._fontproperties
        font.set_size(self.style("taille"))
        #font.set_stretch(self.style("largeur"))  # mal gere par matploltib (version 0.87)
        font.set_weight(100*min(9, max(1, int(self.style("epaisseur")))))
        font.set_style(self.style("style"))
        font.set_family(self.style("famille"))
        text._rotation = self.style("angle")
        text._text = self.label()
        #debug(font.__dict__)



    def __init__(self, texte = "", x = None, y = None):
        if x == None:
            self.synchroniser_feuille()
            if self.__feuille__:
                xmin, xmax, ymin, ymax = self.__canvas__().fen()
                x = random.uniform(xmin,xmax)
                y = random.uniform(ymin,ymax)
                del xmin, xmax, ymin, ymax
            else:
                x = y = 0

        # on convertit l'entree en Variable
        # l'entree peut-etre un nombre, un(e longueur de) segment, un (rayon de) cercle
        x, y = self.nb2var(x), self.nb2var(y)
        
        self.enregistre(str, Variable, Variable)
        Objet.__init__(self)
        if texte:   self.label(texte)
        self.style(**param.textes)

        
    def _distance_inf(self, x, y, d):
        x0, y0 = self.position()
        return (x - x0)**2 + (y - y0)**2 < d**2

    #_call = _coordonnees
        

from points import *
from lignes import *
from cercles import *
from polygones import *
