# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#            fonctions mathlib                #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  
  
from redefinir import *

# Alias :
ln = log
asin = arcsin
acos = arccos
atan = arctan
rc = rac = sqrt

def prod(facteurs):
    return reduce(lambda x,y:x*y, facteurs, 1)

def factoriel(n, maximum = 20000):
    if n>maximum: raise OverflowError
    return prod(range(1, n + 1))

def gcd(a, b):
    "pgcd de a et de b"
    a, b = max(abs(a),abs(b)), min(abs(a),abs(b))
    while b:
        a, b = b, a%b
    return a

def lcm(a, b):
    "ppcm de a et de b"
    return a*b//gcd(a,b)

def pgcd(*termes):
    return reduce(lambda x,y:gcd(x,y), termes)

    
def ppcm(*termes):
    return reduce(lambda x,y:lcm(x,y), termes)


def factor(n):
    "decompose n en produit de facteurs premiers"
    p = [] # liste des facteurs
    if n < 0:
        n = abs(n)
        p.append(-1)
    
    while n%2 == 0:
        n = n//2; p.append(2)
    
    i = 3
    m = math.sqrt(n)
    while i <= m:
        while n%i == 0:
            n = n//i; m = m//math.sqrt(i)
            p.append(i)
        i += 2
        
    if n <> 1:  p.append(n)
    
    return p    


def facteur(n):
    return factor(n)


def diviseur(n):
    "donne la liste des diviseurs d'un entier"
    p = [] # liste des diviseurs
    if n < 0:
        n = abs(n)
        p.append(-1)
    
    i = 1
    m = math.sqrt(n)
    while i <= m:
        if n%i == 0:
            p += [n//i, i]
        i += 1
        print i
    
    return p



def n_premiers(n = 100, maximum = 50000): # securite face aux erreurs de frappe...!
    liste = [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97]
    m = liste[-1] + 2
    if n > maximum:
        raise OverflowError
    while len(liste) < n:
        r = math.sqrt(m)
        resultat = True
        for k in liste:
            if k > r:
                resultat = True
                break
            if not m%k:
                resultat = False
                break
        if resultat:
            liste.append(m)
        m += 2
    return liste
    
    

def frac(x, maximum = 100000):
    "Convertit un flottant en fraction."
    for i in xrange(2, maximum):
        if (int(x*i)-i*x) == 0:
            return int(x*i), i
    for i in xrange(1, maximum):
        if (int(x*10**i)-10**i*x) == 0:
            return int(x*10**i), 10**i
            
            
def bin(n):
    s = ""
    while n:
        s = str(n%2) + s
        n //= 2
    return s
    
    
    
def nchiffres(x, n = 1):
    "Arrondi x en fournissant n chiffres significatifs. Ex: nchiffres(2345, 2)."
    
    return x and round(x/10**floor(log10(abs(x))-n+1))*10**floor(log10(abs(x))-n+1) # Attention au cas x = 0 !

def arrondir(x):
    """Arrondi le nombre : un seul chiffre significatif, compris entre 1 et 5.
    Transforme automatiquement le nombre en entier selon le cas."""
    n = nchiffres(x, sqrt(.5))
    if int(n) == n:
        n = int(n)
    return n
    
    
def point_dans_polygone(couple, polygone):
    """Determine si un point est a l'interieur d'un polygone.
    D'apres un algorithme de Paul Bourke."""
    x, y = couple
    dedans = False
    n = len(polygone)
    p1x, p1y = polygone[0]
    for i in range(1, n + 1):
        p2x, p2y = polygone[i % n]
        if min(p1y, p2y) < y <= max(p1y, p2y) and x <= max(p1x, p2x):
            if p1y != p2y:
                xinters = (y - p1y)*(p2x - p1x)//(p2y - p1y) + p1x
                if p1x == p2x or x <= xinters:
                    dedans = not dedans
        p1x, p1y = p2x, p2y

    return dedans
    
    
def produit_scalaire(u, v):
    """Calcule le produit scalaire des vecteurs u et v.
    u et v doivent etre de type list, tuple, array, ou geolib.Vecteur."""
    return builtin.sum(i*j for i, j in zip(u ,v))