# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                Objets CALC                  #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from intervalles import *

class Expression:
    "Une chaine de caracteres representant une expression mathematiques."
        
    def __init__(self, chaine = "", variables = "", calcul_exact = None, conversion_crochet = True, OOo = True):
        self.calcul_exact = iff(calcul_exact == None, param.calcul_exact, calcul_exact)
        self.conversion_crochet = conversion_crochet
        self.OOo = OOo  # convertir les notations d'OpenOffice.org
        self.expression = chaine
        self.variables = variables
        
    def reformuler(self):
	
        calcul = self.expression.strip()
        if calcul.startswith(">"):
            calcul = "ans()" + calcul
        calcul = self.expression.replace("^", "**").replace(">deg", "*180/pi").replace(">rad", "*pi/180")
        
        for s in ("frac", "hex", "oct", "bin"):
            if calcul.endswith(">%s" %s):
                calcul = "%s(%s)" %(s, calcul[:-len(s)-1])
        
        
        # il est plus intuitif d'utiliser le symbole % pour les pourcentages, et mod pour le modulo.
        calcul = mreplace(calcul.replace("%", "/100"), (" mod ", " modulo "), "%")

        # On peut choisir comme separateur decimal la virgule (convention francaise en particulier)
        if param.changer_separateurs:
            calcul = calcul.replace(param.separateurs_personnels[0], ".").replace(param.separateurs_personnels[1], ",")

        # Transforme les accolades en parentheses (utile par exemple pour les fonctions issues d'OpenOffice.org).
        # De meme, la notation "times" d'OpenOffice.org est convertie.
        if self.OOo:
            calcul = calcul.replace("{", "(").replace("}", ")")
            calcul = regexp("[^a-zA-Z_]times[^a-zA-Z_0-9]", calcul, "x[0] + '*' + x[-1]")
            calcul = regexp("[^a-zA-Z_]over[^a-zA-Z_0-9]", calcul, "x[0] + '/' + x[-1]")

        calcul = " " + calcul.replace(" ", "") # permet de simplifier certains regexp.
        # Par exemple, pour ne pas remplacer log10() par log10*(), les chiffres doivent etre precedes
        # par un caractere non alphanumerique.
        # L'espace en debut de ligne evite de faire le cas particulier ou log10 est en tout debut de ligne.
        
        if self.calcul_exact:
            # Le code qui suit remplace 2/3 par 2*Frac(1, 1)/3, ce qui donne Frac(2,3).
            calcul = calcul.replace("/", "*Frac(1)/")
            # On remplace rac, rc et sqrt par Rac, mais sans que Frac devienne FRac (par exemple).
            # il doit y avoir plus elegant !
            calcul = regexp("[^a-zA-Z_]sqrt[(]|[^a-zA-Z_]rac[(]|[^a-zA-Z_]rc[(]", calcul, "x[0] + 'Rac('")
            calcul = regexp("^sqrt[(]|^rac[(]|^rc[(]", calcul, "'Rac('")
            debug(calcul)
        #else:
            # calcul = calcul.replace("/", "*1./")
            # Si le mode fraction est desactive, on convertit tout en float (sinon, 1/2 reserve des surprises).
        
        # frac est utilise comme alias de Frac
        calcul = regexp("[^a-zA-Z_]frac[(]", calcul, "x[0] + 'Frac('")
        calcul = regexp("^frac[(]", calcul, "'Frac('")

        
        # Transforme les listes en matrices (permet le calcul matriciel).
        if self.conversion_crochet:
            calcul = calcul.replace("[", "array([").replace("]", "])")

        # Le code qui suit remplace les expressions style 3x ou 2cos(x) par 3*x et 2*cos(x).
        calcul = regexp("[^a-zA-Z0-9][0-9]+[a-zA-DF-Z]", calcul, "x[:-1] + '*' + x[-1]")
        # On traite separemment le cas ou ca commence par "E", a cause de 2E17 par exemple, qui signifie 2*10**17
        calcul = regexp("[^a-zA-Z0-9.][E][-+]?[0-9]", calcul, "x.replace('E','1E')")
        calcul = regexp("[^a-zA-Z0-9][0-9]+[E][0-9]*[a-zA-Z]", calcul, "x.replace('E','*E', 1)")
        
        # De meme, on rajoute les * entre deux parentheses...
        calcul = calcul.replace(")(",")*(")
        
        calcul = calcul.replace("(", "( ").replace(")", " )")
        # evite les problemes avec "x(x(" par exemple (les regexp se "chevauchent").
 
        #... et devant des parentheses precedees de chiffres.
        # C'est assez delicat, car les noms de fonctions peuvent contenir des chiffres.
        #calcul = regexp("[^a-zA-Z0-9][0-9]+[(]", calcul, "x[:-1] + '*' + x[-1]")
        
        debug(calcul, 1)
        # Si a, b, c... ne sont pas des fonctions, on remplace "a(" par "a*(", etc...
        if self.variables:
            calcul = regexp("[^a-zA-Z0-9][0-9]*[0-9" + self.variables + "][(]", calcul, "x[:-1] + '*' + x[-1]")
        
        # On remplace ")x" par ")*x"
        calcul = regexp("[)][a-zA-Z0-9]", calcul, "x[0] + '*' + x[1]")

        
        # n! devient factoriel(n).
        calcul = regexp("[0-9]+[!]", calcul, "'factoriel(' + x[:-1] + ')'")
        
        return calcul
