# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#               Objets mathlib                #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

   
from fonctions import *
   
class Reel:
    """Classe mere pour les classes Frac et Rac.
    Permet de d'appliquer les operations mathematiques de pylab."""

    for nom in fonctions_matplotlib:
        exec("""
def %s(self, *args):
    arguments = []
    for arg in args:
        arguments.append(float(arg))
    return %s(float(self), *arguments)""" %(nom, nom))
    
    
    
class Frac(Reel):
    "Une fraction"
    
    def __init__(self, num = 1, den = 1, simplification = True):
        """Definit la fraction num/den."""
        if isinstance(num, Frac) or isinstance(num, Frac):
            fraction = num/den
            num = fraction.num; den = fraction.den
        if isinstance(num, float):
            num, d = frac(num)
            den *= d
            
        if isinstance(den, float):
            den, n = frac(den)
            num *= n

        num, den = int(num), int(den)
        self.numerateur = self.num = num
        self.denominateur = self.den = den
        if simplification:  self.simplifier()
        
        
    def simplifier(self):
        a = self.num; b = self.den

        if b < 0:   a = -a; b = -b
        if b == 0: raise ZeroDivisionError, "le denominateur ne doit pas etre nul."
            
        g = gcd(a, b)
        self.num = a//g; self.den = b//g
        
        return self
                 
    
    def __str__(self):
        if self.den == 1:   return str(self.num)
        return str(self.num) + "/" + str(self.den)
        
        
    def __repr__(self):
        return "Frac(%s,%s)" % (repr(self.num), repr(self.den))

    def __add__(self, y):
        if type(y) == float:    return y + float(self)
        if isinstance(y, (int, long)):      return Frac(y*self.den + self.num, self.den)
        if type(y) == types.ArrayType:    return y + array(self)
        if isinstance(y, Frac): return Frac(self.num*y.den + self.den*y.num, self.den*y.den)
        if isinstance(y, Rac):  return y.__radd__(self)
        #raise TypeError
        
    def __radd__(self,y):   return self.__add__(y)
            
    def __pos__(self):      return self
        
    def __sub__(self, y):   return self + (- y)

    def __rsub__(self, y):  return y + (- self)

    def __neg__(self):      return Frac(-1*self.num, self.den, False)
        
    def __mul__(self, y):
        if type(y) == float:    return y*float(self)
        if type(y) == types.ArrayType:    return y*array(self)
        if isinstance(y, (int, long)):      return Frac(y*self.num, self.den)
        if isinstance(y, Frac): return Frac(self.num*y.num, self.den*y.den)
        if isinstance(y, Rac):  return y.__rmul__(self)
        #raise TypeError

        
    def __rmul__(self, y): return self.__mul__(y)

    def __abs__(self):      return Frac(abs(self.num), abs(self.den), False)
    
    def __eq__(self, y):
        if type(y) == float:    return y == float(self)
        if isinstance(y, (int, long)):      return Frac(self.num, self.den) == Frac(y, 1)
        if type(y) == types.ArrayType:    return y == array(self)
        # Frac(4, 2, False) == 2 donne ainsi True.
        if isinstance(y, Frac): return self.num * y.den - self.den * y.num == 0
        if isinstance(y, Rac):  return y.__eq__(self)
        #raise TypeError

    def __nonzero__(self): # la valeur logique de l'objet (la doc officielle est fausse ??)
        return self <> 0
    
        
    def __float__(self):    return self.num/self.den
    
    def __int__(self):      return self.num//self.den
    
    def __ne__(self, y):    return not self.__eq__(y)
    
    def __gt__(self, y):
        if type(y) == float:    return float(self) > y
        if isinstance(y, (int, long)):      y = Frac(y, 1)
        if type(y) == types.ArrayType:    return y > array(self)
        if isinstance(y, Frac): return (self - y).num > 0
        if isinstance(y, Rac):  return not y.__gt__(self)
        #raise TypeError

    def __ge__(self, y):       return self.__eq__(y) or self.__gt__(y)

    def __pow__(self, y):
        if type(y) == float:    return pow(float(self), y)
        if int(y) == y: y = int(y)
        if isinstance(y, (int, long)):      return Frac(self.num**y, self.den**y)
        if isinstance(y, Frac): return pow(float(self), float(y))
        if isinstance(y, Rac):  return y.__rpow__(self)
        #raise TypeError

    def __rpow__(self, y):
        if self.den == 2:
            return y**((self.num - 1)//2)*Rac(y)
        if int(self) == self and type(y) <> float:   return pow(y, int(self))
        if isinstance(y, Frac) or type(y) == float or type(y) == int:  return pow(float(y), float(self))
        if type(y) == types.ArrayType:    return pow(y, array(self))
        #raise TypeError
        
    def __div__(self, y):
        if type(y) == float:    return float(self)/y
        if type(y) == types.ArrayType:    return array(self)/y
        if isinstance(y, (int, long)):      return Frac(self.num, y*self.den)
        if isinstance(y, Frac): return Frac(y.den*self.num, y.num*self.den)
        if isinstance(y, Rac):  return y.__rdiv__(self)
        #raise TypeError
        
    def __rdiv__(self, y):
        if type(y) == float:    return y/float(self)
        if type(y) == types.ArrayType:    return y/array(self)
        if isinstance(y, (int, long)):      return Frac(y*self.den, self.num)
        if isinstance(y, Frac): return Frac(self.den*y.num, self.num*y.den)
        #raise TypeError
    
        
    def __truediv__(self, y):
        return self.__div__(y)
        
    def __rtruediv__(self, y):
        return self.__rdiv__(y)









class Rac(Reel):
    "Une somme de racines carrees"
            
    def __init__(self, r = 1, n = 1, simplification = True):
        """Definit la racine n*sqrt(r).
        n, r peuvent etre des entiers, ou des fractions.
        r peut aussi etre une liste de couples [ri, ni].
        Dans ce dernier cas, Rac([[r1, n1], [r2, n2]]) signifie n1*Rac(r1) + n2*Rac(r2)
        Pour l'utilisateur final, la derniere notation, plus explicite, est celle recommandee.
        Eventuellement, l'implementation pourrait en effet changer."""
        self.valeur = iff( type(r) == list, r, [[r, n]] ) 
        

        # La somme de racines carrees est stockee dans self.valeur, sous forme [[r1, n1], [r2, n2], ...]
        # Ceci signifie : n1*sqrt(r1) + n2*sqrt(r2) + ...
        
        self.den_entier()
        if simplification:  self.simplifier()
        self.regrouper()
        self.suppr_zeros()
        
        
    def den_entier(self):
        "Renvoie les racines avec des denominateurs entiers."
        valeur = []
        for r, n in self.valeur:
            if isinstance(r, Frac):
                n *= Frac(1, r.den); r = r.num*r.den
            valeur.append([r, n])
        self.valeur = valeur
        return self        
        
    
    def regrouper(self):
        "Regroupe entre elles les racines : 2V3 + 5V3 = 7V3."
        # on regroupe les termes selon les racines r.
        # on ne doit PAS tenir compte du n, qui peut-etre un nombre complexe (classement impossible !)
        self.valeur.sort(key=lambda x:x[0])
        # x est une liste des couples (ri, ni), y est un couple (r,n).
        def f(x, y):
            if x[-1][0] == y[0]:
                x[-1][1] += y[1]
            else: x.append(y)
            return x
        self.valeur = reduce(f, self.valeur[1:], [self.valeur[0]])
        return self
    
    
    def suppr_zeros(self):
        """Supprime les zeros de la liste de racine.
        Exemple : V5 + 0V3 = V5."""
        self.valeur = [[r, n] for r, n in self.valeur if r<>0 and n<>0] or [[1, 0]]
        return self
        
        
    def simplifier(self):
        "Renvoie les racines sous forme a*Rac(b) avec a le plus grand possible."
        valeur = []
        for r, n in self.valeur:
            r_initial = r
            if r < 0:   raise ValueError, "racine carree d'un nombre negatif."
            if r > 10**7:
                print "depassement de capacite de calcul."
                return self
            
            # NOTE : Algorithme exponentiel (t <= .5*exp(.5*n)), a revoir si possible.
            
            m = math.sqrt(r)
            
            if int(m) == m: # la racine tombe juste
                n *= int(m); r = 1
            else:
                while r%4 == 0:
                    r = r//4; m = m//4
                i = 1
                while i < m:
                    i += 2
                    while r%i**2 == 0:
                        r = r//i**2; m = m//i
                n *= int(math.sqrt(r_initial//r))
            valeur.append([r, n])
        self.valeur = valeur
        return self
    
                    
    def __str__(self):
        def signe(x):
            return iff(x<0, "-", "+")
            
        def affiche(racine):
            # representation d'une racine.
            # Exemple: [5, Frac(1,3)] devient sqrt(5)/3
            r, n = racine; d = 1
            if r == 1:   return str(n)
            if isinstance(n, Frac):
                d = n.den; n = n.num
            return signe(n) + iff(abs(n) <> 1, str(abs(n)) + "*") + "sqrt(%s)" %r + iff(d <> 1, "/" + str(d))         
            
        return "".join([affiche(racine) for racine in self.valeur]).lstrip("+")
        
    
    def __repr__(self):
        return "Rac(%s)" % repr(self.valeur)
    
    
    
    def __float__(self):    return sum([n*sqrt(r) for r,n in self.valeur])
        
    def __int__(self):      return int(float(self))

    def __add__(self, y):
        if type(y) == float:   return float(self) + y
        if not isinstance(y, Rac):    return self + Rac(1, y)
        return Rac(y.valeur + self.valeur)
         
    def __radd__(self,y):   return self.__add__(y)
            
    def __pos__(self):      return self
        
    def __sub__(self, y):   return self + (- y)

    def __rsub__(self, y):  return y + (- self)

    def __neg__(self):      return Rac([[r, -n] for r,n in self.valeur])
        
    def __mul__(self, y):
        if type(y) == float:    return y*float(self)
        if not isinstance(y, Rac):    return self * Rac(1, y)
#        print [[r*s,m*n] for r,n in self.valeur for s,m in y.valeur]
        return Rac([[r*s,m*n] for r,n in self.valeur for s,m in y.valeur])
        
    def __rmul__(self, y): return self.__mul__(y)

    def __abs__(self):      return Rac([[r, abs(n)] for r,n in self.valeur])
    
    def __eq__(self, y):
        if isinstance(y, (int, long)):
            return self.valeur == [[1, y]] 
        if isinstance(y, Rac):
            return len(self.valeur) == len(y.valeur) and not sum([i<>j for i,j in zip(self.valeur, y.valeur)])
        return float(self) == y
        
    def __nonzero__(self): # la valeur logique de l'objet (la doc officielle est fausse ??)
        return self <> 0
    
    def __ne__(self, y):    return not self.__eq__(y)
    
    def __gt__(self, y):
        return float(self) > y

    def __ge__(self, y):    return self.__eq__(y) or self.__gt__(y)

    def __pow__(self, y):
        if type(y) <> float and int(y) == y: y = int(y)
        if not isinstance(y, (int, long)):    return pow(float(self), y)
        if y == 0: return 1
        if y == 1: return self
        # Si c'est un entier, on applique l'exponentiation indienne.
        return self**(y//2)*self**(y//2)*self**(y%2)

    def __rpow__(self, y):
        return pow(float(y), float(self))
        
    def __div__(self, y):
        if type(y) == float:
            return float(self)/y
        if isinstance(y, Rac) and len(y.valeur) == 1 and y.valeur[0][0] == 1:
            y = y.valeur[0][1]
        if isinstance(y, (int, long, Frac)):
            return Rac([[r, Frac(n,y)] for r, n in self.valeur])
        if len(y.valeur) == 1:
            expression_conjuguee = y
        else:
            expression_conjuguee = Rac(y.valeur[:-1]) - Rac([y.valeur[-1]])
        return (self*expression_conjuguee)/(y*expression_conjuguee)
        
        
    def __rdiv__(self, y):
        if type(y) == float:    return y/float(self)
        if isinstance(y, (int, long, Frac)): y = Rac(y)
        return y.__div__(self)
        
    def __truediv__(self, y):
        return self.__div__(y)
        
    def __rtruediv__(self, y):
        return self.__rdiv__(y)


    
    
    
class Polynome:
    "Un polynome"

    def __init__(self, *coeffs):
        if isinstance(coeffs[0], (tuple,list)):
            coeffs = coeffs[0]
        if len(coeffs) == 0:
            coeffs = [0]
        self.coefficients = self.coeffs = list(coeffs)
        
        self.simplifier()
        # a poursuivre !
        
    def simplifier(self): # enleve les coeffs nuls au debut
        for i in range(len(self.coeffs)):
            if self.coeffs[i] <> 0: break
        self.coeffs = self.coeffs[i:]
        
    #def __call__(self, valeur):
        
    
    def degre(self):
        if self.coeffs == [0]:
            return None # None se comporte un peu comme -oo
        return len(self.coeffs) - 1
        
    deg = degre
    
    def delta(self):
        if self.deg() == 2:
            a, b, c = self.coeffs
            return b**2 - 4*a*c
    
    def solutions(self, valeur_exacte = True):
        if self.deg() == 2:
            a, b, c = self.coeffs; delta = self.delta()
            un = valeur_exacte and 1 or 1.
            if delta == 0:  return [un*-b*Frac(1)/(2*a)]
            if delta > 0:   return [(-b - un*Rac(delta))*Frac(1)/(2*a), (-b + un*Rac(delta))*Frac(1)/(2*a)]
            return []
            
    def derive(self):
        n = len(self.coeffs)-1
        return Polynome([(n-i)*self.coeffs[i] for i in range(n)])
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        

class PlusInfini:
    def __cmp__(self, y):
        return 1
        
    def __neg__(self):
        return MoinsInfini()
        
    def __pos__(self):
        return self
    
    def __nonzero__(self):
        return True

    def __invert__(self):
        return 0
        
    def __eq__(self, y):
        return y.__class__ == self.__class__
        
    def __ne__(self, y):
        return not self.__eq__(y)
    
    def __gt__(self, y):
        return True
    
    def __lt__(self, y):
        return not self > y

    def __ge__(self, y):
        return self > y or self == y
        
    def __le__(self, y):
        return self < y or self == y
        
    def __add__(self, y):
        if y == -self:
            raise ArithmeticError, "forme indtermine." 
        return self
    
    def __radd__(self, y):
        return self.__add__(y)
        
    def __mul__(self, y):
        if y > 0:
            return self
        elif y < 0:
            return -self
        else:
            raise ArithmeticError, "forme indtermine."
        
    def __rmul__(self, y):
        return self.__mul__(y)
    
    def __div__(self, y):
        if y > 0:
            return self
        elif y < 0:
            return -self
        else:
            raise ArithmeticError, "forme indtermine."
        
    def __rdiv__(self, y):
        if y <> self and y <> -self:
            return 0
        raise ArithmeticError, "forme indtermine."
        
    def __truediv__(self, y):
        return self.__div__(y)
        
    def __rtruediv__(self, y):
        return self.__rdiv(y)

    def __sub__(self, y):
        return self + (-y)
        
    def __rsub__(self, y):
        return y + (-self)
        
    def __str__(self):
        return "+infini"
        
    def __repr__(self):
        return self.__class__.__name__ + "()"


        
        
class MoinsInfini(PlusInfini):
    def __cmp__(self, y):
        return -1

    def __neg__(self):
        return PlusInfini()

    def __gt__(self, y):
        return False

    def __str__(self):
        return "-infini"



infini = Infini = PlusInfini()