#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                Calculatrice                 #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA




from API import *
from GUI import *







class CalculatriceMenuBar(MenuBar):
    def __init__(self, panel):
        MenuBar.__init__(self, panel)
        self.ajouter("Fichier", ["quitter"])
        self.ajouter("Affichage", ["onglet"])
        self.ajouter("Outils",
                        ["Memoriser le resultat", "Copie le resultat du calcul dans le presse-papier, afin de pouvoir l'utiliser ailleurs.", "Ctrl+M", self.panel.vers_presse_papier],
                        ["options"])
        self.ajouter("?")






class Calculatrice(Panel_simple):
    __titre__ = "Calculatrice" # Donner un titre a chaque module
    
    def __init__(self, parent):
        Panel_simple.__init__(self,parent)


        self.entrees = wx.BoxSizer(wx.HORIZONTAL)
        self.entree = wx.TextCtrl(self, size = (500, -1))
        self.entrees.Add(self.entree, 1, wx.ALL|wx.GROW, 5)
        self.valider = wx.Button(self, wx.ID_OK)
        self.entrees.Add(self.valider, 0, wx.ALL, 5)
                
        self.corps = wx.BoxSizer(wx.HORIZONTAL)
        self.resultats = wx.TextCtrl(self, size = (400,310), style = wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_RICH)
        self.corps.Add(self.resultats, 0, wx.ALL, 5)
        
        self.pave  = wx.BoxSizer(wx.VERTICAL)
        self.corps.Add(self.pave, 0, wx.ALL, 5)
        boutons = ["2nde", "ans", "ouv", "ferm", "egal", "7", "8", "9", "div", "x", "4", "5", "6", "mul", "y", "1", "2", "3", "minus", "z", "0", "pt", "pow", "plus", "t", "rac", "sin", "cos", "tan", "exp", "i", "pi", "e", "abs", "mod"]
        inserer = ["", "ans()", "(", ")", "=", "7", "8", "9",  "/", "x", "4", "5", "6", "*", "y", "1", "2", "3", "-", "z", "0", ".", "^", "+", "t", "sqrt(", ("sin(", "asin(", "sinus / arcsinus"), ("cos(", "acos(", "cosinus / arccosinus"), ("tan(", "atan(", "tangente / arctangente"), ("exp(", "ln(", "exponentielle / logarithme neperien"), ("i", "^(1/3.)", "i / racine cubique"), ("pi", "sinh(", "pi / sinus hyperbolique"), ("e", "cosh", "e / cosinus hyperbolique"), ("abs(", "tanh", "valeur absolue / tangente hyperbolique"), (" mod ", "log10(", "modulo / logarithme decimal")]
        
        self.seconde = False # indique si la touche 2nde est activee.
        
        def action(event = None):
            self.seconde = not self.seconde
            if self.seconde: self.message("Touche [2nde] activee.")
            else: self.message("")
            
        self.actions = [action]
        
        for i in range(len(boutons)):
            # On aligne les boutons de la calculatrice par rangees de 5.
            if i%5 == 0:
                self.rangee = wx.BoxSizer(wx.HORIZONTAL)
                self.pave.Add(self.rangee, 0, wx.ALL,0)
            
            # Ensuite, on construit une liste de fonction, parallelement a la liste des boutons.
            if i > 0:
                def action(event = None, entree = self.entree, j = i):
                    if type(inserer[j]) == tuple:
                        entree.WriteText(inserer[j][self.seconde])
                    else:
                        entree.WriteText(inserer[j])
                    n = entree.GetInsertionPoint()
                    entree.SetFocus()
                    entree.SetInsertionPoint(n)
                    self.seconde = False
                    self.message("")
                self.actions.append(action)
                
            bmp = wx.Image('images' + os.sep + 'btn_' + boutons[i] + '.png', wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            bouton = wx.BitmapButton(self, -1, bmp, style=wx.NO_BORDER)
            bouton.SetBackgroundColour(self.GetBackgroundColour())
            self.rangee.Add(bouton, 0, wx.ALL,3)
            # A chaque bouton, on associe une fonction de la liste.
            bouton.Bind(wx.EVT_BUTTON, self.actions[i])
            if type(inserer[i]) == tuple: bouton.SetToolTipString(inserer[i][2])

        self.pave.Add(wx.BoxSizer(wx.HORIZONTAL))

        
        self.options = [("Valeurs exactes.", "calcul_exact"), ("Ecriture scientifique.", "ecriture_scientifique"), ("Virgule comme separateur decimal.", "changer_separateurs"), ("Copie du resultat dans le presse-papier.", "copie_automatique")]
        self.options_box = []
        for i in range(len(self.options)):
            ligne = wx.BoxSizer(wx.HORIZONTAL)
            self.options_box.append(wx.CheckBox(self))
            ligne.Add(self.options_box[i])
            self.options_box[i].SetValue(eval("param." + self.options[i][1]))
            def action(event, chaine = self.options[i][1], entree = self.entree):
                exec("param." + chaine + "= not param." + chaine)
                entree.SetFocus()
            self.options_box[i].Bind(wx.EVT_CHECKBOX, action)
            ligne.Add(wx.StaticText(self, -1, "  " + self.options[i][0]))
            self.pave.Add(ligne)
            
            
        self.option1 = wx.BoxSizer(wx.HORIZONTAL)
        self.pave.Add(self.option1)
        #wx.CheckBox(self)
        
        self.option2 = wx.BoxSizer(wx.HORIZONTAL)
        self.pave.Add(self.option2)
        # </A ECRIRE>
 
        
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.entrees, 0, wx.ALL, 5)
        self.sizer.Add(self.corps, 0, wx.ALL, 5)
        self.SetSizer(self.sizer)
        self.Fit()
        
        # historique des calculs
        self.entree.Bind(wx.EVT_CHAR, self.EvtChar)
        self.valider.Bind(wx.EVT_BUTTON, self.affiche)
        self.initialiser()
        
        
        
    def _sauvegarder(self, fgeo):
        fgeo.contenu["Calculatrice"] = [{}]
        fgeo.contenu["Calculatrice"][0]["Calcul"] = self.historique
        fgeo.contenu["Calculatrice"][0]["Resultats"] = [repr(self.derniers_resultats)]
        fgeo.contenu["Calculatrice"][0]["Affichage"] = [self.resultats.GetValue()]
        fgeo.contenu["Calculatrice"][0]["Dernier_resultat"] = [self.dernier_resultat]
        fgeo.contenu["Calculatrice"][0]["Position"] = [repr(self.position)]
        fgeo.contenu["Calculatrice"][0]["Options"] = [{}]
        for i in range(len(self.options)):
            fgeo.contenu["Calculatrice"][0]["Options"][0][self.options[i][1]] = [str(self.options_box[i].GetValue())]


        
    def _ouvrir(self, fgeo):
        if fgeo.contenu.has_key("Calculatrice"):
            calc = fgeo.contenu["Calculatrice"][0]
            self.initialiser()
            
            self.historique = calc["Calculs"]
            self.derniers_resultats = securite.eval_safe(calc["Resultats"][0])
            self.position = int(calc["Position"][0])
            self.resultats.SetValue(calc["Affichage"][0] + "\n")
            
            liste = calc["Options"][0].items()
            options = [option for aide, option in self.options]
            for key, value in liste:
                value = securite.eval_safe(value[0])
                setattr(param, key, value)
                if key in options:
                    self.options_box[options.index(key)].SetValue(value)
            # il faudrait encore sauvegarder les variables, mais la encore, 2 problemes :
            # - pb de securite pour evaluer les variables
            # - pb pour obtenir le code source d'une fonction.
            # Pour remedier a cela, il faut envisager de :
            # - creer un module d'interpretation securisee.
            # - rajouter a chaque fonction un attribut __code__, ou creer une nouvelle classe.
    
    def vers_presse_papier(self, event = None):
        self.clipBoard=wx.Clipboard()
        if self.clipBoard.Open():
            self.clipBoard.AddData(wx.TextDataObject(self.dernier_resultat))
        self.clipBoard.Close()
    
    def initialiser(self):
        self.dernier_resultat = "" # dernier resultat, sous forme de chaine formatee pour l'affichage
        self.historique = []
        self.position = -1
        self.derniers_resultats = [] # liste des resultats, sous forme de valeurs numeriques
        self.entree.SetFocus()

        # Initialise self.a, self.b, ... self.z
        for lettre in string.ascii_lowercase + string.ascii_uppercase:
            exec("self." + lettre + "=0")
    
    
    def affiche(self, event = None):
        entree = self.entree.GetValue()
        try:
            resultat = self.evaluer(entree)
            longueur = len(self.resultats.GetValue())
            self.resultats.AppendText("ans(" + str(len(self.derniers_resultats)) + ") = " + entree + "\n       ans(" + str(len(self.derniers_resultats)) + ") = " + self.formater(resultat)+"\n__________________\n\n")
            self.message("Calcul effectue." + self.warning)
            self.historique.append(entree)
            self.derniers_resultats.append(resultat)
            self.position = len(self.historique) - 1
            self.entree.Clear()
            self.resultats.SetInsertionPoint(longueur)
            self.resultats.SetFocus()
            self.entree.SetFocus()
        except:
            self.message("Calcul impossible. Essayez en calcul approche ?")
            self.entree.SetFocus()
            if param.debug:
                raise



    
    def EvtChar(self,event):    # gere la ligne d'entree des calculs.
        code = event.GetKeyCode()

        if code == 13:
            self.affiche()

        elif code == 317:   # on remonte dans l'historique des entrees
            if self.position >= 0:
                self.entree.SetValue(self.historique[self.position])
                self.position = self.position - 1

        elif code == 319:   # on redescend dans l'historique vers les entrees recentes
            if self.position < len(self.historique) - 1:
                if self.position == (len(self.historique) - 2):
                    self.entree.Clear()
                else:
                    self.entree.SetValue(self.historique[self.position + 2])
                self.position = self.position + 1
        else:
            event.Skip()
            
            
        
    def evaluer(self, calcul):
        self.warning = ""
        
                
        # Pour multiplier par 2 le dernier resultat, il suffit de taper "*2", le ans() s'insere avant.
        if calcul[0] in "*/^%+" : calcul = "ans()" + calcul
        
        calcul = calcul.replace("ans(", "self.ans(")
        
        
        # Ferme automatiquement les parentheses.
        parentheses = "({[", ")}]"
        for i in range(3):
            correction = parentheses[1][i]*(calcul.count(parentheses[0][i])-calcul.count(parentheses[1][i]))
            calcul = calcul + correction
            if correction: self.warning += " Attention, il manque des parentheses \"" + parentheses[1][i] + "\"."


        # Si a, b, c... ne sont pas des fonctions, on remplace "a(" par "a*(", etc...
        #variables = filter(lambda x:type(eval("self." + x)) <> types.FunctionType, "".ascii_lowercase + "".ascii_uppercase)
        variables =""
        for lettre in string.ascii_lowercase + string.ascii_uppercase:
            if type(eval("self." + lettre)) <> types.FunctionType:
                variables += lettre
        

        # La classe Expression de la librairie CALC_objets permet d'effectuer un certain nombre de conversions.
        calcul = Expression(calcul, variables).reformuler()
        
        
        # On affecte a la variable a la valeur de self.a, a la variable A celle de self.A, etc... et i vaut 1j.
        for lettre in string.ascii_lowercase + string.ascii_uppercase:
            exec(lettre + "=self." + lettre)
            
        i = 1j
        e = math.e
        # on evite d'attribuer la valeur 2.718... a E, pour eviter les confusions entre 3E-7 et 3*E-7 (!)
        try:
            del E
        except NameError:
            pass
        pi = PI = math.pi

        # Le code qui suit sert a affecter une valeur aux variables a, b, ... z.
        # On remplace a=3, d=sin(2) par self.a=3, self.d=sin(2), et on execute.
        l = re.split("([ ]*?[a-zA-Z][ ]*?=[^=])", calcul)
        if len(l) <> 1:
            if l[1][0] not in "Eei":
                exec("self." + l[1].strip() + l[2])
                l = (l[1]+l[2]).split("=")
                return "La variable \"" + l[0].strip() + "\" vaut maintenant " + str(eval(l[1])) + "."
            else:
                return "Les constantes \"i\", \"E\" et \"e\" sont protegees."
        
        # On definit des fonctions de la sorte : "f(x)=2x".
        # Attention : a cause des corrections automatiques, une chaine de cette forme arrive ici sous la forme
        # "f*( x)=2*x", donc il faudra supprimer le premier "*".
        l = re.split("([a-zA-Z][*]?[(][ ]?[a-zA-Z,]*[ ]?[)]=)", calcul)
        if len(l) <> 1:
            if l[1][0] not in "Eei":
                vars = globals()
                vars.update(locals())
                exec("def " + l[1][:-1].replace("*", "") + ":return " + l[2], vars) # def f(x):return 2*x dans cet exemple
                exec("self." + l[1][0] + "=" + l[1][0], vars) # self.f = f dans cet exemple
                return "La fonction \"" + l[1][0] + "\" est definie par " + l[2] + "."
            else:
                return "Les constantes \"i\", \"E\" et \"e\" sont protegees."
        
        debug(calcul)
        return eval(calcul)


    def ans(self, n = -1):
        if self.derniers_resultats: return self.derniers_resultats[n]
        self.warning += " Ans(): aucun calcul anterieur." 
        return 0
    
    def formater(self, resultat):
        resultat = str(resultat)
        # Les nombres complexes sont notes entre parentheses, ce qui n'est pas tres "esthetique"...
        # L'expression reguliere suivante les enleve.
        # resultat = regexp("[^a-zA-Z][(][-]?[0-9]+[.]?[0-9]*[+-][0-9]+[.]?[0-9]*j[)]", " " + resultat, "x[0]+x[2:-1]")[1:]
        # Cela posait probleme dans certains cas (produits de facteurs...), on les enleve seulement dans les cas simples.
        resultat = regexp("^[(][^()]*[)]$", resultat, "x[1:-1]")
        

        # Une bonne part du formatage concerne les nombres complexes : 0j est remplace par 0, par exemple.
        resultat = resultat.replace("+0j", "").replace("0j", "0").replace("1j", "i").replace("j", "i").replace("-", " - ").replace("+", " + ").replace("e - ", "E-").replace("e + ", "E+")
        
        # Gestion de l'ecriture scientifique
        if param.ecriture_scientifique:
            try:
                if type(eval(resultat)) == float:
                    resultat = eval(resultat)
                    mantisse = int(math.floor(math.log10(abs(resultat))))
                    resultat = str(resultat/10.**mantisse)+"E"+str(mantisse)
            except: pass
	if param.changer_separateurs:
		resultat = resultat.replace(",", param.separateurs_personnels[1]).replace(".", param.separateurs_personnels[0])
        
        # Cette partie sert au presse-papier.
        self.dernier_resultat = resultat
        if param.copie_automatique: 
            self.vers_presse_papier()
        return resultat