# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Traceur                   #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA




from API import *
from GUI import *
import tableau


class TraceurMenuBar(MenuBar):
    def __init__(self, panel):
        MenuBar.__init__(self, panel)

        self.ajouter("Fichier", ["nouveau"], ["ouvrir"], ["ouvrir ici"], ["enregistrer"], ["exporter"], ["exporter&sauver"], None, ["mise en page"], ["imprimer"], None, self.panel.doc_ouverts, None, ["fermer"], ["quitter"])
        self.ajouter("Editer", ["annuler"], ["refaire"], ["modifier"], ["supprimer"])
        self.ajouter("creer")
        self.ajouter("Affichage", ["onglet"], ["repere"], ["quadrillage"], ["fenetre"], ["zoomer"], ["dezoomer"], ["orthonormaliser"], ["zoom_auto"])
        self.ajouter("Outils", ["Tableau de valeurs", "Tableaux de valeurs des fonctions.", "Ctrl+T", self.panel.tableau], ["historique"], ["securise"], ["debug"], ["options"])
        self.ajouter("?")




class Traceur(Panel_API_graphique):
    
    __titre__ = "Traceur de courbes" # Donner un titre a chaque module
    
    def __init__(self, parent):
        
    
        self.couleurs = "bgrmkcy"

        self.nombre_courbes = param.nombre_courbes
        self.boites = []
        self.equations = []
        self.intervalles = []

        Panel_API_graphique.__init__(self, parent)

        self.entrees = wx.BoxSizer(wx.VERTICAL)
        
        
        self.entrees.Add(wx.StaticText(self, -1, " Equations :"), 0, wx.ALL,5)
        
        for i in range(self.nombre_courbes):
                ligne = wx.BoxSizer(wx.HORIZONTAL)
                
                self.boites.append(wx.CheckBox(self))
                self.boites[-1].SetValue(True) # Par defaut, les cases sont cochees.
                self.boites[i].Bind(wx.EVT_CHECKBOX, self.affiche)
                ligne.Add(self.boites[i], 0, wx.ALIGN_CENTRE|wx.ALL,5)
                
                ligne.Add(wx.StaticText(self, -1, "Y" + str(i+1) + " = "), 0, wx.ALIGN_CENTRE|wx.ALL,5)
                self.equations.append(wx.TextCtrl(self, size = (120, -1)))
                self.equations[i].Bind(wx.EVT_CHAR, self.EvtChar)
                ligne.Add(self.equations[i], 0, wx.ALIGN_CENTRE|wx.ALL,5)
                
                ligne.Add(wx.StaticText(self, -1, "sur"), 0, wx.ALIGN_CENTRE|wx.ALL,5)
                self.intervalles.append(wx.TextCtrl(self, size = (100, -1)))
                self.intervalles[i].Bind(wx.EVT_CHAR, self.EvtChar)
                ligne.Add(self.intervalles[i], 0, wx.ALIGN_CENTRE|wx.ALL,5)
                
                self.entrees.Add(ligne, 0, wx.ALL, 5)
                
        #self.dessiner = wx.Button(self, wx.ID_REFRESH)
        #self.entrees.Add(self.dessiner, 0, wx.ALL, 5)
        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer.Add(self.canvas, 1, wx.LEFT | wx.TOP | wx.GROW)
        self.sizer.Add(self.entrees, 0, wx.ALL, 5)
        self.SetSizer(self.sizer)
        self.Fit()


    
    def _sauvegarder(self, fgeo):
        Panel_API_graphique._sauvegarder(self, fgeo)
        fgeo.contenu["Courbe"] = [{"Y" : [self.equations[i].GetValue()], "intervalle" : [self.intervalles[i].GetValue()], "active" : [str(self.boites[i].GetValue())]} for i in range(self.nombre_courbes)]
        
    
    def _ouvrir(self, fgeo):
        Panel_API_graphique._ouvrir(self, fgeo)
        if fgeo.contenu.has_key("Courbe"):
            for i in range(min(len(fgeo.contenu["Courbe"]), self.nombre_courbes)):
                self.equations[i].SetValue(fgeo.contenu["Courbe"][i]["Y"][0])
                self.intervalles[i].SetValue(fgeo.contenu["Courbe"][i]["intervalle"][0])
                self.boites[i].SetValue(fgeo.contenu["Courbe"][i]["active"][0] == "True")
        self.affiche()


    
    def EvtChar(self, event):
        code = event.GetKeyCode()

        if code == 13:
            self.affiche()
        else:
            event.Skip()
            
        
        
    def tableau(self, event = None):
        table = tableau.TableauValeurs(self)
        table.Show(True) 
        #table.SetSize(wx.Size(200,250))
        #table.SetDimensions(-1, -1, -1, 300)
    
    
    def conversion(self, equation, intervalle = False):
        # La fonction reformuler() du module CALC_objets effectue un grand nombre de conversions bien utiles...
        # Par exemple, 2x devient 2*x, 2^3 devient 2**3, ln() devient log(), etc...
        equation = Expression(equation, "xX", False, False, not intervalle).reformuler().replace(" ","")
        debug("Conversion de formule :", "'" + equation + "'")
        
        return intervalle and equation or (equation + "+0.*x")
        # Si c'est une vraie equation (pas un intervalle), le "+0.*x" est tres important !
        # 1. Evite un plantage quand equation = "3" (3 n'est pas un tableau !).
        # 2. Convertit tout en float (sinon, 1/x reserve des surprises aux extremites).
        

    def _affiche(self):
        for i in range(len(self.boites)):
            if self.boites[i].GetValue():
                self.couleur = self.couleurs[i%len(self.couleurs)]
                intervalles = self.intervalles[i].GetValue().split("|")
                equations = self.equations[i].GetValue().split("|")
                for j in range(len(intervalles)):
                    if equations[j]: self.dessine_courbe(equations[j], intervalles[j])
        

    def evaluer(self, x):
        X = x
        dico = globals().copy()  # evaluation securisee (certains modules et fonctions ne sont plus accessibles)
        dico.update(locals())
        try:
            return 1.*securite.eval_restricted(self.conversion(self.eq_actuelle), dico)
        except (OverflowError, ValueError):
            self.message("Verifiez votre intervalle de definition.")
            raise

    def y(self, x):
        X = x # = self.optimise_x(x)
        return self.evaluer(x)
    

#    def optimise_x(self, x):
#        # cette fonction sert a resoudre les problemes du type pow(array([1e-300]), 2) (-> renvoie OverflowError).
#        # a noter que pow(1e-300, 2) fonctionne bien (-> renvoie 0.0).
#        # inutile avec les dernieres versions de Numeric ??
#        
#        # x doit etre un tableau (et un "vrai", pas du style array(2)!)
#        try:
#            int(x)
#            x = array([x])
#        except:
#            pass
#        
#        xx = x
#        for j in range(1000): # pour eviter une eventuelle boucle infinie...
#            xxx = filter(lambda t: t <> 0, abs(xx))
#            if not xxx:
#                break # il n'y a aucun element non nul.
#            m = min(xxx)
#            # m est le plus petit element non nul.
#            i = list(abs(xx)).index(m)
#            try:
#                X = x = array([m])
#                self.evaluer(x)
#                break
#            except OverflowError:
#                if param.debug: print "Optimisation..."
#                xx[i] = 0.
#            except:
#                pass
#        return xx
    
    
    def arc(self, x, sens = 1):
        # affiche un petit arc de cercle [sens = 1 : ")", sens = -1 : "("]
        # on travaille sur les pixels pour faire de la trigonometrie plus simplement
        # (le repere devient ainsi orthonorme)
        x1, x2 = self.canvas.Xcoo2pix([x, x + sens*self.canvas.pas()])
        y1, y2 = self.canvas.Ycoo2pix([self.y(x), self.y(x + sens*self.canvas.pas())])
        angle = arctan((y2 - y1)/(x2 - x1))
        # donne l'angle d'incidence a l'extremite
        t, R = arange(angle - sens*pi/2, angle + pi - sens*pi/2, 0.01), self.canvas.taille["("]
        self.canvas.plot(self.canvas.Xpix2coo(x1 + R*(cos(t) - sens*cos(angle))), self.canvas.Ypix2coo(y1 + R*(sin(t) - sens*sin(angle))), self.couleur)
    
    def point(self, x):
        # affiche un gros point.
        t=arange(0, 2*pi, 0.01)
        x1, y1, R = self.canvas.Xcoo2pix(x), self.canvas.Ycoo2pix(self.y(x)), self.canvas.taille["o"]
        self.canvas.fill(self.canvas.Xpix2coo(x1 + R*cos(t)), self.canvas.Ypix2coo(y1 + R*sin(t)), self.couleur)
        # self.canvas.axes.plot([x], [self.y(x)], self.couleur+"o", markersize = 2*self.canvas.taille["o"]) -> a faire marcher ?
    









    def dessine_courbe(self, equation, intervalle):
        self.eq_actuelle = equation    # equation en cours de traitement
        
        if not intervalle: intervalle = "R"
        intervalle = self.conversion(intervalle, True).replace(",", ";") # le separateur peut etre ";" ou ",".
        a, b = self.canvas.fen()[:2] # x_min, x_max
        intervalle = intervalle.replace("-{", "\\{").replace("R+*", "]0;+i[").replace("R-*", "]-i;0[").replace("R*+", "]0;+i[").replace("R*-", "]-i;0[").replace("R+", "[0;+i[").replace("R-", "]-i;0]").replace("R*", "]-i;0[U]0;+i[").replace("R", "]-i;+i[").replace("inf", "i").replace("oo", "i") # R -> ensemble des reels, R+ -> ensemble des reels positifs, etc.
        # print "coucou", intervalle
        
        # gestion de R-{0}, ]-2;5] - {1;3}, etc...
        val_interdites = ()
        if "\\{" in intervalle:
            intervalle, val_interdites = intervalle.split("\\{")
            # print "decoupe", intervalle, val_interdites
            val_interdites = securite.eval_safe("("+ val_interdites.replace("}", "").replace(";", ",") + ",)")
            # dans l'exemple ci-dessus, val_interdites = (1,3).
                    
        
        # gestion de la syntaxe [5;6]U{5}
        intervalle = regexp("[{][-]?[.0-9]+[}]", intervalle, "'[' + x[1:-1] + ';' + x[1:-1] + ']'")
        
        intervalles = intervalle.split("U")
        nouveaux_intervalles = []
        eq = self.conversion(equation)
        
        
        for inter in intervalles:
            # par defaut, les intervalles sont de type indetermine.
            if not (inter[0] in "[]?"):    inter = "?" + inter
            if not (inter[-1] in "[]?"):   inter = inter + "?"
            
            # on remplace l'infini par des valeurs.
            inter = mreplace(inter, ("];", "[;", "?;", "]-i;", "[-i;"), "?" + str(2*a - b) + ";")
            inter = mreplace(inter, (";[", ";]", ";?", ";+i[", ";+i]"), ";" + str(2*b - a) + "?")
            # On simule l'infini de la sorte :
            #               |------//-------|-------//------|-------//------|
            #        -infini=2*a-b          a               b          2*b-a=+infini


            # on recupere les extremites de l'intervalle.
            a, b = [float(securite.eval_safe(x)) for x in inter[1:-1].split(";")]


            deb, fin = inter.split(";")
            intervalle = deb + ";"
            for c_i in val_interdites:
                if a < c_i < b:
                    intervalle += str(c_i) + "[U]" + str(c_i) + ";"
                elif a == c_i:
                    intervalle = "]" + intervalle[1:]
                elif b == c_i:
                    fin = fin[:-1] + "["
                
            intervalle += fin
            
            nouveaux_intervalles += intervalle.split("U")


        for inter in nouveaux_intervalles:
            debug("intervalle :", inter)
            a, b = [float(securite.eval_safe(x)) for x in inter[1:-1].split(";")]
            # print a
            
            if a == b and (inter[0] == "]" or inter[-1] == "["):
                continue # intervalle de la forme ]a;a[, [a;a[ ou ]a;a] -> rien a afficher !
            
            
            if inter[0]=="[":
                self.point(a)    # affiche "o--"
                
            elif inter[0]=="]":
                try:
                    self.y(a)**1 # sale hack qui risque de ne pas marcher avec numpy ou numarray (!!)
                    # le probleme, c'est que par exemple 1/array(0.) ne renvoit pas d'erreur, mais
                    # une valeur 1.#INF. Suivant les fonctions, on a ainsi soit une erreur, soit une valeur infinie.
                    self.arc(a)    # affiche ")--"
                except (OverflowError, ZeroDivisionError):
                    # Une fonction definie sur un intervalle ouvert peut exploser aux bornes.
                    debug("Correction de la borne inf.")
                    a += .1*self.canvas.pas()
                    
            if inter[-1]=="]":
                self.point(b)     # affiche "--o"
                
            elif inter[-1]=="[":
                try:
                    self.y(b)**1
                    self.arc(b, -1)     # affiche "--("
                except (OverflowError, ZeroDivisionError):
                    # Une fonction definie sur un intervalle ouvert peut exploser aux bornes.
                    debug("Correction de la borne sup.")
                    b -= .1*self.canvas.pas() 
                
            
            
            # La boucle qui suit corrige un bug genant de matplotlib quand des courbes sortent largement de la fenetre d'affichage ; par exemple, y = x**7 sur [-8;8]. Des artefacts apparaissent alors sur le graphique.
            # Pour contourner le probleme, on filtre les points qui vont sortir de la fenetre.
            
            # Pour qu'un point puisse etre supprime, il faut les deux conditions suivantes reunies :
            # 1. qu'il depasse de la fenetre
            # 2. que le point qui le precede immediatement, et le point qui le suit immediatement, sortent aussi de la fenetre.
            # Si on ne prend pas cette precaution, on risque de relier 2 points de maniere incorrecte (par exemple, dans le cas d'une fonction en cloche, style f(x) = -x**2 + 30 : si elle depasse, les 2 derniers points affiches de part et d'autre de la partie qui depasse seront relies, a tort !) 
            
            # A REECRIRE sans changer les types (trop long)
            # list(arange(..))+[0] -> concatenate(arange(..)), [0])
            
            ymin, ymax = self.canvas.fen(2), self.canvas.fen(3)
            x = list(arange(a, b, self.canvas.pas()))
            if x:
                if x[-1] <> b:
                    x.append(b)
                    debug("Borne superieure ajoutee.")
                xx = array([a] + x + [b])
                yy = self.y(xx)
                
                x = []
                y = []
                old_old_yi = old_yi = old_xi = None
                for i in range(len(xx)):
                    xi = xx[i]
                    yi = yy[i]
                    if old_old_yi <> None:
                        if (old_old_yi > ymin or old_yi > ymin or yi > ymin) and (old_old_yi < ymax or old_yi < ymax or yi < ymax):
                            x.append(old_xi)
                            y.append(old_yi)
                    old_old_yi = old_yi
                    old_yi = yi
                    old_xi = xi    
                            
                if len(y) > 1 and (y[-1] < ymin or y[-1] > ymax) and (y[-2] < ymin or y[-2] > ymax):
                    debug("Avant l'affichage, suppression du dernier point.")
                    y = y[:-1]
                    x = x[:-1]
    
    
                #print x
                #print y
                #print self.couleur
                # print y[0]
                self.canvas.plot(x, y, self.couleur)
                self.message("Courbe tracee.")
            else:
                self.message("Courbe tracee.")
                debug("Aucun point a tracer.")
            

