# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

######################################
#
#    FICHIER DE CONFIGURATION
#
######################################
#
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
######################################

# Note :
# eviter d'utiliser des listes au maximum, preferer les tuples.
# en effet, il est souvent utile de copier les parametres.
# Mais par defaut, une liste n'est pas reellement copiee (il faut faire l2 = l1[:]).
# Si de plus, on utilise des listes de listes, on a vite fait d'obtenir des bugs etranges...



from os import getcwd as repertoire_actuel
from wx import Platform as plateforme
from sys import version as python_version
python_version = float(python_version[:3])

# Numero de version
version = "0.106"

# Si le parametre suivant a la valeur True, le mode securite est enclenche et les macros ne sont executees qu'apres confirmation.
# De plus, les erreurs n'apparaissent plus, et le bouton recharger les librairies est masque.
# Ce parametre devrait valoir True par defaut dans les versions ulterieures.
# Pour l'instant, en attendant la gestion des macros, il vaut mieux le laisser sur False.

securite = False

# debuguage
debug = True

# le programme tourne-t-il en version "executable" ?
py2exe = False


# Modules a importer
# ----------------

modules = ("geometre", "traceur", "statistiques", "calculatrice", "scicalc")

# Multi-threading pour les macros
# -------------------------------

multi_threading = False

# c'est assez instable...

if multi_threading == None:
    if plateforme in ("__WXMSW__", "__WXMAC__"): # ca ne marche pas avec le serveur X (sous Linux par ex.)
        multi_threading = True


# Repertoires par defaut
# ----------------------


repertoire = repertoire_actuel() # memorise le repertoire de lancement
rep_save = repertoire_actuel() # repertoire ou on sauve les fichiers
rep_open = repertoire_actuel() # repertoire ou on ouvre les fichiers
rep_export = repertoire_actuel() # repertoire ou on exporte les fichiers



# Parametres generales
# --------------------

dimensions_fenetre = (750,510)
confirmer_quitter = True
nbr_annulations = 50
historique_log = True # cree un fichier .log (conseille)
historique_log_continu = False # modifier ce fichier en temps reel (peu utile)
emplacement_log = "log" # dans log/ par defaut
enregistrer_messages = True # enregistre les messages (notamment d'erreur) dans messages.log

tolerance = 1e-13 # seuil de tolrance, utilis en particulier par geolib pour savoir si 2 points sont confondus
# ce paramtre permet un compromis acceptable entre les faux negatifs
# (2 points confondus considrs  tort comme distincts, du fait d'imprcisions de calculs qui se cumulent)
# et les faux positifs (deux points proches considers  tort comme confondus).

# Parametres d'affichage
# ----------------------

orthonorme = False

#constantes
RIEN = 0
NOM = 1
LABEL = 2

styles_de_lignes = ['-', '--', '-.', ':', 'steps', 'None']
styles_de_polygones = ['-', '--', '-.', ':', 'None']
styles_de_points = ['+', 'x', ',', '.', '1', '2', '3', '4', '<', '>', '^', 'v', 'D', 'H', '_', '|', 'd', 'h', 'o', 'p', 's']
styles_de_textes = ["normal", "italic", "oblique"]
familles_de_textes = ["sans-serif", "serif", "cursive", "fantasy", "monospace"]


defaut_objets = {
    "legende": RIEN, 
    "visible": True,
    "label": "",
    }
points = {
    "couleur": "b", 
    "epaisseur": 1., 
    "style": "+", 
    "categorie": "points", 
    "taille": 8, 
    "visible": True, 
    "fixe": False, 
    "legende": NOM,
    "niveau": 5,
    }
points_deplacables = {
    "couleur": "r", 
    "niveau": 10,
    }
segments = {
    "couleur": "g",
    "epaisseur": 1., 
    "style": "-", 
    "visible": True, 
    "niveau": 3,
    "categorie": "lignes",
    }
droites = {"couleur": "b", 
    "epaisseur": 1., 
    "style": "-", 
    "visible": True, 
    "niveau": 2,
    "categorie": "lignes",
    }
vecteurs = {
    "couleur": "g", 
    "epaisseur": 1., 
    "style": "-", 
    "taille": 10, 
    "visible": True, 
    "niveau": 4,
    "categorie": "lignes",
    }
cercles = {
    "couleur": "b", 
    "epaisseur": 1., 
    "style": "-", 
    "visible": True, 
    "niveau": 1,
    "categorie": "lignes",
    }
polygones = {
    "couleur": "y", 
    "epaisseur": 1., 
    "style": "-", 
    "visible": True, 
    "alpha": .2, 
    "niveau": 0,
    "categorie": "polygones",
    }
textes = {
    "couleur": "k", 
    "epaisseur": 4, # 1 -> 9
    #"largeur": ("normal", "narrow", "condensed", "wide")[0],  # mal gere par matploltib (version 0.87)
    "taille": 12., 
    "style": "normal",
    "famille": familles_de_textes[0],
    "visible": True,
    "angle": 0, 
    "legende": LABEL,
    "fixe": False, 
    "categorie": "textes",
    "niveau": 6,
    }

# le parametre niveau est utilis pour dtecter l'objet sur la feuille :
# en cas de conflit entre deux objets proches, il permet de savoir l'objet  slectionner.
# les petits objets (points, ...) doivent tre au dessus des plus gros, et avoir un niveau suprieur.
# Rq: les objets modifiables devraient etre "au dessus" des autres, indpendamment de leur taille.
# Attention, ce parametre n'influe pas sur l'affichage des objets (pour cela, c'est le parametre "zorder" de matplotlib)

# le parametre "categorie" indique quelle liste de styles est  appliquer.
# par dfaut, on applique souvent 'styles_de_lignes'.

chiffres_significatifs = 7 # intervient dans l'affichage des proprits des objets essentiellement.

# Options de style pour les objets geometriques :
#   couleur:      str
#   epaisseur:    float
#   taille:       float
#   style:        str
#   visible:      bool
#   fixe:         bool 
#   label:        str
#   extra:       dict
# 'extra' est un dictionnaire d'options de matplotlib
# attention, ces options seront appliquees a toutes les composantes (plot, fill, text) de la representation graphique


# A REECRIRE :

# taille de differents elements graphiques

taille = {"o" : 3, "(" : 5, ">" : 10, "+" : 4}

style_point = "+" # ou "x", "." ou "o"
# Autres possibilites : ',', '1', '2', '3', '4', '<', '>', 'D', 'H', '^', '_', 'd', 'h', 'p', 's', 'v', '|' (cf. doc matplotlib)
largeur_point = (1., 0.6)  # largeur pour l'affichage, et largeur pour l'exportation en image ou l'impression
style_ligne = '-' # ou '--', '-.', ':', 'steps' ou 'None'
largeur_ligne = 1.

liste_axes = (0, 1)
# 0 -> axe des abscisses ;  1 -> axe des ordonnees

affiche_axes = True
affiche_fleches = True # fleches au bout des axes.

# Pour se reperer, deux possibilites :
# 1. Un repere proprement dit
repere = ("O", "I", "J") # en majuscule, ce sont des points ; en minuscule, des vecteurs.
# 2. Deux axes gradues
origine_axes = (0, 0) 
# Pour choisir le mode :
utiliser_repere = True

gradu = (1, 1) # espacement entre deux graduations pour chaque axe. (0 => pas de graduations).
saturation =  0.3 # valeur de saturation pour le coefficient pas()/gradu
# au dela du coefficient de saturation, les graduations ne sont plus affichees
# cela evite de bloquer le programme...

affiche_quadrillage = True # affiche un quadrillage a la place des graduations
quadrillages = (((None, None), ":", .5, "k"),)
# Chaque couple correspond a un quadrillage.
# Le format est : ([espacement horizontal, espacement vertical], style, epaisseur, couleur).
# On peut omettre des elements :
# quadrillages = ((),)
# Si un espacement vaut None, la valeur de gradu correspondante sera utilisee.
# Valeurs possibles pour le style : (cf http://matplotlib.sourceforge.net/matplotlib.pylab.html#-plot)
# "--" tirets
# "-" lignes continues
# ":" points
# "-." alternance de points et de tirets
# On peut superposer plusieurs quadrillages :
# quadrillages = (((1, 1), "-", 0.25, "k"), ((0.25, 0.25), ":", .25, "k"),)
# ou encore :
# quadrillages = (((1, 1), ":", 1, "k"), ((0.2, 0.2), ":", .125, "k"),)


resolution = 1000 # resolution utilisee pour le tracage des courbes (plus la valeur est importante, plus la courbe est lisse)
fenetre = (origine_axes[0]-8, origine_axes[0]+8, origine_axes[1]-5, origine_axes[1]+5)   # xmin, xmax, ymin, ymax

zoom_in = 1.1 # coefficient d'agrandissement des objets lors d'un zoom.

zoom_out = 0.9 # coefficient de reduction des objets lors d'un zoom.

dpi_export = 200 # resolution utilisee lors de l'exportation des images

affiche_coordonnees = True # affiche en permanence les coordonnees

precision_selection = 10 # plus la precision est importante, plus il faut etre pres d'un objet pour pouvoir le saisir
# Si le chiffre est trop petit, ca devient fastidieux de selectionner un objet
# A l'inverse, s'il est trop grand, il devient difficile de selectionner celui qu'on veut entre deux objets proches



# Paramtres lis  certains modules spcifiques
# ----------------------------------------------

# Geometrie dynamique :

afficher_boutons = True  # Possibilite de ne pas afficher les boutons, pour obliger a une construction pas a pas.


# Calculatrice :

calcul_exact = True    # gerer les fractions et les racines de maniere exacte si possible.
ecriture_scientifique = False # afficher les resultats en ecriture scientifique.
changer_separateurs = False # appliquer les separateurs personnalises
separateurs_personnels = (",", ";") # separateurs personnalises (separateur decimal, separateur de listes)
# d'autres choix sont possibles, mais pas forcement heureux...
copie_automatique = False # copie automatique de chaque resultat dans le presse-papier


# Traceur :

nombre_courbes = 9