# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##########################################################################
#
#       Fonctions couramment utilisees, et non implementees en Python
#
##########################################################################
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



import param # parametres du programme
import re   
from weakref import WeakValueDictionary
import sys
    
def iff(condition, value1, value2 = ""):
    """Renvoit "value1" si "condition" est verifie, "value2" sinon (par defaut, '')."""
    if condition: return value1
    return value2
    

def mreplace(main_string, list_of_strings, new_string = ""):
    """Remplace, dans "main_string", toutes les sous-chaines de "list_of_strings", par la chaine "new_string"."""
    for old_string in list_of_strings:
        main_string = main_string.replace(old_string, new_string)
    return main_string
    

def mfind(chaine, car):
    n = 0
    l = []
    while True:
        n = chaine.find(car, n)
        if n == -1: break
        l.append(n)
        n += 1
    return l
    
    

def msplit(main_string, list_of_separators):
    """Decoupe la chaine "main_string", selon les separateurs definis dans "list_of_separators"."""
    return mreplace(main_string, list_of_separators[1:], list_of_separators[0]).split(list_of_separators[0])


def no_twin(liste):
    """Elimine les doublons dans une liste.
    Si tous les elements de la liste sont 'hashables', mieux vaut utiliser la fonction set."""
    dico = {}
    for elt in liste:dico[id(elt)] = elt
    return dico.values()
    


def advanced_split_old(main_string, separator):
    """Decoupe la chaine "main_string" de maniere intelligente, 
    en ignorant les separateurs compris dans un groupe entre parentheses, crochets ou accolades."""
    lieux = mfind(main_string, separator)
    liste = []
    old = 0
    for lieu in lieux:  # on ne conserve que les "bonnes" separations.
        ecarts = 0
        parentheses = "({[", ")}]"
        for i in range(3):
            ecarts += abs(main_string.count(parentheses[0][i], 0, lieu)-main_string.count(parentheses[1][i], 0, lieu))
        if not ecarts:
            liste.append(main_string[old:lieu])
            old = lieu + 1
    return liste + [main_string[old:]]


    
def advanced_split(main_string, separator, keep_empty_str = False, symbols = "([{}])"):
    """Decoupe la chaine "main_string" de maniere intelligente, 
    en ignorant les separateurs compris dans un groupe entre parentheses, crochets, accolades, guillemets.
    Attention, separateur ne peut donc pas etre une parenthese, un crochet, une accolade ou un guillemet !
    Par defaut, supprime egalement les chaines vides."""
    in_string = False # est-on dans une chaine ?
    in_string_sep = "'" # caractere encadrant la chaine (" ou ')
    parentheses = 0 # tient le compte des parentheses ouvertes non fermees
    crochets = 0 # idem pour les crochets
    accolades = 0 # idem
    coupures = [-1] # endroits ou il faudra couper la chaine
    for i in xrange(len(main_string)):
        a = main_string[i]
        if a in ("'", '"'):
            if in_string:
                if in_string_sep == a: # attention, il y a 2 indicateurs de chaine (" et ')
                    in_string = False
            else:
                in_string = True
                in_string_sep = a
        elif a in symbols:
            if a == "(" and not in_string:
                parentheses += 1
            elif a == ")" and not in_string:
                parentheses -= 1
            elif a == "[" and not in_string:
                crochets += 1
            elif a == "]" and not in_string:
                crochets -= 1
            elif a == "{" and not in_string:
                accolades += 1
            elif a == "}" and not in_string:
                accolades -= 1
        elif a == separator and not (in_string or parentheses or crochets or accolades) :
            coupures.append(i)
    coupures.append(None)
    # chaine[i:None] retourne la fin de chaine
    
    return [main_string[i+1:j] for i, j in zip(coupures[:-1], coupures[1:]) if main_string[i+1:j] or keep_empty_str]




def regexp(regular_exp, main_string, action = "''"):
    """Transforme la chaine "main_string" :
    Il applique aux parties verifiant "regular_exp" le traitement "action".
    Le traitement "action" doit avoir pour variable "x",
    x correspondant a un bout de chaine verifiant "regular_exp".
    Par exemple, action peut etre "x[:-1] + '*' + x[-1]"."""
    l = re.split("(" + regular_exp + ")", main_string)
    exec("def f(x): return " + (action or "''"))
    for i in range(len(l)//2):
        l[2*i+1] = f(l[2*i+1])
    return "".join(l)
    
    
    
    
class WeakList(WeakValueDictionary):
    """Une 'liste' de references faibles.
    Le terme 'liste' est trompeur, la syntaxe des listes de python n'est pas implementee,
    exceptee les methodes append(), et remove().
    En outre, le contenu s'obtient par la methode values().
    Note: l'implementation de remove est un peu differente.
    remove(valeur) supprime toutes les occurences de valeur trouvees, et ne renvoit pas d'erreur.
    Par ailleurs, remove utilise le comparateur 'is', et non "==", ce qui fait que remove([]) ne fera rien, par exemple.
    (Il faut qu'il s'agisse du meme objet, et non d'un objet egal).
    Sinon, il faut utiliser compare_and_remove."""
    
    def __init__(self):
        WeakValueDictionary.__init__(self)
        
    def append(self, valeur):
        self[max(self.keys() or [0]) + 1] = valeur
        
    def remove(self, valeur):
        for key, value in self.items():
            if value is valeur: del self[key] # il faut qu'il s'agisse du mme objet
            
    def compare_and_remove(self, valeur):
        for key, value in self.items():
            if value == valeur: del self[key] # un objet gal suffit

    def __str__(self):
        return str(self.values()) + " (WeakList)"
            
            
def print_error():
    "Afiche l'erreur sans interrompre le programme."
    sys.excepthook(*sys.exc_info())
    print "Warning: this error was not raised."