# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##########################################################################
#
#                       Gestion de la securite
#
##########################################################################
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



from fonctions import *
import sets # pour la compatibilite avec python 2.3
import types
import math # pour avoir droit aux fonctions mathematiques dans eval_restricted par defaut.
import random # idem

def __avertissement__(*arg, **kw): print "Instruction interdite en mode securise."
#fonctions_interdites = ["eval", "compile", "execfile", "file", "open", "write", "getattr", "setattr"]

__liste_blanche__ = sets.Set(('False', 'None', 'True', 'abs', 'basestring', 'bool',  'callable', 'chr', 'close', 'cmp', 'coerce', 'complex', 'dict', 'divmod', 'enumerate', 'filter', 'float', 'frozenset', 'hash', 'hex', 'id', 'int', 'isinstance', 'issubclass', 'iter', 'len', 'list', 'long', 'map', 'max', 'min', 'object', 'oct', 'ord', 'pow', 'range', 'reduce', 'repr', 'reversed', 'round', 'set', 'slice', 'sorted', 'str', 'sum', 'tuple', 'type', 'unichr', 'unicode', 'xrange', 'zip'))

__liste_noire__ = sets.Set(__builtins__.keys())
__liste_noire__.difference_update(__liste_blanche__)

dictionnaire_builtins = {}.fromkeys(list(__liste_noire__), __avertissement__)
dictionnaire_modules = {}.fromkeys([key for key, objet in globals().items() if type(objet) == types.ModuleType])

ajouts_math = {}
for key, obj in math.__dict__.items():
    if type(obj) == types.BuiltinFunctionType:
        ajouts_math[key] = obj

for key, obj in random.__dict__.items():
    if type(obj) == types.BuiltinFunctionType:
        ajouts_math[key] = obj

# Exemple d'usage :
# dico = {"pi":3.1415926535897931, "e":2.7182818284590451}    # dictionnaire local
# dico.update(securite.dictionnaire_builtins)
# exec(string, dico)

def eval_restricted(s, dico_perso = None):
    """eval_restricted(s) evalue s dans un contexte vierge et securise.
    
    Toutes les fonctions disponibles par defaut sont filtrees.
    
    Note: eval_restricted est le plus securise possible, mais contrairement
     eval_safe, il utilise la fonction eval ; il peut donc y avoir des failles
    de scurit. Merci de m'en informer.
    """
    dico = {"rand": random.random}
    dico.update(dictionnaire_builtins) # supprime certaines fonctions par defaut (en les redefinissant)
    dico.update(ajouts_math) # ajoute les fonctions mathematiques usuelles
    if dico_perso is not None:
        dico.update(dico_perso)
    for module in ("os", "sys", "securite"):
        dico.pop(module, None)
    return eval(s, dico)
    

def eval_safe(s):
    """eval_safe(repr(x)) retourne x pour les types les plus usuels
    (int, long, str, unicode, float, bool, None, list, tuple, dict.)
    Mais aucune evaluation n'est faite, ce qui evite d'executer un code dangereux.
    Le type de s est detecte, et la transformation appropriee appliquee.
    
    NB1: eval_safe n'est pas destine a remplacer eval :
    - il est relativement lent,
    - le nombre de types supportes est volontairement reduit,
    - il n'est pas souple (eval_safe("2-3") renvoit une erreur).
    La fonction eval_safe est orientee uniquement securite.
    
    NB2: eval_safe est recursif (il peut lire des listes de tuples de ...).
    
    NB3: eval_safe est parfaitement securise, car il ne fait jamais appel a une instruction eval.
    Contrairement a la fonction eval_restricted, qui est 'probablement' securisee."""

    s = s.strip()
    
    def filtre(chaine, caracteres):
        return re.match("^[%s]*$" %caracteres, chaine) <> None
    
        
    
    if filtre(s, "-0-9") or s[-1] == "L":   # entier
        return int(s)
    
    if filtre(s, "-+.e0-9"):    # flottant
        return float(s)
        
    if len(s) > 1 and s[0] == s[-1] and s[0] in ("'", '"'):    # chaine
        return s[1:-1]
        
    if len(s) > 2 and s[0] == "u" and s[1] == s[-1] and s[1] in ("'", '"'):
        return unicode(s[2:-1])
    
    if s == "None":
        return None
        
    if s == "True":
        return True     # Eviter return eval(s), car True peut etre redefini. Exemple : True = False (!)
        
    if s == "False":
        return False

    if s[0] in ("(", "["):  # tuple ou liste
        liste = [eval_safe(t) for t in advanced_split(s[1:-1], ",") if t]
        if s[0] == "(":
            liste = tuple(liste)
        return liste
        
    if s[0] == "{":     # dictionnaire
        dict = {}
        liste = [advanced_split(t, ":") for t in advanced_split(s[1:-1], ",") if t]
        for key, val in liste:
            dict[eval_safe(key)] = eval_safe(val)
        return dict
                
        
    raise TypeError, "types int, str, float, bool, ou None requis (ou liste ou tuple de ces types)"