# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##########################################
#            CONSOLE                      
##########################################
#    WxGeometrie
#    Dynamic geometrie, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


# NOTES :
# Un fichier .geo est essentiellement un fichier python (structure par du XML).
# C'est potentiellement dangereux (en particulier sous Windows), car certains modules python (comme os, sys, ...), et certaines fonctions (write...) peuvent endommager le systeme d'exploitation.
# Une bonne partie du travail de ce module consiste donc a securiser l'environnement d'execution.
# Comme je suis loin d'etre un expert, il est possible (voire probable) que des failles subsistent neanmoins.
# Merci de m'en informer.


    
#  Il faudra donner a l'utilisateur la possibilite d'autoriser la macro ponctuellement, ou meme de changer le parametre de securite


from LIB import *

class Commande:
    def __init__(self, parent):
        self.parent = parent
        self.securite = param.securite
        self.historique = self.parent.historique # historique des commandes
        # Liste des commandes d'affichage (gerees par le canevas).
        self.commandes_canvas = ["zoomer(", "set_fenetre(", "fen(", "orthonormer(", "zoom_auto("]
        # Liste des commandes de manipulation de feuilles (gere par le module actif)
        self.commandes_panel = ["creer_feuille(", "charger_feuille(", "rafraichir_feuille(", "fermer_feuille("]
        #self.macro = ":"
        
        self.compteur = 0 # s'incrmente  chaque action.
        # cela permet aux autres parties du programme de savoir si quelque chose s'est pass.

        # sert essentiellement au debogage :
        self.afficher = "&"
        # ':&"bonjour"' est un equivalent de ':print "bonjour"' par exemple.
        
        
        
    def executer(self, commande, abreviations = True):
        self.commande = commande_originale = commande.strip() # enleve les espaces, les retours a la ligne, etc...

        if self.commande:
            self.parent.canvas.regenerer_liste = True
                        
            macro = (self.commande[0] ==":")

            if macro and self.securite:
                print "Les parametres de securite n'autorisent pas l'execution de macros"
                return


            if macro:  self.commande = self.commande[1:]
            
            if self.commande[0] == ":" and param.debug:
                exec(self.commande[1:]) # execution brute ("de bas niveau") pour le deboguage
                return



            afficher = self.debute_par(self.afficher)
            if afficher:  self.commande = self.commande[1:]
            
            
            
            
            self.transformer()
            # transforme (A B) en Droite(A,B), [A B] en Segment(A,B), et |A B| en Variable(Segment(A,B)) (longueur de [AB])


            
            archiver = True # archiver l'etat actuel de la feuille
            
            if macro:               
                # Les racourcis suivants sont utilisables dans une macro :
                # ! pour self.parent.
                # !! pour self.parent.canvas.
                # !n pour self.parent.parent.onglet(n).
                # !n! pour self.parent.parent.onglet(n).canvas.
                # !: pour self.parent.actuelle.
                # !:: pour self.parent.actuelle.objets.
                if abreviations:
                    self.commande = self.commande.replace("!::", "self.parent.actuelle.objets.")
                    self.commande = self.commande.replace("!:", "self.parent.actuelle.")
                    liste = re.split("(![!0-9]*)", self.commande)
                    self.commande = ""
                    for i in range(len(liste)):
                        if i%2:
                            self.commande += "self.parent."
                            if liste[i] == "!!":
                                self.commande += "canvas."
                            elif liste[i] <> "!":
                                a, b, c = re.split("([0-9]+)", liste[i])
                                self.commande += "parent.onglet(" + b + ")."
                                if c == "!":
                                    self.commande += "canvas."
                        else:
                            self.commande += liste[i]
                        
            else:
                # NOTE CONCERNANT LA SECURITE :
                # Le format d'encodage de sauvegarde est du code Python (!).
                # Les lignes commencant par ":" sont des macros.
                # Les autres lignes subissent un traitement pour les rendre inoffensives.
                #
                # Une instruction standard ne doit pas avoir acces a n'importe quoi,
                # et en particulier pas a la feuille elle-meme, mais seulement aux objets de la feuille.
                #
                # Toutes les fonctions sensibles susceptibles d'etre accessibles comportent un "__".
                # On remplace donc les occurences successives de "_" par un seul "_".
                # La concatenation pourrait eventuellement poser un probleme,
                # d'ou la necessite d'empecher l'execution d'eval et d'exec.
                #
                # L'utilisation de __self__. permet de passer par la methode __setattr__() de feuille.objets.
                # Or, cette methode a ete modifiee, il est donc important de passer par elle,
                # et de ne pas se contenter d'une affectation ordinaire (A=Point(1,2))
                # 
                # Il est important d'empecher l'execution de la fonction eval(),
                # qui permet de contourner certaines limitations (voir plus bas).
                #
                # L'instruction exec() ne peut pas etre desactivee aussi facilement,
                # il faut s'arranger pour qu'elle provoque une erreur de syntaxe.
                # Exemple de code problematique : exec("print 3;exec ('K = __'+'feuille__')")
                # On remplace donc ";" par ","
                
                self.commande = self.commande.replace(";", ",")
                self.commande = regexp("[_]+", self.commande, "'_'")
                
                if self.parent.actuelle:
                    self.parent.canvas.editeur.init() # evite d'enregistrer l'etat avec un nom d'objet en cours d'edition.

                if self.debute_par(self.commandes_canvas):
                    self.prefixe("__canvas__.")
                    dictionnaire = {"__canvas__" : self.parent.canvas}
                elif self.debute_par(self.commandes_panel):
                    archiver = False # ne pas archiver l'etat actuel de la feuille (commandes non annulables)
                    self.prefixe("__parent__.")
                    dictionnaire = {"__parent__" : self.parent}
                    # Pour ces commandes, il n'y a pas forcement de feuille actuelle.
                    # Il faut donc faire un dictionnaire personnalise.
                    # Attention a bien mettre __self__ et non self (pour ne pas qu'on puisse y acceder.
                    # Voir la note plus haut a ce sujet ("__" est remplace par "_")
                else:
                    dictionnaire = self.parent.actuelle.objets.__dict__
                    if self.debute_par(["del "]):
                        self.commande = self.commande[4:]
                        self.commande = "__self__." + self.commande + ".supprimer()"
                        # Il aurait ete plus agreable de definir une fonction __del__ pour la classe objet.
                        # Mais cette fonction est appelee lorsque la derniere reference est detruite, et en pratique,
                        # c'est tres aleatoire...
                    elif re.match("[_A-Za-z]", self.commande):
                        self.prefixe("__self__.")
                
            
            if afficher: self.prefixe("print ")
            

            if param.debug: print "REQUETE:" + self.commande
            
            if macro:
                feuille = self.parent.actuelle # alias pratiques
                if feuille is not None:
                    objets = feuille.objets
                try:
                    exec(self.commande)
                    self.historique.enregistrer(commande_originale, archiver)
                    if self.parent.actuelle:
                        self.parent.canvas.detecter() # redetection des objets a proximite du pointeur
                except:
                    self.historique.enregistrer("#" + commande_originale, archiver)
                    #if param.debug: raise
                    raise
                return
                                    
            
            def avertissement(*arg, **kw): print "Instruction interdite en mode securise."
            fonctions_interdites = ["eval", "compile", "execfile", "open", "write", "getattr", "setattr"]
            securite = {}
            for fonction in fonctions_interdites:
                securite[fonction] = avertissement
            try:
                exec(self.commande, securite, dictionnaire)
                self.historique.enregistrer(commande_originale, archiver)
                if self.parent.actuelle:
                    self.parent.canvas.detecter() # redetection des objets a proximite du pointeur
            except:
                self.historique.enregistrer("#" + commande_originale, archiver)
                #if param.debug: raise
                raise
                

                            




    def debute_par(self, liste):
        return filter(lambda x: self.commande.startswith(x), liste)

    def prefixe(self, texte):
        self.commande = texte + self.commande


    def transformer(self):
        # transforme (A B) en Droite(A,B), [A B] en Segment(A,B), et |A B| en Variable(Segment(A,B)) (longueur de [AB])
        liste = re.split("([[(|][ ]*[A-Za-z_][A-Za-z0-9_]*[ ]+[A-Za-z_][A-Za-z0-9_]*[ ]*[])|])", self.commande)
        texte = ""
        for i in range(len(liste)):
            s = liste[i]
            if i%2:
                # morceau de chaine a traiter ("(A B)", "[A B]" ou "|A B|")
                ajouts = {"(" : ("Droite(", ")"), "[" : ("Segment(", ")"), "|" : ("Variable(Segment(", "))")}[s[0]]
                s = ajouts[0] + s[1:-1].strip().replace(" ", ",", 1) + ajouts[1]
            texte += s
        self.commande = texte

        # transforme (2 3), ou (2;3) en Point(2,3)
        liste = re.split("([(][ ]*[-]?[0-9.]+[ ]*[; ][ ]*[-]?[0-9.]+[ ]*[)])", self.commande)
        texte = ""
        for i in range(len(liste)):
            s = liste[i]
            if i%2:
                # morceau de chaine a traiter ("(2 3)", ou "(2;3)")
                s = "Point(" + s[1:-1].strip().replace(";", " ").replace(" ", ",", 1) + ")"
            texte += s
        self.commande = texte
        
        # Les expressions mathematiques style "$\alpha > \beta$" de matplotlib doivent etre rentrees comme raw_string,
        # sinon, le parser de matplotlib plante.
        self.commande = regexp("""[^r]["][$][^$"]+[$]["]""", " " + self.commande, "x[0] + 'r' + x[1:].replace('\\\\\\\\','\\\\')")[1:] # avec ""
        self.commande = regexp("""[^r]['][$][^$"]+[$][']""", " " + self.commande, "x[0] + 'r' + x[1:].replace('\\\\\\\\','\\\\')")[1:] # avec ''
        
        